@@ -33,8 +33,9 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "gdis.h"
 #include "coords.h"
 #include "interface.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "matrix.h"
+#include "model.h"
 #include "spatial.h"
 #include "numeric.h"
 #include "morph.h"
@@ -658,7 +659,7 @@ for (list=locality ; list ; list=g_slist_next(list))
 /* ie can happen when the lattice matrix has large non-diagonal elements */
   ARR3SET(x2, core2->x);
   ARR3SUB(x2, x1);
-  fractional_minsq(x2, model->periodic);
+  fractional_min(x2, model->periodic);
 
 #if DEBUG_ATOM_BONDS
 printf(" [%s ?< %f]\n", core2->atom_label, r2cut);
@@ -759,7 +760,7 @@ for (list1=data->ubonds ; list1 ; list1=g_slist_next(list1))
     core2 = bond2->atom2;
     ARR3SET(x, core2->x);
     ARR3SUB(x, core1->x);
-    fractional_minsq(x, data->periodic);
+    fractional_min(x, data->periodic);
     VEC3MUL(x, 0.5);
     ARR3SET(bond2->offset, x);
 
@@ -942,45 +943,79 @@ while (list)
 data->bonds=NULL;
 }
 
+/******************************/
+/* redo bond midpoint offsets */
+/******************************/
+void connect_midpoints(struct model_pak *model)
+{
+gdouble x[3];
+GSList *list;
+struct core_pak *core1, *core2;
+struct bond_pak *bond;
+
+g_assert(model != NULL);
+
+for (list=model->bonds ; list ; list=g_slist_next(list))
+  {
+  bond = list->data;
+
+/* compute offset */
+  core1 = bond->atom1;
+  core2 = bond->atom2;
+  ARR3SET(x, core2->x);
+  ARR3SUB(x, core1->x);
+  fractional_min(x, model->periodic);
+  VEC3MUL(x, 0.5);
+  ARR3SET(bond->offset, x);
+  }
+}
+
 /*********/
 /* BONDS */
 /*********/
 #define DEBUG_CALC_BONDS 0
-void connect_bonds(struct model_pak *data)
+void connect_bonds(struct model_pak *model)
 {
 GSList *list;
 struct core_pak *core;
 
-g_assert(data->zone_array != NULL);
+g_assert(model != NULL);
 
-if (data->anim_fix)
+/* enforce a midpoint recalculation (gulp noautobonds + selection rotate fix) */
+connect_midpoints(model);
+
+if (model->anim_fix)
   return;
+if (!model->build_molecules)
+  return;
+
+g_assert(model->zone_array != NULL);
 
 /* TODO - delete polyhedra */
 /* redo atom connectivity */
-wipe_bonds(data);
+wipe_bonds(model);
 
-for (list=data->cores ; list ; list=g_slist_next(list))
+for (list=model->cores ; list ; list=g_slist_next(list))
   {
   core = list->data;
 
   core->status &= ~ZEOL_HIDDEN;
   if (!(core->status & HIDDEN))
-    connect_atom_compute(core, data);
+    connect_atom_compute(core, model);
   }
 
 /* user bond modification */
-connect_merge_user(data);
+connect_merge_user(model);
 
 /* special building modes */
-build_hbonds(data);
+build_hbonds(model);
 
-if (data->build_zeolite)
-  build_zeolite(data);
+if (model->build_zeolite)
+  build_zeolite(model);
 
 #if DEBUG_CALC_BONDS
-dump_bonds(data);
-dump_atom_bonds(data);
+dump_bonds(model);
+dump_atom_bonds(model);
 #endif
 }
 
@@ -1152,7 +1187,7 @@ for (list=data->cores ; list ; list=g_slist_next(list))
         core2->status |= PRUNED;
 
 /* always attempt to unfragment molecules */
-/* CURRENT - fix for fractional_minsq not giving cartesian minimum */
+/* CURRENT - fix for fractional_min not giving cartesian minimum */
         ARR3ADD(x, core->x);
         ARR3SUB(x, core2->x);
         ARR3ADD(core2->x, x);
