@@ -156,10 +156,10 @@ switch(type)
     calc_emp(data);
 /* repetition here, since there is interdependence */
 /* between centroid (coords_center) and latmat (coords_compute) calc */
-    coords_center(1, data);
+    coords_center(data);
     coords_compute(data);
   case CENT_COORDS:
-    coords_center(1, data);
+    coords_center(data);
   case REDO_COORDS:
     coords_compute(data);
     break;
@@ -177,7 +177,6 @@ void coords_make_cartesian(struct model_pak *model)
 GSList *list;
 struct core_pak *core;
 struct shel_pak *shel;
-struct vertex_pak *vec;
 
 g_assert(model != NULL);
 
@@ -193,12 +192,7 @@ for (list=model->shels ; list ; list=g_slist_next(list))
   shel = list->data;
   vecmat(model->latmat, shel->x);
   }
-/* morphology vertices */
-for (list=model->vertices ; list ; list=g_slist_next(list))
-  {
-  vec = list->data;
-  vecmat(model->latmat, vec->x);
-  }
+
 matrix_identity(model->latmat);
 matrix_identity(model->ilatmat);
 matrix_identity(model->rlatmat);
@@ -212,7 +206,6 @@ void coords_make_fractional(struct model_pak *model)
 GSList *list;
 struct core_pak *core;
 struct shel_pak *shel;
-struct vertex_pak *vec;
 
 g_assert(model != NULL);
 
@@ -228,12 +221,6 @@ for (list=model->shels ; list ; list=g_slist_next(list))
   shel = list->data;
   vecmat(model->ilatmat, shel->x);
   }
-/* morphology vertices */
-for (list=model->vertices ; list ; list=g_slist_next(list))
-  {
-  vec = list->data;
-  vecmat(model->ilatmat, vec->x);
-  }
 }
 
 /*************************************/
@@ -317,7 +304,7 @@ VEC3SET(data->cell[7].x, 1.0, 0.0, c);
 void shell_make_links(struct model_pak *model)
 {
 gint match;
-gdouble min, sep, vec[3];
+gdouble min, sep, x[3], r[3];
 gpointer zone;
 GSList *list1, *list2, *locality, *floating;
 struct core_pak *core;
@@ -349,14 +336,23 @@ for (list1=model->shels ; list1 ; list1=g_slist_next(list1))
     if (shel->atom_code == core->atom_code)
       {
 /* get the minimum fractional separation */
-      ARR3SET(vec, core->x);
-      ARR3SUB(vec, shel->x);
-      fractional_minsq(vec, model->periodic);
+      ARR3SET(x, shel->x);
+      ARR3SUB(x, core->x);
+      fractional_min(x, model->periodic);
 /* convert to cartesian & compare with cutoff */
-      vecmat(model->latmat, vec);
-      sep = VEC3MAGSQ(vec);
+      ARR3SET(r, x);
+      vecmat(model->latmat, r);
+      sep = VEC3MAGSQ(r);
       if (sep < min)
         {
+/* relocate shell to be close to core */
+        ARR3SET(shel->x, core->x);
+        ARR3ADD(shel->x, x);
+/*
+        ARR3SET(shel->rx, core->rx);
+        ARR3ADD(shel->rx, r);
+*/
+
 /* FIXME - what to do with previous matches? eg set (core->shell)->shell = NULL ? */
 /* reference each other */
         core->shell = shel;
@@ -403,164 +399,18 @@ printf("Floating shell=%p\n", shel);
   }
 }
 
-/*********************************/
-/* coordinate matching routine 1 */
-/*********************************/
-/* done via 3D world coordinate matching */
-#define DEBUG_SEEK_COORD3D 0
-struct core_pak *seek_coord3d(gdouble *x, struct model_pak *data)
-{
-gdouble d2, tol, vec[3], piv[3];
-GSList *list=NULL, *ilist=NULL;
-struct core_pak *core, *best=NULL;
-struct image_pak *image;
-
-/* tolerance (angstroms squared)  */
-tol = data->rmax * 20.0/sysenv.size;
-
-#if DEBUG_SEEK_COORD3D
-printf(" size = %d\n", sysenv.size);
-printf(" rmax = %f\n", data->rmax);
-printf("  tol = %f\n", tol);
-#endif
-
-do
-  {
-  if (ilist)
-    {
-/* retrieve periodic image vector */
-    image = ilist->data;
-    ARR3SET(piv, image->rx);
-    ilist = g_slist_next(ilist);
-    }
-  else
-    {
-    VEC3SET(piv, 0.0, 0.0, 0.0);
-    ilist = data->images;
-    }
-
-/* attempt to match core coords */
-  for (list=data->cores ; list ; list=g_slist_next(list))
-    {
-    core = list->data;
-
-    ARR3SET(vec, core->rx);
-    ARR3ADD(vec, piv);
-    ARR3SUB(vec, x);
-    d2 = vec[0]*vec[0] + vec[1]*vec[1];
-    if (d2 < tol)
-      {
-      best = core;
-      tol = d2;
-      }
-    }
-  }
-while (ilist);
-
-return(best);
-}
-
-/*********************************/
-/* coordinate matching routine 2 */
-/*********************************/
-/* done via 2D window coordinate matching */
-#define DEBUG_SEEK_COORD2D 0
-struct core_pak *seek_coord2d(gint x, gint y, struct model_pak *data)
-{
-gint tol, d2, w[2];
-gdouble vec[3];
-GSList *list=NULL;
-struct core_pak *core, *best=NULL;
-
-/* tolerance (pixels squared) */
-tol = 49;
-
-#if DEBUG_SEEK_COORD2D
-printf("search: [%d,%d]\n", x, y);
-#endif
-
-/* TODO - periodic images */
-for (list=data->cores ; list ; list=g_slist_next(list))
-  {
-  core = list->data;
-
-  ARR3SET(vec, core->rx);
-
-  gl_unproject(w, vec, canvas_find(data));
-
-#if DEBUG_SEEK_COORD2D
-printf("  (%d, %d)\n", w[0], w[1]);
-#endif
-
-  w[0] -= x;
-  w[1] -= y;
-
-  d2 = w[0]*w[0] + w[1]*w[1];
-  if (d2 < tol)
-    {
-    best = core;
-    tol = d2;
-    }
-  }
-return(best);
-}
-
-/******************************/
-/* vertex coordinate extremes */
-/******************************/
-void vertex_limits(gdouble *min, gdouble *max, struct model_pak *model)
-{
-gint i;
-gdouble x[3];
-GSList *list;
-struct vertex_pak *v;
-
-g_assert(model != NULL);
-
-/* failsafe limit init */
-VEC3SET(min, 0.0, 0.0, 0.0);
-VEC3SET(max, 0.0, 0.0, 0.0);
-list = model->vertices;
-if (!list)
-  return;
-
-/* failsafe limit init */
-v = list->data;
-ARR3SET(x, v->x);
-ARR3SET(min, x);
-ARR3SET(max, x);
-
-/* scan list and update limits */
-while (list)
-  {
-  v = list->data;
-
-  ARR3SET(x, v->x);
-
-  for (i=3 ; i-- ; )
-    {
-    if (x[i] < min[i])
-      min[i] = x[i];
-    if (x[i] > max[i])
-      max[i] = x[i];
-    }
-  list = g_slist_next(list);
-  }
-}
-
 /**********/
 /* COORDS */
 /**********/
 #define DEBUG_CENT 0
-gint coords_center(gint global, struct model_pak *data)
+gint coords_center(struct model_pak *data)
 {
 gint i, n;
-gdouble min[3], max[3], vec[3], r, r2, rmax;
-GSList *list=NULL, *ilist, *clist;
+gdouble vec[3], r, r2, rmax;
+GSList *list=NULL, *ilist;
 struct image_pak *image;
 struct spatial_pak *spatial;
 struct core_pak *core;
-struct vertex_pak *v;
 struct vec_pak *p1;
 
 /* initialize */
@@ -568,14 +418,9 @@ VEC3SET(data->offset, 0, 0, 0);
 VEC3SET(data->centroid, 0.0, 0.0, 0.0);
 r2 = 0.0;
 
-/* core list is (non-empty) selection for non global scaling */
-clist = data->cores;
-if (!global && g_slist_length(data->selection))
-  clist = data->selection;
-
-/* centroid calc */
+/* atom centroid calc */
 n=0;
-for (list=clist ; list ; list=g_slist_next(list))
+for (list=data->cores ; list ; list=g_slist_next(list))
   {
   core = list->data;
   if (core->status & DELETED)
@@ -588,28 +433,12 @@ if (n)
   {
   VEC3MUL(data->centroid, 1.0 / (gdouble) n);
   }
-else
-  {
-/* if we have no atoms in a periodic model - make pbc/2 the centroid */
-  if (data->periodic)
-    {
-    VEC3SET(data->centroid, 0.5, 0.5, 0.5);
-    }
-  else
-    {
-/* morphology centroid (NB: won't be the origin if non-centrosymetric) */
-    vertex_limits(min, max, data);
-    ARR3SET(data->centroid, min);
-    ARR3ADD(data->centroid, max);
-    VEC3MUL(data->centroid, 0.5);
-    }
-  }
 
-/* NEW - always make a pbc/2 the centroid */
+/* always make a pbc/2 the centroid in periodic cases */
 for (i=data->periodic ; i-- ; )
   data->centroid[i] = 0.5; 
 
-/* adjust for periodic images */
+/* adjust for images */
 n=1;
 VEC3SET(vec, 0.0, 0.0, 0.0);
 for (list=data->images ; list ; list=g_slist_next(list))
@@ -623,7 +452,7 @@ VEC3MUL(vec, 1.0 / (gdouble) n);
 ARR3ADD(data->centroid, vec);
 
 /* get distance of furtherest displayed item from centroid */
-for (list=clist ; list ; list=g_slist_next(list))
+for (list=data->cores ; list ; list=g_slist_next(list))
   {
   core = list->data;
   if (core->status & DELETED)
@@ -636,6 +465,7 @@ for (list=clist ; list ; list=g_slist_next(list))
     if (ilist)
       {
       image = ilist->data;
+
 /* image */
       ARR3ADD(vec, image->pic);
       ilist = g_slist_next(ilist);
@@ -673,22 +503,11 @@ if (data->periodic)
     }
   }
 
-/* check morphology vertices */
-for (list=data->vertices ; list ; list=g_slist_next(list))
-  {
-  v = list->data;
-  ARR3SET(vec, v->x);
-  ARR3SUB(vec, data->centroid);
-  vecmat(data->latmat, vec);
-  r = VEC3MAGSQ(vec);
-  if (r > r2)
-    r2 = r;
-  }
-
-/* check spatial objects */
+/* check spatial objects (includes morphology) */
 for (list=data->spatial ; list ; list=g_slist_next(list))
   {
   spatial = list->data;
+/* FIXME - better scheme? this only counts 1st vertex */
   p1 = g_slist_nth_data(spatial->list, 0);
   if (p1)
     {
@@ -697,7 +516,7 @@ for (list=data->spatial ; list ; list=g_slist_next(list))
     vecmat(data->latmat, vec);  
     r = VEC3MAGSQ(vec);
     if (r > r2)
-      r2 = r; 
+      r2 = r;
     }
   }
 
@@ -716,7 +535,7 @@ rmax = sqrt(r2);
 if (rmax)
   data->rmax = rmax;
 else
-  data->rmax = 5.0;   /* seems to give ~ correct axes lengths */
+  data->rmax = 5.0;   /* ~ correct axes lengths (completely empty model) */
 
 data->zoom = data->rmax;
 
@@ -724,8 +543,6 @@ data->zoom = data->rmax;
 P3VEC("centroid = ", data->centroid);
 printf("rmax = %f\n", data->rmax);
 printf("zoom = %f\n", data->zoom);
-printf("coords_center() done.\n");
-P3MAT("latmat", data->latmat);
 #endif
 
 return(0);
@@ -995,17 +812,6 @@ for (olist=data->spatial ; olist ; olist=g_slist_next(olist))
     }
   }
 
-/* morphology */
-  for (list1=data->vertices ; list1 ; list1=g_slist_next(list1))
-    {
-    v = list1->data;
-
-    ARR3SET(vec4, v->x);
-    vec4[3] = 1.0;
-    vec4mat(mat4, vec4);
-    ARR3SET(v->rx, vec4);
-    }
-
 /* compute facet center (miller label placement) */
   for (list1=data->planes ; list1 ; list1=g_slist_next(list1))
     {
