@@ -33,12 +33,14 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "coords.h"
 #include "edit.h"
 #include "matrix.h"
+#include "measure.h"
 #include "opengl.h"
 #include "render.h"
 #include "select.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "zone.h"
+#include "undo.h"
 
 #define DEBUG 0
 
@@ -47,6 +49,220 @@ extern struct elem_pak elements[];
 
 gint current_colour[3];
 
+/**********************/
+/* undelete primitive */
+/**********************/
+gint core_undelete(struct model_pak *model, GSList *list)
+{
+struct core_pak *core;
+struct shel_pak *shell;
+
+g_assert(model != NULL);
+g_assert(list != NULL);
+
+core = list->data;
+
+core->status &= ~DELETED;
+
+model->cores = g_slist_prepend(model->cores, core);
+if (core->shell)
+  {
+  shell = core->shell;
+
+  shell->status &= ~DELETED;
+  model->shels = g_slist_prepend(model->shels, core->shell);
+  }
+
+/* FIXME - this doesnt work */
+/*
+connect_atom_compute(core, model);
+*/
+
+/* TODO - more fine grained molecule recalc (ie recalc of affected molecules only) */
+connect_bonds(model);
+connect_molecules(model);
+
+return(TRUE);
+}
+
+/**********************************/
+/* flag core and associated shell */
+/**********************************/
+void delete_core(struct core_pak *core)
+{
+core->status |= DELETED;
+if (core->shell)
+  (core->shell)->status |= DELETED;
+}
+
+/*********************************/
+/* delete primitive for one core */
+/*********************************/
+void core_delete_single(struct core_pak *core, struct model_pak *model)
+{
+GSList *list;
+
+/* flag the core as deleted */
+delete_core(core);
+
+/* remove core references */
+connect_atom_clear(core, model);
+
+/* register the undo */
+/* TODO - when a model is deleted - flush related undo's to avoid mem leaks */
+list = NULL;
+list = g_slist_prepend(list, core);
+undo_register(model, core_undelete, list);
+
+/* remove from lists */
+model->cores = g_slist_remove(model->cores, core);
+if (core->shell)
+  model->shels = g_slist_remove(model->shels, core->shell);
+
+/* TODO - more fine grained molecule recalc (ie recalc of affected molecules only) */
+connect_molecules(model);
+}
+
+/*********************************/
+/* remove all cores from a model */
+/*********************************/
+/* NB: leave symmetry / pbc structure intact */
+void core_delete_all(struct model_pak *model)
+{
+g_assert(model != NULL);
+
+/* free core reference lists */
+free_mol_list(model);
+wipe_bonds(model);
+g_slist_free(model->unique_atom_list);
+model->unique_atom_list = NULL;
+
+/* free cores */
+free_slist(model->cores);
+model->cores = NULL;
+
+/* redo dependancies */
+zone_init(model);
+calc_emp(model);
+gui_refresh(GUI_MODEL_PROPERTIES);
+model->state = 0;
+}
+
+/*******************************************************/
+/* permanently remove all deleted cores from all lists */
+/*******************************************************/
+#define DEBUG_DELETE_COMMIT 0
+void delete_commit(struct model_pak *data)
+{
+gint flag1=FALSE, flag2=FALSE;
+gpointer m;
+GSList *list1, *list2;
+struct core_pak *core;
+struct shel_pak *shell;
+struct bond_pak *bond;
+
+g_assert(data != NULL);
+
+/* delete flaged cores in list */
+list1 = data->cores;
+while (list1)
+  {
+  core = list1->data;
+  list1 = g_slist_next(list1);
+
+  if (core->status & DELETED)
+    {
+#if DEBUG_DELETE_COMMIT
+printf("Deleting %s core [%p] ...\n", core->label, core);
+#endif
+    flag1 = TRUE;
+
+/* flag assoc. shell */
+    if (core->shell)
+      {
+      shell = core->shell;
+      shell->status |= DELETED;
+      }
+
+/* update connectivity */
+    connect_atom_clear(core, data);
+    list2 = data->ubonds;
+    while (list2)
+      {
+      bond = list2->data;
+      list2 = g_slist_next(list2);
+
+      if (bond->atom1 == core || bond->atom2 == core)
+        {
+        data->ubonds = g_slist_remove(data->ubonds, bond);
+        g_free(bond);
+        }
+      }
+
+/* update selection */
+    data->selection = g_slist_remove(data->selection, core);
+
+/* delete any labels that reference the deleted core */
+    list2 = data->measure_list;
+    while (list2)
+      {
+      m = list2->data;
+      list2 = g_slist_next(list2);
+      if (measure_has_core(core, m))
+        measure_free(m, data);
+      }
+/* update main list */
+    data->cores = g_slist_remove(data->cores, core);
+    g_free(core);
+    }
+  }
+
+/* delete flaged shells in list */
+list1 = data->shels;
+while (list1)
+  {
+  shell = list1->data;
+  list1 = g_slist_next(list1);
+
+  if (shell->status & DELETED)
+    {
+    flag2 = TRUE;
+/* update main list */
+    data->shels = g_slist_remove(data->shels, shell);
+    g_free(shell);
+    }
+  }
+
+/* refresh totals */
+data->num_atoms = g_slist_length(data->cores);
+data->num_shells = g_slist_length(data->shels);
+
+/* refresh spatial partitioning */
+/* it's probably best to refresh the partitioning here, rather than */
+/* incrementally as it's speedups for large models we're targeting */
+zone_init(data);
+
+/* cope with deleted bonds - expensive, so only do if required */
+if (flag1)
+  {
+  connect_bonds(data);
+  connect_molecules(data);
+  }
+
+/* refresh net charge calc */
+calc_emp(data);
+
+/* refresh unique atom list */
+g_slist_free(data->unique_atom_list);
+data->unique_atom_list = find_unique(ELEMENT, data);
+
+/* refresh widgets */
+meas_graft_model(data);
+gui_refresh(GUI_MODEL_PROPERTIES);
+
+/* reset functions with static pointers to cores */
+data->state = 0;
+}
 
 /*********************/
 /* copy an atom core */
@@ -129,10 +345,14 @@ core->mol = NULL;
 core->primary_core = NULL;
 
 /* duplicate strings */
-core->atom_label = g_strdup(orig->atom_label);
-core->atom_type = g_strdup(orig->atom_type);
-core->res_name = g_strdup(orig->res_name);
-core->flags = g_strdup(orig->flags);
+if (orig->atom_label)
+  core->atom_label = g_strdup(orig->atom_label);
+if (orig->atom_type)
+  core->atom_type = g_strdup(orig->atom_type);
+if (orig->res_name)
+  core->res_name = g_strdup(orig->res_name);
+if (orig->flags)
+  core->flags = g_strdup(orig->flags);
 
 core->vibx_list = NULL;
 core->viby_list = NULL;
@@ -191,11 +411,18 @@ else
 core->atom_code = code;
 core->atom_type = NULL;
 core->atom_label = NULL;
-core->res_name = g_strdup("");
+core->res_name = NULL;
 core->res_no = 1;
 core->chain = NULL;
 core->atom_order = 0;
 
+/* default values for NMR values */
+core->atom_nmr_shift=0.0;
+core->atom_nmr_aniso=0.0;
+core->atom_nmr_asym=0.0;
+core->atom_nmr_cq=0.0;
+core->atom_nmr_efgasym=0.0;
+
 /* element related initialization */
 get_elem_data(code, &elem_data, model);
 core->bond_cutoff = elem_data.cova;
@@ -385,7 +612,7 @@ return(copy);
 void delete_duplicate_cores(struct model_pak *model)
 {
 gdouble vec[3];
-GSList *list1, *list2;
+GSList *list1, *list2, *za_list;
 gpointer zone;
 struct core_pak *core1, *core2;
 struct shel_pak *s1, *s2;
@@ -409,7 +636,6 @@ for (list1=model->cores ; list1 ; list1=g_slist_next(list1))
 /* enumerate cores in current locality */
   zone = zone_get(core1->x, model->zone_array);
 
-
 #if DEBUG_REMOVE_DUPLICATES_MORE
 printf(" + %s [%p] : [%p] :", core1->atom_label, core1, zone);
 P3VEC(" ", core1->x);
@@ -420,9 +646,8 @@ P3VEC(" ", core1->x);
   for (list2=zone_cores(zone) ; list2 ; list2=g_slist_next(list2))
 */
 
-
-  for (list2=zone_area_cores(1, zone, model->zone_array) ; list2 ; list2=g_slist_next(list2))
-
+  za_list = zone_area_cores(1, zone, model->zone_array);
+  for (list2=za_list ; list2 ; list2=g_slist_next(list2))
     {
     core2 = list2->data;
     if (core2->status & DELETED)
@@ -444,7 +669,7 @@ P3VEC(" ", core2->x);
     ARR3SET(vec, core1->x);
     ARR3SUB(vec, core2->x);
 
-    fractional_minsq(vec, model->periodic);
+    fractional_min(vec, model->periodic);
 
     if (VEC3MAGSQ(vec) < FRACTION_TOLERANCE)
       {
@@ -469,6 +694,7 @@ printf(" * rm 2\n");
         }
       }
     }
+  g_slist_free(za_list);
   }
 
 /* commit before searching for duplicate shells, as a commit */
@@ -496,7 +722,7 @@ for (list1=model->shels ; list1 ; list1=g_slist_next(list1))
     ARR3SUB(vec, s2->x);
 
 /* adjust for periodicity */
-    fractional_minsq(vec, model->periodic);
+    fractional_min(vec, model->periodic);
 
     if (VEC3MAGSQ(vec) < FRACTION_TOLERANCE)
       {
@@ -517,10 +743,10 @@ printf("Final shels: %d\n", g_slist_length(model->shels));
 }
 
 /********************************************************/
-/* set vector to minimum fractional magnitute (squared) */
+/* set vector to minimum fractional offset (-0.5 - 0.5) */
 /********************************************************/
 #define DEBUG_FRAC_MINSQ 0
-void fractional_minsq(gdouble *x, gint dim)
+void fractional_min(gdouble *x, gint dim)
 {
 gint i, j;
 gdouble whole;
@@ -546,9 +772,9 @@ P3VEC("af: ", x);
 #endif
 }
 
-/*************************************/
-/* clamp coords to fractional limits */
-/*************************************/
+/*************************************************/
+/* clamp coords to fractional limits [0.0 - 1.0) */
+/*************************************************/
 #define DEBUG_CLAMP 0
 void fractional_clamp(gdouble *vec, gint *mov, gint dim)
 {
@@ -589,11 +815,10 @@ P3VEC("out: ", vec);
 /********************************************/
 /* confine a list of atoms to the unit cell */
 /********************************************/
-#define DEBUG_PBC_ATOMS 0
 void coords_confine_cores(GSList *cores, struct model_pak *model)
 {
-gint xlat[3];
-gdouble mov[3];
+gint dummy[3];
+gdouble x[3];
 GSList *list;
 struct core_pak *core;
 struct shel_pak *shell;
@@ -607,14 +832,19 @@ if (!model->periodic)
 for (list=cores ; list ; list=g_slist_next(list))
   {
   core = list->data;
-  fractional_clamp(core->x, xlat, model->periodic);
+  fractional_clamp(core->x, dummy, model->periodic);
 
 /* move shell */
   if (core->shell)
     {
     shell = core->shell;
-    ARR3SET(mov, xlat);
-    ARR3ADD(shell->x, mov);
+
+/* want core-shell distance to be smallest possible */
+    ARR3SET(x, shell->x);
+    ARR3SUB(x, core->x);
+    fractional_min(x, model->periodic);
+    ARR3SET(shell->x, core->x);
+    ARR3ADD(shell->x, x);
     }
   }
 }
@@ -738,3 +968,183 @@ printf("rejected.\n");
 return(0);
 }
 
+/*************************************************************/
+/* region changing routine - for dealing with polar surfaces */
+/*************************************************************/
+/* can now go both ways via the direction flag (UP or DOWN) */
+#define DEBUG_REGION_SWITCH_ATOM 0
+gint region_move_atom(struct core_pak *core, gint direction,
+                                    struct model_pak *data)
+{
+gint flag, primary, secondary, mov[2];
+gdouble vec[3], tmp[3], d[3];
+GSList *list;
+struct core_pak *comp;
+
+#if DEBUG_REGION_SWITCH_ATOM
+printf("       model: %s\n", data->basename);
+printf(" periodicity: %d\n", data->periodic);
+printf("         hkl: %f %f %f\n", data->surface.miller[0],
+          data->surface.miller[1], data->surface.miller[2]);
+printf("        dhkl: %f\n", data->surface.dspacing);
+printf("region sizes: %f %f\n", data->surface.region[0], data->surface.region[1]);
+printf("      moving: ");
+if (direction == UP)
+  printf("UP\n");
+else
+  printf("DOWN\n");
+#endif
+
+/* checks */
+g_return_val_if_fail(data != NULL, 1);
+g_return_val_if_fail(data->periodic == 2, 1);
+if (data->surface.region[0] < 1)
+  {
+  gui_text_show(ERROR, "region 1 is empty.\n");
+  return(1);
+  }
+
+/* setup region switching labels */
+if (direction == UP)
+  {
+  primary = REGION1A;
+  secondary = REGION2A;
+  }
+else
+  {
+  primary = REGION2A;
+  secondary = REGION1A;
+  }
+
+/* get fractional depth translation vector */
+ARR3SET(vec, data->surface.depth_vec);
+vecmat(data->ilatmat, vec);
+
+/* calculate offset to region boundary */
+ARR3SET(tmp, vec);
+if (direction == DOWN)
+  {
+  VEC3MUL(tmp, data->surface.region[0]);
+  VEC3MUL(tmp, -1.0);
+  }
+else
+  {
+  if (data->surface.region[1] == 0)
+    {
+    VEC3MUL(tmp, data->surface.region[0]);
+    }
+  else
+    {
+    VEC3MUL(tmp, data->surface.region[1]);
+    }
+  }
+
+/* if region 2 is empty, just move core to the bottom */
+if (data->surface.region[1] == 0.0)
+  {
+  ARR3ADD(core->x, tmp);
+  if (core->shell)
+    {
+    ARR3ADD((core->shell)->x, tmp);
+    }
+  atom_colour_scheme(data->colour_scheme, core, data);
+  return(0);
+  }
+
+/* get coordinates of target atom */
+ARR3ADD(tmp, core->x);
+
+#if DEBUG_REGION_SWITCH_ATOM
+P3VEC("    translation: ", vec);
+P3VEC("  target coords: ", tmp);
+#endif
+
+/* find the target */
+flag=0;
+for (list=data->cores ; list ; list=g_slist_next(list))
+  {
+  comp = list->data;
+
+/* only atoms of the same type need apply */
+  if (core->atom_code != comp->atom_code)
+    continue;
+
+/* get difference vector */
+  ARR3SET(d, comp->x);
+  ARR3SUB(d, tmp);
+
+/* pbc constraint */
+  while(d[0] < -FRACTION_TOLERANCE)
+    d[0] += 1.0;
+  while(d[0] > 0.5)
+    d[0] -= 1.0;
+  while(d[1] < -FRACTION_TOLERANCE)
+    d[1] += 1.0;
+  while(d[1] > 0.5)
+    d[1] -= 1.0;
+
+/* test difference vector's magnitude */
+  if (VEC3MAGSQ(d) < FRACTION_TOLERANCE)
+    {
+/* change its labelling */
+#if DEBUG_REGION_SWITCH_ATOM
+printf("Matched core: %p\n", comp);
+#endif
+    comp->region = secondary;
+    if (comp->shell)
+      {
+      (comp->shell)->region = secondary;
+      }
+    atom_colour_scheme(data->colour_scheme, comp, data);
+    flag++;
+    break;
+    }
+  }
+
+if (!flag)
+  {
+  gui_text_show(ERROR, "Failed to find a boundary image.\n");
+  return(1);
+  }
+
+/* now move selected atom to bottom of region 2 */
+ARR3SET(tmp, vec);
+VEC3MUL(tmp, (data->surface.region[0] + data->surface.region[1]));
+if (direction == UP)
+  VEC3MUL(tmp, -1.0);
+ARR3SUB(core->x, tmp);
+core->region = primary;
+/* pbc constrain */
+fractional_clamp(core->x, mov, 2);
+
+if (core->shell)
+  {
+  ARR3SUB((core->shell)->x, tmp);
+  ARR2ADD((core->shell)->x, mov);
+  (core->shell)->region = primary;
+  }
+
+atom_colour_scheme(data->colour_scheme, core, data);
+
+return(0);
+}
+
+/*********************************************/
+/* find and record the maximum region number */
+/*********************************************/
+gint region_max(struct model_pak *mdata)
+{
+gint max;
+GSList *list;
+struct core_pak *core;
+
+max = 0;
+for (list=mdata->cores ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+  if (core->region > max)
+    max = core->region;
+  }
+return(max);
+}
+
