@@ -15,7 +15,6 @@ void info_torsion(gint, gint, struct model_pak *);
 
 void elem_change_colour(GtkWidget *, gpointer *);
 
-void gulp_init(gpointer);
 void gulp_files_init(struct model_pak *);
 void gulp_data_free(struct model_pak *);
 void gulp_files_free(struct model_pak *);
@@ -37,3 +36,9 @@ void periodicity_page(GtkWidget *);
 
 void atom_numbers_box(GtkWidget *);
 
+void delete_commit(struct model_pak *);
+
+void core_delete_single(struct core_pak *, struct model_pak *);
+void core_delete_all(struct model_pak *);
+
+
