@@ -33,6 +33,7 @@ The GNU GPL can also be found at http://www.gnu.org
 
 #include "gdis.h"
 #include "coords.h"
+#include "error.h"
 #include "file.h"
 #include "parse.h"
 #include "matrix.h"
@@ -41,7 +42,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "zone.h"
 #include "render.h"
 #include "select.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
 #include "opengl.h"
@@ -144,7 +145,6 @@ file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = BIOGRAF;
 file_data->group = BIOGRAF;
 file_data->menu = TRUE;
-file_data->label = g_strdup("bgf");
 file_data->label = g_strdup("Biograf");                 
 file_data->ext = NULL;
 file_data->ext = g_slist_prepend(file_data->ext, "bgf");
@@ -175,7 +175,8 @@ file_data->group = CASTEP;
 file_data->menu = TRUE;
 file_data->label = g_strdup("CASTEP");
 file_data->ext = NULL;
-file_data->write_file = NULL;
+file_data->ext = g_slist_append(file_data->ext, "cell");
+file_data->write_file = write_castep_cell;
 file_data->read_file = NULL;
 file_data->read_frame = NULL;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
@@ -212,7 +213,6 @@ file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = CRYSTAL_GRAPH;
 file_data->group = CRYSTAL_GRAPH;
 file_data->menu = TRUE;
-file_data->label = g_strdup("cgf");
 file_data->label = g_strdup("Crystal Graph Files");                 
 file_data->ext = NULL;
 file_data->ext = g_slist_prepend(file_data->ext, "cgf");
@@ -269,7 +269,7 @@ file_data->ext = NULL;
 file_data->ext = g_slist_prepend(file_data->ext, "dmol");
 file_data->write_file = write_dmol;
 file_data->read_file = read_dmol;
-file_data->read_frame = NULL;
+file_data->read_frame = read_dmol_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
@@ -339,6 +339,19 @@ file_data->read_file = read_gauss_out;
 file_data->read_frame = read_gauss_out_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = GROMACS;
+file_data->group = GROMACS;
+file_data->menu = FALSE;
+file_data->label = g_strdup("GROMACS input");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "gro");
+file_data->write_file = write_gromacs;
+file_data->read_file = read_gromacs_gro;
+file_data->read_frame = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
 /* supported file type */
 file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = MORPH;
@@ -423,6 +436,19 @@ file_data->read_file = read_mvnout;
 file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = META_DATA;
+file_data->group = META_DATA;
+file_data->menu = FALSE;
+file_data->label = g_strdup("Meta data");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "meta");
+file_data->write_file = write_meta;
+file_data->read_file = NULL;
+file_data->read_frame = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
 /* supported file type */
 file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = NWCHEM;
@@ -555,6 +581,19 @@ file_data->read_file = read_sout;
 file_data->read_frame = read_sout_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = TEXT;
+file_data->group = TEXT;
+file_data->menu = FALSE;
+file_data->label = g_strdup("Text file");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "txt");
+file_data->write_file = NULL;
+file_data->read_file = NULL;
+file_data->read_frame = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
 /* supported file type */
 file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = XML;
@@ -601,6 +640,26 @@ sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 sysenv.file_list = g_slist_reverse(sysenv.file_list);
 }
 
+/******************************/
+/* free file recognition list */
+/******************************/
+void file_free(void)
+{
+GSList *list;
+
+/* free data embedded in the file structure */
+for (list=sysenv.file_list ; list ; list=g_slist_next(list))
+  {
+  struct file_pak *file_data = list->data;
+
+  g_slist_free(file_data->ext);
+  g_free(file_data->label);
+  }
+
+/* free the list and the file structure pointers */
+free_slist(sysenv.file_list);
+}
+
 /************************/
 /* current version info */
 /************************/
@@ -620,7 +679,7 @@ return(g_ascii_strcasecmp((gchar *) ptr1, (gchar *) ptr2));
 /*********************************************/
 /* an all-platform directory listing routine */
 /*********************************************/
-GSList *get_dir_list(const char *path, gint sort)
+GSList *file_dir_list(const gchar *path, gint sort)
 {
 const gchar *name;
 GDir *dir;
@@ -647,6 +706,47 @@ if (sort)
 return(files);
 }
 
+/*********************************************************************/
+/* get a directory listing matched against a supplied (glob) pattern */
+/*********************************************************************/
+#define DEBUG_FILE_PATTERN 0
+GSList *file_dir_list_pattern(const gchar *dir, const gchar *pattern)
+{
+gchar *name;
+GPatternSpec *ps;
+GSList *list, *files;
+
+g_assert(dir != NULL);
+
+files = file_dir_list(dir, FALSE);
+
+if (pattern)
+  ps = g_pattern_spec_new(pattern);
+else
+  return(files);
+
+list = files;
+while (list)
+  {
+  name = list->data;
+  list = g_slist_next(list);
+
+  if (!g_pattern_match_string(ps, name))
+    {
+    files = g_slist_remove(files, name);
+    g_free(name);
+    }
+  }
+
+g_pattern_spec_free(ps);
+
+#if DEBUG_FILE_PATTERN
+printf("Found %d matches\n", g_slist_length(files));
+#endif
+
+return(files);
+}
+
 /*******************************************************/
 /* allocate a new string containing the file extension */
 /*******************************************************/
@@ -862,7 +962,7 @@ do
   {
   g_string_sprintf(filename,"dummy_%d.%s", i, ext);
 #if DEBUG_GUN
-printf("testing: %s\n",filename->str);
+printf("testing: %s\n", filename->str);
 #endif
   i++;
   }
@@ -960,10 +1060,15 @@ return(0);
 /*****************************************/
 /* read in and return a line of any size */
 /*****************************************/
-/* TODO - replacement routine for fgetline() */
+gint skip_comment = TRUE;
+void file_skip_comment(gint skip)
+{
+skip_comment = skip;
+}
+/* returns NULL on EOF */
 gchar *file_read_line(FILE *fp)
 {
-gboolean comment;
+gboolean skip;
 gchar c;
 GString *buff;
 
@@ -977,10 +1082,10 @@ if (c == EOF)
 buff = g_string_new(NULL);
 while (c != EOF)
   {
-  if (c == '#')
-    comment = TRUE;
-  else
-    comment = FALSE;
+  skip = FALSE;
+  if (c == '#' && skip_comment)
+    skip = TRUE;
+
 /* read a complete line */
   while (c != EOF)
     {
@@ -1005,8 +1110,8 @@ while (c != EOF)
       }
     c = fgetc(fp);
     }
-/* ignore comments (keep reading) */
-  if (comment)
+/* ignore? (keep reading) */
+  if (skip)
     {
     g_string_assign(buff, "");
     c = fgetc(fp);
@@ -1136,21 +1241,42 @@ else
 /* trg files contain ALL atoms & symmetry may be broken */
     model->sginfo.lookup = FALSE;
 
-/* FIXME - trj files resist every attempt at random access... why??? */
     filename = g_strdup_printf("%s/%s", sysenv.cwd, model->gulp.trj_file);
-    fp2 = fopen(filename, "r");
+/* NB: GULP binary trajectory files - MUST specify "rb" */
+/* FIXME - some GULP traj files may be ascii I think? */
+    fp2 = fopen(filename, "rb");
     if (!fp2)
       {
       printf("Failed to open: %s\n", filename);
       g_free(filename);
       return(3);
       }
+
 /* FIXME - ugly hack to get the nth frame */
+#define CRAP 0
+#if CRAP
     read_trj_header(fp2, model);
     for (i=0 ; i<n-1 ; i++)
       read_trj_frame(fp2, model, FALSE);
     read_trj_frame(fp2, model, TRUE);
     fclose(fp2);
+#else
+
+/* CURRENT - implementing fgetpos/fsetpos method */
+  offset = g_list_nth_data(model->frame_list, n);
+  if (fsetpos(fp2, offset))
+    {
+    printf("Error positioning file pointer.\n");
+    g_free(filename);
+    return(2);
+    }
+
+  read_trj_frame(fp2, model, TRUE);
+
+#endif
+
+
+
     g_free(filename);
     flag = 0;
     }
@@ -1175,6 +1301,10 @@ struct file_pak *file_data;
 printf("loading: [%s] into: %p\n", filename, mdata);
 #endif
 
+/* FIXME - new error reporting - doesnt work for gulp */
+error_table_clear();
+error_table_enable();
+
 /* get the file structure required to parse the file */
 switch (sysenv.file_type)
   {
@@ -1321,11 +1451,14 @@ if (data->gulp.trj_file && !data->gulp.orig_cores)
 /* finished - only now we destroy the file dialog */
 dialog_destroy_type(FILE_SELECT);
 tree_select_active();
+
+error_table_print_all();
 }
 
 /****************/
 /* load dialog */
 /***************/
+#ifdef WITH_GUI
 void file_load_dialog(void)
 {
 /* revert to data filetypes for special cases */
@@ -1340,6 +1473,7 @@ switch (sysenv.file_type)
 
 file_dialog("Load file", NULL, FILE_LOAD, (gpointer) file_load, sysenv.file_type);
 }
+#endif
 
 /**********************************/
 /* save file with name */
@@ -1365,7 +1499,7 @@ if (file_data)
   id = file_data->id;
   strcpy(model->filename, filename);
   g_free(model->basename);
-  model->basename = strdup_basename(filename);
+  model->basename = parse_strip(filename);
   }
 else
   {
@@ -1381,7 +1515,7 @@ else
 
   id = file_data->id;
   g_free(model->basename);
-  model->basename = strdup_basename(filename);
+  model->basename = parse_strip(filename);
 
   name = g_strdup_printf("%s.%s", model->basename, ext);
   strcpy(model->filename, name);
@@ -1437,6 +1571,7 @@ g_free(name);
 /******************/
 /* save as dialog */
 /******************/
+#ifdef WITH_GUI
 void file_save_dialog(void)
 {
 gchar *text=NULL;
@@ -1449,6 +1584,7 @@ if (model)
 file_dialog("File save", text, FILE_SAVE, (gpointer) file_save_as,
                                                 sysenv.file_type);
 }
+#endif
 
 /*************************************/
 /* get the size of a file (in bytes) */
@@ -1462,4 +1598,87 @@ stat(filename, &buff);
 return(buff.st_size);
 }
 
+/************************************/
+/* search for an executable program */
+/************************************/
+/* TODO - put elsewhere ... look for a program ... ask for a path (can cancel) if not found */
+gchar *file_find_program(const gchar *name)
+{
+gchar *path;
+
+path = g_find_program_in_path(name);
+
+return(path);
+}
+
+/***********************************/
+/* change to a specified directory */
+/***********************************/
+#define DEBUG_SET_PATH 0
+gint set_path(const gchar *txt)
+{
+gint i, n, status=0;
+gchar **buff, *text;
+GString *path;
+
+/* split by directory separator */
+text = g_strdup(txt);
+g_strstrip(text);
+buff = g_strsplit(text, DIR_SEP, 0);
+path = g_string_new(NULL);
+
+/* find the number of tokens */
+n=0;
+while(*(buff+n))
+  {
+#if DEBUG_SET_PATH
+printf("[%s] ", *(buff+n));
+#endif
+  n++;
+  }
+#if DEBUG_SET_PATH
+printf(": found %d tokens.\n", n);
+#endif
+
+/* truncate token list if parent directory selected */
+if (n > 1)
+  if (g_ascii_strncasecmp("..",*(buff+n-1),2) == 0)
+    n -= 2;
+
+/* build new path */
+/* this is a bit fiddly, as we don't want a dir_sep on the end */
+/* EXCEPT if it's the root directory (blame windows for this) */
+g_string_sprintf(path, "%s", *buff);
+i=1;
+while (i < n)
+  {
+  g_string_sprintfa(path, "%s%s", DIR_SEP, *(buff+i));
+  i++;
+  }
+if (n < 2)
+  g_string_sprintfa(path, "%s", DIR_SEP);
+
+#if DEBUG_SET_PATH
+printf("testing path [%s] ... \n", path->str); 
+#endif
+
+if (g_file_test(path->str, G_FILE_TEST_IS_DIR))
+  {
+  g_free(sysenv.cwd);
+  sysenv.cwd = g_strdup(path->str);
+  }
+else
+  {
+#if DEBUG_SET_PATH
+printf(" Not a directory.\n");
+#endif
+  status++;
+  }
+
+g_free(text);
+g_strfreev(buff);
+g_string_free(path, TRUE);
+
+return(status);
+}
 
