@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,30 +20,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 The GNU GPL can also be found at http://www.gnu.org
 */
 
-#include <math.h>
 #include <stdio.h>
 #include <string.h>
-#include <strings.h>
-#include <ctype.h>
-#include <stdlib.h>
 #include <time.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-
-#ifdef __sun
-#include <sys/dirent.h>
-#else
-#ifdef __WIN32
-#include <dirent.h>
-#else
-#include <sys/dir.h>
-#include <sys/param.h>
-#endif
-#endif
 
 #include "gdis.h"
 #include "coords.h"
+#include "model.h"
 #include "file.h"
 #include "parse.h"
 #include "matrix.h"
@@ -56,7 +39,6 @@ The GNU GPL can also be found at http://www.gnu.org
 extern struct sysenv_pak sysenv;
 extern struct elem_pak elements[];
 
-
 /****************************/
 /* write a coordinate block */
 /****************************/
@@ -97,14 +79,14 @@ if (model->periodic)
 m=0;
 for (mlist=model->moles ; mlist ; mlist=g_slist_next(mlist))
   {
-  mol = (struct mol_pak *) mlist->data;
+  mol = mlist->data;
 
 /* flag - atom written to file? (ie if UNDELETED) */
   flag=0;
 /* m - number of atoms in molecule */
   for (clist=mol->cores ; clist ; clist=g_slist_next(clist))
     {
-    core = (struct core_pak *) clist->data;
+    core = clist->data;
     if (core->status & DELETED)
       continue;
 
@@ -115,13 +97,13 @@ for (mlist=model->moles ; mlist ; mlist=g_slist_next(mlist))
     if (core->atom_type)
       {
       fprintf(fp, "%-4s  %14.9f %14.9f %14.9f CORE %-6d %-7s %-2s ",
-                  core->label,x[0],x[1],x[2],m+1, core->atom_type,
+                  core->atom_label,x[0],x[1],x[2],m+1, core->atom_type,
                   elements[core->atom_code].symbol);
       }
     else
       {
       fprintf(fp, "%-4s  %14.9f %14.9f %14.9f CORE %-6d ?       %-2s ",
-                  core->label,x[0],x[1],x[2],m+1, 
+                  core->atom_label,x[0],x[1],x[2],m+1, 
                   elements[core->atom_code].symbol);
       }
 
@@ -247,10 +229,7 @@ for (;;)
   {
   buff = get_tokenized_line(fp, &num_tokens);
   if (!buff)
-    {
-    g_strfreev(buff);
     break;
-    }
 
 /* increment/reset end counter according to input line */
   if (g_ascii_strncasecmp("end", *buff, 3) == 0)
@@ -265,10 +244,7 @@ for (;;)
     continue;
     }
   if (end_count == 2)
-    {
-    g_strfreev(buff);
     break;
-    }
 
 /* cell dimension search */
   if (g_ascii_strncasecmp("pbc", *buff, 3) == 0)
@@ -281,17 +257,31 @@ for (;;)
       data->pbc[3] = PI*str_to_float(*(buff+4))/180.0;
       data->pbc[4] = PI*str_to_float(*(buff+5))/180.0;
       data->pbc[5] = PI*str_to_float(*(buff+6))/180.0;
+/* implicit 2D */
+      if (data->pbc[2] == 0)
+        {
+        data->pbc[2] = 1.0;
+        data->pbc[3] = 0.5*PI;
+        data->pbc[4] = 0.5*PI;
+        data->periodic = 2;
+        }
       }
-/* cope with 2D */
-    if (data->pbc[2] == 0)
-      data->periodic = 2;
+    else if (num_tokens > 3)
+      {
+      data->pbc[0] = str_to_float(*(buff+1));
+      data->pbc[1] = str_to_float(*(buff+2));
+      data->pbc[5] = PI*str_to_float(*(buff+3))/180.0;
+      }
+
+    g_strfreev(buff);
     continue;
     }
 
 /* process a single coord line */
   if (num_tokens < 8) 
     {
-    show_text(ERROR, "unexpected end of file reading arc file\n");
+    gui_text_show(ERROR, "unexpected end of file reading arc file\n");
+    g_strfreev(buff);
     return;
     }
 
@@ -316,7 +306,7 @@ for (;;)
       {
       if (clist)
         {
-        core = (struct core_pak *) clist->data;
+        core = clist->data;
         clist = g_slist_next(clist);
         }
       else
@@ -330,8 +320,8 @@ for (;;)
         }
 
 /* set the proper label */
-/* FIXME - dangerous (TODO - make the label a pointer) */
-strcpy(core->label, *(buff+0));
+g_free(core->atom_label);
+core->atom_label = g_strdup(*buff);
 
 /* set values */
       core->x[0] = str_to_float(*(buff+1));
@@ -350,7 +340,7 @@ core->atom_type = g_strdup(*(buff+6));
 /* overwrite existing shell (if any) */
       if (slist)
         {
-        shel = (struct shel_pak *) slist->data;
+        shel = slist->data;
         slist = g_slist_next(slist);
         }
       else
@@ -359,9 +349,10 @@ core->atom_type = g_strdup(*(buff+6));
         shel = new_shell(*(buff+7), data);
         data->shels = g_slist_append(data->shels, shel);
         }
+
 /* set the proper label */
-/* FIXME - dangerous (TODO - make the label a pointer) */
-strcpy(shel->label, *(buff+0));
+g_free(shel->shell_label);
+shel->shell_label = g_strdup(*buff);
 
 /* set values */
       shel->x[0] = str_to_float(*(buff+1));
@@ -374,6 +365,16 @@ strcpy(shel->label, *(buff+0));
     }
   g_strfreev(buff);
   }
+
+/* convert input cartesian coords to fractional */
+if( data->periodic)
+  {
+  matrix_lattice_init(data);
+  coords_make_fractional(data);
+  }
+data->fractional = TRUE;
+
+g_strfreev(buff);
 }
 
 /*********************/
@@ -389,7 +390,6 @@ return(0);
 /******************/
 /* Biosym reading */
 /******************/
-#define DEBUG_READ_ARC 0
 gint read_arc(gchar *filename, struct model_pak *data)
 {
 gint frame=0, num_tokens;
@@ -400,7 +400,8 @@ FILE *fp;
 g_return_val_if_fail(data != NULL, 1);
 g_return_val_if_fail(filename != NULL, 2);
 
-fp = fopen(filename, "rt");
+/* NEW - under windows, must be "rb" instead of "rt" - even though it's a text file */
+fp = fopen(filename, "rb");
 if (!fp)
   return(3);
 
@@ -417,8 +418,10 @@ for (;;)
 /* pbc on/off search */
   if (g_ascii_strncasecmp("pbc=on", *buff, 6) == 0)
     data->periodic = 3;
-  if (g_ascii_strncasecmp("pbc=off", *buff, 7) == 0)
+  else if (g_ascii_strncasecmp("pbc=off", *buff, 7) == 0)
     data->periodic = 0;
+  else if (g_ascii_strncasecmp("pbc=2d", *buff, 6) == 0)
+    data->periodic = 2;
 
 /* coords search */
   if (g_ascii_strncasecmp("!date", *buff, 5) == 0)
@@ -449,9 +452,9 @@ if (data->num_frames == 1)
 /* model setup */
 strcpy(data->filename, filename);
 g_free(data->basename);
-data->basename = strdup_basename(filename);
+data->basename = parse_strip(filename);
 
-prep_model(data);
+model_prep(data);
 
 return(0);
 }
