@@ -1243,7 +1243,7 @@ switch (state)
 #define DEBUG_READ_FDF_BLOCK 0
 gint read_fdf_block(FILE *fp, struct model_pak *model, gboolean read_inital_cooords)
 {
-gint count, num_tokens, block_type;
+gint count, num_tokens, block_type, siesta_sucks = FALSE;
 gchar **buff, *line;
 gdouble scale_lattice = 1.0, scale_coords = 1.0;
 GSList *species_list=NULL;
@@ -1346,7 +1346,11 @@ printf("end of block.\n");
       else
         {
         model->fractional = FALSE;
-/* NEW */
+
+/* cope with cartesians that arent really cartesian */
+        if (g_strrstr(line, "ScaledCartesian"))
+          siesta_sucks = TRUE;
+
         if (g_strrstr(line, "Bohr"))
           scale_coords = AU2ANG;
         }
@@ -2165,16 +2169,6 @@ printf("end of block.\n");
         zmat_angle_units_set(model->zmatrix, RADIANS); 
       }
     }
-
-#if OLD_STYLE
-  if (read_inital_cooords)
-  {
-  if (g_ascii_strncasecmp("AtomicCoordinates", *(buff+1), 17) == 0)
-    {
-    }
-  }
-#endif
-    
     
     // exit condition - "************************** End of input data file"
     if (g_ascii_strncasecmp("**************************", *buff, 26) == 0)
@@ -2198,14 +2192,26 @@ printf("end of block.\n");
 siesta_done_file:;
 
 
+/* convert scaled cartesians to proper cartesians */
+if (siesta_sucks)
+  {
+  GSList *list;
+  struct core_pak *core;
+
+  for (list=model->cores ; list ; list=g_slist_next(list))
+    {
+    core = list->data;
+    VEC3MUL(core->x, scale_lattice);
+    }
+  }
+
+
 /* NEW - process zmatrix cores */
 zmat_type(model->zmatrix, species_list);
 zmat_process(model->zmatrix, model);
 
-
 /* free the species list */
 free_slist(species_list);
-
   
 return 0;
 }
@@ -2279,6 +2285,8 @@ buff = tokenize(line, &num_tokens);
 
 while (num_tokens > 4)
   {
+  core = NULL;
+
   if (clist)
     {
     core = clist->data;
@@ -2286,13 +2294,24 @@ while (num_tokens > 4)
     }
   else
     {
-    core = new_core(*(buff+4), model);
-    model->cores = g_slist_append(model->cores, core);
+/* NEW - siesta can change the column the element symbol appears in */
+    for (i=3 ; i<num_tokens ; i++)
+      {
+      if (elem_symbol_test(*(buff+i)))
+        {
+        core = new_core(*(buff+i), model);
+        model->cores = g_slist_append(model->cores, core);
+        break;
+        }
+      }
     }
 
-  core->x[0] = str_to_float(*(buff+0));
-  core->x[1] = str_to_float(*(buff+1));
-  core->x[2] = str_to_float(*(buff+2));
+  if (core)
+    {
+    core->x[0] = str_to_float(*(buff+0));
+    core->x[1] = str_to_float(*(buff+1));
+    core->x[2] = str_to_float(*(buff+2));
+    }
 
 /* get next line */
   g_strfreev(buff);
@@ -2377,6 +2396,7 @@ while (!fgetline(fp, line))
     g_strfreev(buff);
     }
 
+  /* FIXME This only works at the end of an optimization. Need to fix for all frames */
   if (g_ascii_strncasecmp(line, "siesta: Pressure (static)", 25) == 0)
     {
     for (i=0; i<4; i++)
@@ -2446,8 +2466,13 @@ if (!fp)
   return(1);
 
 fgetline(fp, line);
+/*
 while (g_ascii_strncasecmp("************************** Dump of input data file", line, 50) != 0)
     fgetline(fp, line);
+*/
+
+while (!strstr(line, "Dump of input data file"))
+    fgetline(fp, line);
 
 //should be at the input file params -- dont want inital co-ords.
 //read_fdf_block(fp,model, TRUE);
@@ -2513,6 +2538,10 @@ while (!fgetline(fp, line))
       return(2);
     if (g_ascii_strncasecmp(line, "xc_check: GGA Perdew, Burke & Ernzerhof 1996", 44) == 0)
       property_add_ranked(7, "Functional", "PBE", model);
+    else if (g_ascii_strncasecmp(line, "xc_check: GGA Becke Lee Yang Parr", 33) == 0)
+      property_add_ranked(7, "Functional", "BLYP", model);
+    else
+      property_add_ranked(7, "Functional", "Unknown", model);
     }
 
 /* k-points */
@@ -2547,7 +2576,7 @@ while (!fgetline(fp, line))
 /* coordinates */
   if (model->siesta.md_type_of_run != FC_MDRUN)
   {
-      if (g_ascii_strncasecmp(line, "outcoor: Atomic coordinates", 27) == 0)
+    if (g_ascii_strncasecmp(line, "outcoor: ", 9) == 0)
       {
         if (g_strrstr(line, "Ang") != NULL)
           model->fractional = FALSE;
@@ -2555,7 +2584,6 @@ while (!fgetline(fp, line))
           model->fractional = TRUE;
         else if (g_strrstr(line, "Bohr") != NULL)
           model->coord_units = BOHR;
-    
         else
           {
           gui_text_show(ERROR, "unexpected coordinate type\n");
@@ -2586,7 +2614,7 @@ while (!fgetline(fp, line))
 /* done */
 strcpy(model->filename, filename);
 g_free(model->basename);
-model->basename = strdup_basename(filename);
+model->basename = parse_strip(filename);
 model->num_frames = model->cur_frame = frame;
 model->cur_frame--;
 
