@@ -59,7 +59,7 @@ if (!fp)
   return(3);
 
 g_free(model->basename);
-model->basename = strdup_basename(filename);
+model->basename = parse_strip(filename);
 fprintf(fp, "%%chk=%s.chk\n", model->basename);
 fprintf(fp, "\n");
 fprintf(fp, "# sp hf/sto-3g\n");
@@ -396,7 +396,7 @@ while (!fgetline(fp, line))
 /* done */
 strcpy(model->filename, filename);
 g_free(model->basename);
-model->basename = strdup_basename(filename);
+model->basename = parse_strip(filename);
 model->num_frames = model->cur_frame = frame;
 model->cur_frame--;
 
