@@ -35,7 +35,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "render.h"
 #include "space.h"
 #include "surface.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 
 
@@ -300,26 +300,10 @@ return(0);
 gint facet_equiv(struct model_pak *data, gint *f1, gint *f2)
 {
 gint i, index[3];
-gdouble d1, d2, vec[3];
+gdouble vec[3];
 
-/* Dhkl test (necessary, but not sufficient - some Dhkl's are very close) */
-ARR3SET(vec, f1);
-vecmat(data->rlatmat, vec);
-d1 = 1.0/VEC3MAG(vec);
-ARR3SET(vec, f2);
-vecmat(data->rlatmat, vec);
-d2 = 1.0/VEC3MAG(vec);
-
-/*
-if (fabs(d2-d1) > FRACTION_TOLERANCE)
-  return(FALSE);
-*/
-
-/* CURRENT - if we make the test very small - may be good enough */
-if (fabs(d2-d1) < 0.000000001)
-  return(TRUE);
-
-return(FALSE);
+/* NB: Dhkl test is just plain wrong, some symmetry related faces */
+/* will have exactly the same Dhkl and be different surfaces */
 
 #if DEBUG_FACET_EQUIV
 printf("[%d %d %d] and (%d %d %d)", f1[0], f1[1], f1[2], f2[0], f2[1], f2[2]);
@@ -585,7 +569,7 @@ data->axes_type = OTHER;
 data->morph_label = TRUE;
 strcpy(data->filename, filename);
 g_free(data->basename);
-data->basename = strdup_basename(data->filename);
+data->basename = parse_strip(data->filename);
 
 /* read from a gmf file */
 if (load_planes(filename, data))
