@@ -138,7 +138,66 @@ extern struct sysenv_pak sysenv;
 extern struct elem_pak elements[];
 
 /* structures for gamess */
-extern struct basis_pak basis_sets[];
+struct basis_pak basis_sets[] = {
+{"User Defined", GMS_USER, 0},
+{"MNDO", GMS_MNDO, 0},
+{"AM1", GMS_AM1, 0},
+{"PM3", GMS_PM3, 0},
+{"MINI", GMS_MINI, 0},
+{"MIDI", GMS_MIDI, 0},
+{"STO-2G", GMS_STO, 2},
+{"STO-3G", GMS_STO, 3},
+{"STO-4G", GMS_STO, 4},
+{"STO-5G", GMS_STO, 5},
+{"STO-6G", GMS_STO, 6},
+{"3-21G", GMS_N21, 3},
+{"6-21G", GMS_N21, 6},
+{"4-31G", GMS_N31, 4},
+{"5-31G", GMS_N31, 5},
+{"6-31G", GMS_N31, 6},
+{"6-311G", GMS_N311, 6},
+{"DZV", GMS_DZV, 0},
+{"DH", GMS_DH, 0},
+{"TZV", GMS_TZV, 0},
+{"MC", GMS_MC, 0},
+{NULL, 0, 0}
+};
+
+/***********************************/
+/* GAMESS structure initialization */
+/***********************************/
+void gamess_init(gpointer data)
+{
+struct gamess_pak *gamess = data;
+
+gamess->units = GMS_ANGS;
+gamess->exe_type = GMS_RUN;
+gamess->run_type = GMS_ENERGY;
+gamess->scf_type = GMS_RHF;
+gamess->basis = GMS_MNDO;
+gamess->dft = FALSE;
+gamess->dft_functional = 0;
+gamess->ngauss = 0;
+gamess->opt_type = GMS_QA;
+gamess->total_charge = 0;
+gamess->multiplicity = 1;
+gamess->time_limit = 600;
+gamess->mwords = 1;
+gamess->num_p = 0;
+gamess->num_d = 0;
+gamess->num_f = 0;
+gamess->have_heavy_diffuse = FALSE;
+gamess->have_hydrogen_diffuse = FALSE;
+gamess->converged = FALSE;
+gamess->wide_output = FALSE;
+gamess->nstep = 20;
+gamess->maxit = 30;
+gamess->title = g_strdup("none");
+gamess->temp_file = g_strdup("none");
+gamess->out_file = g_strdup("none");
+gamess->energy = gamess->max_grad = gamess->rms_grad = 0.0;
+gamess->have_energy = gamess->have_max_grad = gamess->have_rms_grad = FALSE;
+}
 
 /****************/
 /* file writing */
@@ -691,7 +750,7 @@ gui_text_show(STANDARD, "\n");
 
 strcpy(model->filename, filename);
 g_free(model->basename);
-model->basename = strdup_basename(filename);
+model->basename = parse_strip(filename);
 model_prep(model);
 return(0);
 }
@@ -1314,7 +1373,7 @@ if (flag)
   /* set frame if don't want last? */
   strcpy(model->filename, filename);
   g_free(model->basename);
-  model->basename = strdup_basename(filename);
+  model->basename = parse_strip(filename);
 
   model->num_frames = model->cur_frame = frame;
   model->cur_frame--;
