@@ -158,7 +158,7 @@ model->id = MVNOUT;
 model->periodic = 2;
 strcpy(model->filename, filename);
 g_free(model->basename);
-model->basename = strdup_basename(filename);
+model->basename = parse_strip(filename);
 
 for (i=REGION2B ; i-- ; )
   count[i] = 0;
@@ -568,7 +568,7 @@ for(i=0;i<9;i++)
 /* model setup */
 strcpy(model->filename, filename);
 g_free(model->basename);
-model->basename = strdup_basename(filename);
+model->basename = parse_strip(filename);
 
 model_prep(model);
 
