@@ -36,17 +36,42 @@ The GNU GPL can also be found at http://www.gnu.org
 extern struct sysenv_pak sysenv;
 extern struct elem_pak elements[];
 
+
+/* FIXME - deal with res_name , chain values of NULL */
+void write_pdb_core(FILE *fp, gint i, struct core_pak *core, struct model_pak *model)
+{
+gdouble x[3];
+gchar *res_name, *chain;
+
+ARR3SET(x, core->x);
+vecmat(model->latmat, x);
+
+if (!core->res_name)
+  res_name = "UNK";
+else
+  res_name = core->res_name;
+
+if (!core->chain)
+  chain = "A";
+else
+  chain = core->chain;
+
+fprintf(fp,"ATOM  %5d %-4s %3s %1s %3d    %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%-2s\n",
+            i, core->atom_label, res_name, chain, core->res_no,
+            x[0], x[1], x[2], core->sof, 0.0, "", elements[core->atom_code].symbol, "");
+}
+
+
 /******************/
 /* PDB writing */
 /******************/
 gint write_pdb(gchar *filename, struct model_pak *model)
 {
 gdouble c;
-gdouble x[3];
 GSList *list, *list2;
 struct bond_pak *bond;
 struct core_pak *core, *core2;
-gint i = 0;
+gint i;
 FILE *fp;
 
 /* checks */
@@ -127,20 +152,12 @@ COLUMNS        DATA TYPE       FIELD         DEFINITION
 79 - 80        LString(2)      charge        Charge on the atom.
 */
 
+i=1;
 for (list=model->cores ; list ; list=g_slist_next(list))
   {
-  core = list->data;
-
-/* everything is cartesian after latmat mult */
-  ARR3SET(x, core->x);
-  vecmat(model->latmat, x);
-  i++;
-  fprintf(fp,"ATOM  %5d %-4s %-3s %1s %3d    %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%-2s\n",
-              i, core->atom_label, core->res_name, core->chain, core->res_no, x[0], x[1], x[2], core->sof, 0.0, "", elements[core->atom_code].symbol, "");
+  write_pdb_core(fp, i++, list->data, model);
   }
 
-
-
 /* CURRENT */
 /* connectivity */
 i=1;
@@ -406,7 +423,7 @@ while (read_pdb_block(fp, data) == 0)
 /* model setup */
 strcpy(data->filename, filename);
 g_free(data->basename);
-data->basename = strdup_basename(filename);
+data->basename = parse_strip(filename);
 
 data->num_frames = data->cur_frame = frame;
 data->cur_frame--;
