@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -25,35 +25,45 @@ The GNU GPL can also be found at http://www.gnu.org
 #include <string.h>
 #include <unistd.h>
 #include <math.h>
+#ifdef __APPLE__
+#include <OpenGL/gl.h>
+#else
 #include <GL/gl.h>
+#endif
 
 #include "gdis.h"
 #include "coords.h"
 #include "file.h"
 #include "matrix.h"
+#include "quaternion.h"
+#include "numeric.h"
 #include "morph.h"
 #include "opengl.h"
 #include "render.h"
 #include "select.h"
 #include "spatial.h"
 #include "interface.h"
+#include "colourlib.h"
 
 extern struct sysenv_pak sysenv;
 extern struct elem_pak elements[];
 
 /* global render parameters */
-gdouble acm[9];
 gdouble p2a;
 
 /***************************/
 /* Setup POVray parameters */
 /***************************/
-/* TODO - put this stuff in file_povray.c */
 void povray_hdr(FILE *fp, struct model_pak *data)
 {
-gdouble xvec, yvec, amb, cam[3], pos[3], colour[3];
+gdouble xvec, yvec, amb, pos[3], colour[3];
+gdouble x[3], o[3], v[3], e[3];
 GSList *list;
 struct light_pak *light;
+struct camera_pak *camera;
+
+g_assert(data != NULL);
+g_assert(data->camera != NULL);
 
 fprintf(fp,"#include \"colors.inc\" \n");
 fprintf(fp,"#include \"finish.inc\" \n");
@@ -66,65 +76,134 @@ fprintf(fp,"background { color rgb<%f,%f,%f0> }\n", sysenv.render.bg_colour[0],
                         sysenv.render.bg_colour[1], sysenv.render.bg_colour[2]);  
 
 /* pixel to angstrom conversion, with yet another magic number... */
-p2a = 0.565 * (gdouble) sysenv.render.width * data->scale / data->rmax;
+p2a = 0.565 * (gdouble) sysenv.render.width / data->rmax;
 
-/* setup camera position (z is just a value far enough away) */
-VEC3SET(cam, data->offset[0]/p2a, data->offset[1]/p2a, 4.0*sysenv.rsize*data->scale);
-
-/* width and height */
-xvec = yvec = 2.0*sysenv.rsize/data->scale;
-
-/* NEW - adjust to preserve aspect ratio at the desired image size */
+/* preserve model aspect ratio for the given image size */
+xvec = yvec = 2.0*sysenv.rsize;
 if (sysenv.render.width > sysenv.render.height)
   xvec *= sysenv.render.width/sysenv.render.height;
 if (sysenv.render.height > sysenv.render.width)
   yvec *= sysenv.render.height/sysenv.render.width;
 
-/* orientation */
-fprintf(fp,"camera { orthographic location <%f,%f,%f>\n", cam[0], cam[1], cam[2]);
-fprintf(fp,"    right <%f,0,0> up <0,%f,0>\n", xvec, yvec);
-fprintf(fp,"    look_at <%f,%f,0> }\n", data->offset[0]/p2a, data->offset[1]/p2a);
+/* compute camera position and orientation */
+camera = data->camera;
+ARR3SET(x, camera->x);
+ARR3SET(o, camera->o);
+ARR3SET(v, camera->v);
 
-/* attempt to mimic OpenGL lights */
+switch (camera->mode)
+  {
+  case FREE:
+    break;
+
+  default:
+  case LOCKED:
+    quat_rotate(x, camera->q);
+    quat_rotate(o, camera->q);
+    quat_rotate(v, camera->q);
+    break;
+  }
+/* convert viewing vector to a location */
+ARR3ADD(v, x);
+
+/* camera zoom */
+xvec *= camera->zoom;
+yvec *= camera->zoom;
+
+/* NEW - enable movies of left/right eye to be produced */
+if (sysenv.stereo)
+  {
+/* get axis for eye translation (view x up vector) */
+  crossprod(e, v, o);
+  normalize(e, 3);
+
+/* the old 2% rule ... */
+  VEC3MUL(e, 0.02 * sysenv.rsize); 
+
+/* default is left eye only */
+  if (sysenv.render.stereo_right)
+    {
+    ARR3ADD(x, e);
+    ARR3ADD(v, e);
+    }
+  else
+    {
+    ARR3SUB(x, e);
+    ARR3SUB(v, e);
+    }
+  }
+
+/* sky is the orientation vector */
+/* right and up give the perspective */
+if (camera->perspective)
+  {
+  fprintf(fp,"camera { location <%f,%f,%f>\n", x[0], x[1], x[2]);
+  fprintf(fp,"    sky <%f,%f,%f>\n", o[0], o[1], o[2]);
+  fprintf(fp,"    right <%f,0,0> up <%f,0,0>\n", xvec, yvec);
+  fprintf(fp,"    look_at <%f,%f,%f>\n", v[0], v[1], v[2]);
+  fprintf(fp,"    angle %f }\n", camera->fov);
+  }
+else
+  {
+  fprintf(fp,"camera { orthographic location <%f,%f,%f>\n", x[0], x[1], x[2]);
+  fprintf(fp,"    sky <%f,%f,%f>\n", o[0], o[1], o[2]);
+  fprintf(fp,"    right <%f,0,0> up <%f,0,0>\n", xvec, yvec);
+  fprintf(fp,"    look_at <%f,%f,%f> }\n", v[0], v[1], v[2]);
+  }
+
+/* create light sources */
 for (list=sysenv.render.light_list ; list ; list=g_slist_next(list))
   {
-  light = (struct light_pak *) list->data;
+  light = list->data;
   ARR3SET(pos, light->x);
 
-/* FIXME - I don't fully understand all the coordinate transforms */
-/* necessary to make these positions match up with the on screen axes */
+/* OpenGL -> POVRay axes */
+  pos[0] *= -1.0;
+  pos[1] *= -1.0;
+  pos[2] *= -1.0;
+
+  quat_rotate(pos, camera->q);
+
   switch (light->type)
     {
     case POSITIONAL:
-      vecmat(acm, pos);
-/* NB: y & z switch mimics gl_acm */
-      fprintf(fp,"light_source\n  {\n <%f,%f,%f>\n", pos[0], -pos[2], pos[1]);
+      fprintf(fp,"light_source\n  {\n <%f,%f,%f>\n", pos[0], pos[1], pos[2]);
       break;
 
     case DIRECTIONAL:
-      vecmat(acm, pos);
 /* move away far enough so the rays are ~ // */
-      VEC3MUL(pos, -100.0*data->rmax);
-/* NB: y & z switch mimics gl_acm */
-      fprintf(fp,"light_source\n  {\n <%f,%f,%f>\n", pos[0], -pos[2], pos[1]);
+      VEC3MUL(pos, 100.0*data->rmax);
+      fprintf(fp,"light_source\n  {\n <%f,%f,%f>\n", pos[0], pos[1], pos[2]);
       break;
 
     default:
       continue;
     }
 
+/* simulate OpenGL style lights */
   ARR3SET(colour, light->colour);
-/* NEW - hack to simulate OpenGL style lights */
   VEC3MUL(colour, light->specular);
 
   if (sysenv.render.shadowless)
     fprintf(fp,"  color rgb<%f,%f,%f> shadowless }\n", colour[0], colour[1], colour[2]);
   else
+    {
+/* old style lighting */
+/*
     fprintf(fp,"  color rgb<%f,%f,%f> }\n", colour[0], colour[1], colour[2]);
+*/
+    fprintf (fp,"color White\n");
+    fprintf (fp,"   area_light <5, 0, 0,>, <0, 0, 5>, 5, 5\n");
+    fprintf (fp,"   adaptive 1\n   jitter\n}\n");
+    }
   }
 
+/* fill-light to bring out the shadows */
+fprintf(fp,"light_source{<%f,%f,%f> color Gray80 shadowless}\n", pos[0], pos[1], pos[2]);
+
 /* morph is too dark with just the above, sky_sphere is *nice* */
-/* TODO - leave choice of colour eg white/grey/light blue to the user? */
+/* TODO - choice of colour eg white/grey/light blue */
+/* NB: white is a bit too bright (even with 0 ambience) */
 if (data->id == MORPH)
   {
   if (!sysenv.render.wire_surface && !sysenv.render.shadowless)
@@ -134,46 +213,116 @@ if (data->id == MORPH)
     }
   }
 
-/* white is a bit too bright (even with 0 ambience) */
-/*
-if (data->id == MORPH && !sysenv.render.wire_surface)
-  fprintf(fp,"sky_sphere { pigment {White}}\n");
-*/
-
 /* POVRay is a bit darker than OpenGL */
 amb = 20.0*sysenv.render.ambience;
 
-/* FIXME - adjustable gamma? */
 fprintf(fp,"global_settings { ambient_light rgb<%f, %f, %f> assumed_gamma 2.2}\n",amb,amb,amb);
 }
 
+/* TODO -rename these something sensible */
+/*********************/
+/* forward algorithm */
+/*********************/
+void povray_rgb_decode(gint n, gdouble *c)
+{
+c[0] = ((n >> 10) & 31)/31.0;
+c[1] = ((n >> 5) & 31)/31.0;
+c[2] = (n & 31)/31.0;
+}
+
+/**********************/
+/* backward algorithm */
+/**********************/
+gint povray_rgb_encode(gdouble *c)
+{
+gint n;
+
+n = nearest_int(c[2]*31);
+n += nearest_int(c[1]*31) << 5;
+n += nearest_int(c[0]*31) << 10;
+
+return(n);
+}
+
+/********************************************/
+/* write a set of pre-defined textures to   */
+/* a POV-Ray file. These textures describe  */
+/* 256 HSV colours for linearly mapping     */
+/* a property from red to blue              */
+/********************************************/
+void write_povray_colour_textures(FILE *povfile, struct model_pak *data, int colour_style)
+{
+int i;
+float hue, red,green,blue;
+  
+switch (colour_style)
+  {
+  case HSV:
+    for (i=0; i<360; i++)
+      {
+      hue = (float)i;
+      hsv2rgb(hue,1.0,1.0,&red,&green,&blue);
+      fprintf(povfile,"#declare HSV%d = texture{pigment{color rgbt<%.2f,%.2f,%.2f,%.1f>}",i,red,green,blue,1.0-sysenv.render.transmit);
+      fprintf(povfile," finish { Phong_Shiny } }\n");
+      }
+    break;
+
+/* FIXME - we only need to do this texture declaration for meshes (ie smooth tri spatials) */
+  case REDWHITEBLUE:
+    for (i=0 ; i<32768 ; i++)
+      {
+      gdouble c[3];
+
+      fprintf(povfile,"#declare RGB_%d = texture{pigment{color rgb ", i);
+      povray_rgb_decode(i, c);
+      fprintf(povfile," <%f,%f,%f> } finish { Phong_Shiny } }\n", c[0], c[1], c[2]);
+
+/* debug */
+/*
+{
+gint j;
+
+printf("[%d] : <%f,%f,%f> :", i, c[0], c[1], c[2]);
+j = povray_rgb_encode(c);
+printf("[%d]\n", j);
+}
+*/
+
+      }
+    break;
+  }
+}
+
 /****************************/
 /* make a POVRAY input file */
 /****************************/
 /* TODO - eliminate redundancy between this and the opengl code */
 /* TODO - put this stuff in file_povray.c */
+
+/* Note by JJM. Need to minimise unnecessary blankspaces in the POV-Ray file - it can get very big! */
+
 #define DEBUG_MKPOV 0
 gint write_povray(gchar *povfile, struct model_pak *data)
 {
 gint n, m, i, j;
-gint r, g, b, flag;
+gint r, g, b;
+gint style;				/* for selecting the colour style of iso-surfaces */
 gdouble rad, scale, len;
 gdouble x1, y1, z1, x2, y2, z2;
 gdouble xmin, ymin, zmin, xmax, ymax, zmax;
 gdouble rf, gf, bf;
 gdouble vec[3], vec1[3], vec2[3];
 gdouble v1[4], v2[4], v3[3], n1[3], n2[3], n3[3];
-gdouble ctrl[8][3], mat4[16], tmp4[16];
-GSList *list, *ilist, *plist, *rlist, *list1, *list2, *pipe_list[4];
+gdouble ctrl[8][3], mat4[16];
+gint colour_index[3];
+GSList *list, *ilist, *plist, *rlist, *pipe_list[4];
 struct core_pak *core1;
-struct plane_pak *plane;
 struct object_pak *object;
 struct ribbon_pak *ribbon;
 struct spatial_pak *spatial;
 struct pipe_pak *pipe;
 struct image_pak *image;
 struct vec_pak *p1, *p2, *p3;
-struct vertex_pak *vx1, *vx2;
 struct elem_pak elem;
 FILE *fp;
 
@@ -188,22 +337,25 @@ if (!fp)
   return(1);
   }
 
-/* setup axes conversion matrix */
-VEC3SET(&acm[0],-1.0, 0.0, 0.0);
-VEC3SET(&acm[3], 0.0,-1.0, 0.0);
-VEC3SET(&acm[6], 0.0, 0.0,-1.0);
-
 /* strcpy(povfile,models[model].filename); */
 /* file_extension -> .pov */
 
 /* setup */
+/* JJM temporary solution for picking the colour-style of surfaces */
+//style = HSV;
+style = REDWHITEBLUE;
+
 povray_hdr(fp,data);
+
+write_povray_colour_textures(fp,data,style);
+
 /* limits - for intelligent axes placement */
 xmin = ymin = zmin = 99999999.9;
 xmax = ymax = zmax = -99999999.9;
 /* pixel to coord scaling factor */
-scale =  data->scale * sysenv.subscale * 0.5 / data->rmax;
+scale = sysenv.subscale * 0.5 / data->rmax;
 
+/* FIXME - can we just sub in data->display_lattice??? */
 /* precalc matrix products */
 ARR3SET(&mat4[0], &data->latmat[0]);
 ARR3SET(&mat4[4], &data->latmat[3]);
@@ -215,15 +367,6 @@ mat4[7] = -v1[1];
 mat4[11] = -v1[2];
 VEC4SET(&mat4[12], 0.0, 0.0, 0.0, 1.0);
 
-ARR3SET(&tmp4[0], &data->rotmat[0]);
-ARR3SET(&tmp4[4], &data->rotmat[3]);
-ARR3SET(&tmp4[8], &data->rotmat[6]);
-tmp4[3] = 0.0;
-tmp4[7] = 0.0;
-tmp4[11] = 0.0;
-VEC4SET(&tmp4[12], 0.0, 0.0, 0.0, 1.0);
-mat4mat(tmp4, mat4);
-
 /* do atoms */ 
 #if DEBUG_MKPOV
 printf("Doing atoms...\n");
@@ -235,7 +378,7 @@ do
   if (plist)
     {
 /* image */
-    image = (struct image_pak *) plist->data;
+    image = plist->data;
     ARR3SET(vec, image->rx);
     plist = g_slist_next(plist);
     }
@@ -249,13 +392,12 @@ do
 /* only if same type */
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
-  core1 = (struct core_pak *) list->data;
+  core1 = list->data;
   if (core1->status & (DELETED | HIDDEN))
     continue;
 
   ARR3SET(vec1, core1->rx);
   ARR3ADD(vec1, vec);
-  vecmat(acm, vec1);
 
 /* get current colour */
     r = core1->colour[0];
@@ -283,8 +425,9 @@ for (list=data->cores ; list ; list=g_slist_next(list))
         else
           fprintf(fp, "}\n");
 
-        fprintf(fp,"  finish{phong %f phong_size %d }}}\n",
-                    sysenv.render.ahl_strength, (gint) sysenv.render.ahl_size); 
+		fprintf(fp, " finish{Phong_Shiny}}}\n");
+/*        fprintf(fp,"  finish{phong %f phong_size %d }}}\n",
+                    sysenv.render.ahl_strength, (gint) sysenv.render.ahl_size); */
 /*
         fprintf(fp,"  finish{specular 0.6 diffuse 0.8 }}}\n");
 */
@@ -304,8 +447,9 @@ for (list=data->cores ; list ; list=g_slist_next(list))
         else
           fprintf(fp, "}\n");
 
-        fprintf(fp,"  finish{phong %f phong_size %d }}}\n",
-                    sysenv.render.ahl_strength, (gint) sysenv.render.ahl_size); 
+		fprintf(fp, " finish{Phong_Shiny}}}\n");
+/*        fprintf(fp,"  finish{phong %f phong_size %d }}}\n",
+                    sysenv.render.ahl_strength, (gint) sysenv.render.ahl_size); */
 /*
         fprintf(fp,"  finish{specular 0.6 diffuse 0.8 }}}\n");
 */
@@ -327,7 +471,7 @@ for (i=0 ; i<4 ; i++)
   {
   for (list=pipe_list[i] ; list ; list=g_slist_next(list))
     {
-    pipe = (struct pipe_pak *) list->data;
+    pipe = list->data;
 
 /* original + image iteration */
     ilist = NULL;
@@ -338,7 +482,7 @@ for (i=0 ; i<4 ; i++)
       ARR3SET(v2, pipe->v2);
       if (ilist)
         {
-        image = (struct image_pak *) ilist->data;
+        image = ilist->data;
 /* image */
         ARR3ADD(v1, image->rx);
         ARR3ADD(v2, image->rx);
@@ -347,9 +491,6 @@ for (i=0 ; i<4 ; i++)
       else
         ilist = data->images;
 
-      vecmat(acm, v1);
-      vecmat(acm, v2);
-
 /* bond type */
       switch(i)
         {
@@ -359,8 +500,9 @@ for (i=0 ; i<4 ; i++)
                       v1[0],v1[1],v1[2],v2[0],v2[1],v2[2], sysenv.render.stick_rad);
           fprintf(fp,"open texture{pigment{color rgb<%f,%f,%f>}\n",
                       pipe->colour[0], pipe->colour[1], pipe->colour[2]);
-          fprintf(fp,"  finish{phong %f phong_size %d }}}\n",
-                      sysenv.render.ahl_strength, (gint) sysenv.render.ahl_size); 
+		  fprintf(fp, " finish{Phong_Shiny}}}\n");
+/*          fprintf(fp,"  finish{phong %f phong_size %d }}}\n",
+                      sysenv.render.ahl_strength, (gint) sysenv.render.ahl_size); */
           break;
 
 /* ghosts */
@@ -369,8 +511,9 @@ for (i=0 ; i<4 ; i++)
                       v1[0],v1[1],v1[2],v2[0],v2[1],v2[2], sysenv.render.stick_rad);
           fprintf(fp,"open texture{pigment{color rgb<%f,%f,%f> transmit 0.6}",
                       pipe->colour[0], pipe->colour[1], pipe->colour[2]);
-          fprintf(fp,"  finish{phong %f phong_size %d }}}\n",
-                      sysenv.render.ahl_strength, (gint) sysenv.render.ahl_size); 
+		  fprintf(fp, " finish{Phong_Shiny}}}\n");
+/*          fprintf(fp,"  finish{phong %f phong_size %d }}}\n",
+                      sysenv.render.ahl_strength, (gint) sysenv.render.ahl_size); */
           break;
 
 /* line */
@@ -380,8 +523,10 @@ for (i=0 ; i<4 ; i++)
                       v1[0],v1[1],v1[2],v2[0],v2[1],v2[2], 2.0/p2a);
           fprintf(fp,"open texture{pigment{color rgb<%f,%f,%f>}\n",
                       pipe->colour[0], pipe->colour[1], pipe->colour[2]);
+		  fprintf(fp, " finish{Phong_Shiny}}}\n");
+		  /*
           fprintf(fp,"  finish{phong %f phong_size %d }}}\n",
-                      sysenv.render.ahl_strength, (gint) sysenv.render.ahl_size); 
+                      sysenv.render.ahl_strength, (gint) sysenv.render.ahl_size); */
           break;
 
 /* TODO - wire frame bonds??? */
@@ -404,23 +549,29 @@ printf("Doing special objects...\n");
 /********/
 /* AXES */
 /********/
-/* FIXME - can't cope with model translation */
+/* FIXME - stuffs up under new camera stuff */
+if (0)
 if (data->show_axes)
   {
 /* origin */
   VEC3SET(vec1, 1.0, 1.0, 0.0);
-  VEC3MUL(vec1, data->rmax/data->scale);
+  VEC3MUL(vec1, data->rmax);
 /* cope with scaling */
-  vec1[2] -= 4.0*sysenv.rsize*data->scale;
+  vec1[2] -= 4.0*sysenv.rsize;
   for (j=0 ; j<3 ; j++)
     {
 /* get end point */
     ARR3SET(vec2, data->axes[j].rx);
-    VEC3MUL(vec2, 1.0/data->scale);
-    vecmat(acm, vec2);
     ARR3ADD(vec2, vec1);
 /* cope with scaling */
-    vec2[2] -= 4.0*sysenv.rsize*data->scale;
+    vec2[2] -= 4.0*sysenv.rsize;
+
+/* HACK for degenerate cylinders (povray spews on these) */
+ARR3SET(v3, vec1);
+ARR3SUB(v3, vec2);
+if (VEC3MAGSQ(v3) < 0.1)
+  continue;
+
 /* draw */
     fprintf(fp,"cylinder { <%f,%f,%f>,<%f,%f,%f>, %f\n",
                 vec1[0],vec1[1],vec1[2],vec2[0],vec2[1],vec2[2], 2.0/p2a);
@@ -442,22 +593,32 @@ if (data->show_cell && data->periodic)
     n = 4*(j/4);
 
     ARR3SET(vec, data->cell[m].rx);
-    vecmat(acm, vec);
     x1 = vec[0];
     y1 = vec[1];
     z1 = vec[2];
 
     ARR3SET(vec, data->cell[n].rx);
-    vecmat(acm, vec);
     x2 = vec[0];
     y2 = vec[1];
     z2 = vec[2];
+
+/* HACK for degenerate cylinders (povray spews on these) */
+ARR3SET(v3, data->cell[m].rx);
+ARR3SUB(v3, data->cell[n].rx);
+if (VEC3MAGSQ(v3) < 0.1)
+  continue;
+
     fprintf(fp,"cylinder { <%f,%f,%f>,<%f,%f,%f>, %f\n",
                                  x1,y1,z1,x2,y2,z2,rad);
     fprintf(fp," texture { pigment {White} } }\n");
 
+/* HACK for degenerate cylinders (povray spews on these) */
+ARR3SET(v3, data->cell[m].rx);
+ARR3SUB(v3, data->cell[n+2].rx);
+if (VEC3MAGSQ(v3) < 0.1)
+  continue;
+
     ARR3SET(vec, data->cell[n+2].rx);
-    vecmat(acm, vec);
     x2 = vec[0];
     y2 = vec[1];
     z2 = vec[2];
@@ -475,13 +636,11 @@ if (data->show_cell && data->periodic)
     for (j=0 ; j<4 ; j++)
       {
       ARR3SET(vec, data->cell[m].rx);
-      vecmat(acm, vec);
       x1 = vec[0];
       y1 = vec[1];
       z1 = vec[2];
 
       ARR3SET(vec, data->cell[n].rx);
-      vecmat(acm, vec);
       x2 = vec[0];
       y2 = vec[1];
       z2 = vec[2];
@@ -500,14 +659,13 @@ if (data->show_cell && data->periodic)
 /***********/
 for (list=data->ribbons ; list ; list=g_slist_next(list))
   {
-  object = (struct object_pak *) list->data;
+  object = list->data;
   g_assert(object->type == RIBBON);
 
   rlist = (GSList *) object->data;
   while (rlist)
     {
-
-    ribbon = (struct ribbon_pak *) rlist->data;
+    ribbon = rlist->data;
 
   fprintf(fp, "bicubic_patch {\n  type 1 flatness 0.001\n");
 /* NB: POVRay is very slow at rendering ribbons, so halve the quality */
@@ -566,14 +724,6 @@ for (list=data->ribbons ; list ; list=g_slist_next(list))
     ARR3SUB(&ctrl[6][0], vec2);
     ARR3SUB(&ctrl[7][0], vec2);
 
-    for (i=0 ; i<8 ; i++)
-      {
-      vecmat(acm, &ctrl[i][0]);
-/*
-      VEC3MUL(&ctrl[i][0], data->scale);
-*/
-      }
-
     fprintf(fp, "<%f, %f, %f>\n", ctrl[0][0], ctrl[0][1], ctrl[0][2]);
     for (i=1 ; i<4 ; i++)
       fprintf(fp, ", <%f, %f, %f>\n", ctrl[i][0], ctrl[i][1], ctrl[i][2]);
@@ -595,19 +745,18 @@ for (list=data->ribbons ; list ; list=g_slist_next(list))
 /******************/
 /* spatial planes */
 /******************/
+/* FIXME - currently broken for morphology display */
 for (list=data->spatial ; list ; list=g_slist_next(list))
   {
-  spatial = (struct spatial_pak *) list->data;
+  spatial = list->data;
 
   switch(spatial->type)
     {
-/* special cases */
+/* special spatials */
     case SPATIAL_VECTOR:
       break;
-    case SPATIAL_PLANE:
-      break;
 
-/* general cases */
+/* generic spatials */
     default:
 /* enumerate periodic images */
       plist=NULL;
@@ -616,7 +765,7 @@ for (list=data->spatial ; list ; list=g_slist_next(list))
         if (plist)
           {
 /* image */
-          image = (struct image_pak *) plist->data;
+          image = plist->data;
           ARR3SET(vec2, image->rx);
           plist = g_slist_next(plist);
           }
@@ -627,7 +776,7 @@ for (list=data->spatial ; list ; list=g_slist_next(list))
           plist = data->images;
           }
 /* enumerate vertices */
-        rlist = (GSList *) spatial->data;
+        rlist = spatial->list;
         p1 = NULL;
 
 /* TODO - the best way of doing this would be through smooth_triangle() and mesh() */
@@ -638,59 +787,72 @@ for (list=data->spatial ; list ; list=g_slist_next(list))
         switch(spatial->method)
           {
           case GL_TRIANGLES:
-            g_assert(g_slist_length(spatial->data) == 3);
-
-            fprintf(fp, "triangle { \n");
+            fprintf(fp, "#declare Surface=mesh{\n");
             while (rlist)
               {
-              p1 = (struct vec_pak *) rlist->data;
-              ARR3SET(vec1, vec2);
-              ARR3ADD(vec1, p1->rx);
-              vecmat(acm, vec1);
-/* supply normal at the vertex as well */
-              ARR3SET(vec, p1->n);
-              vecmat(acm, vec);
+              fprintf(fp, "smooth_triangle{ \n");
+              for (i=3 ; i-- ; )
+                {
+                p1 = rlist->data;
+                ARR3SET(vec1, vec2);
+                ARR3ADD(vec1, p1->rx);
+/* can we supply normal at the vertex as well? - YES! */
+                ARR3SET(vec, p1->rn);
+                fprintf(fp, "<%f,%f,%f><%f,%f,%f>\n",
+                             vec1[0], vec1[1], vec1[2],
+                             vec[0], vec[1], vec[2]);
+                colour_index[i] = povray_rgb_encode(p1->colour);
+                rlist = g_slist_next(rlist);
+                }
+              fprintf(fp,"texture_list { ");
+              fprintf(fp,"RGB_%d RGB_%d RGB_%d ",
+                      colour_index[2],colour_index[1],colour_index[0]);
+              fprintf(fp,"}}\n");
+              
 
-              fprintf(fp, "< %f, %f, %f>  ", vec1[0], vec1[1], vec1[2]);
+/* CURRENT - attempting to fix up Josh's dodgy colour stuff :-) */
+ /* 
+              switch (style)
+                {
+                case HSV:
+                  fprintf(fp,"texture_list{HSV%d HSV%d HSV%d}}\n",
+                              colour_index[2],colour_index[1],colour_index[0]);
+                  break;
 
-              rlist = g_slist_next(rlist);
-  
-              if (rlist)
-                fprintf(fp, ",\n");
-              else
-                fprintf(fp, "\n");
+                case REDWHITEBLUE:
+                  fprintf(fp,"texture_list { ");
+                  fprintf(fp,"RGB_%d RGB_%d RGB_%d ",
+                              colour_index[2],colour_index[1],colour_index[0]);
+                  fprintf(fp,"}}\n");
+                  break;
+                }
+  */
               }
+            fprintf(fp,"}\nobject { Surface translate <0,0,0> rotate <0,0,0> }\n");
             break;
 
           case GL_TRIANGLE_STRIP:
-            g_assert(g_slist_length(spatial->data) > 2);
 
-            p1 = (struct vec_pak *) rlist->data;
+            p1 = rlist->data; 
             ARR3SET(v1, vec2);
             ARR3ADD(v1, p1->rx);
-            vecmat(acm, v1);
-            ARR3SET(n1, p1->n);
-            vecmat(acm, n1);
+            ARR3SET(n1, p1->rn);
 
             rlist = g_slist_next(rlist);
 
-            p2 = (struct vec_pak *) rlist->data;
+            p2 = rlist->data;
             ARR3SET(v2, vec2);
             ARR3ADD(v2, p2->rx);
-            vecmat(acm, v2);
-            ARR3SET(n2, p2->n);
-            vecmat(acm, n2);
+            ARR3SET(n2, p2->rn);
 
             rlist = g_slist_next(rlist);
 
             while (rlist)
               {
-              p3 = (struct vec_pak *) rlist->data;
+              p3 = rlist->data;
               ARR3SET(v3, vec2);
               ARR3ADD(v3, p3->rx);
-              vecmat(acm, v3);
-              ARR3SET(n3, p3->n);
-              vecmat(acm, n3);
+              ARR3SET(n3, p3->rn);
 
               fprintf(fp, "triangle { \n");
 
@@ -705,8 +867,10 @@ for (list=data->spatial ; list ; list=g_slist_next(list))
                            1.0-sysenv.render.transmit);
 
 /* NB: POVRay highlight size has reverse sense. */
+			  fprintf(fp, " finish{Phong_Shiny}}}\n");
+			  /*
               fprintf(fp, " finish { phong %f phong_size %d } }\n",
-                           sysenv.render.shl_strength, (gint) sysenv.render.shl_size);
+                           sysenv.render.shl_strength, (gint) sysenv.render.shl_size);*/
 
               ARR3SET(v1, v2);
               ARR3SET(v2, v3);
@@ -722,10 +886,9 @@ for (list=data->spatial ; list ; list=g_slist_next(list))
             fprintf(fp, "polygon { %d\n", g_slist_length(rlist));
             while (rlist)
               {
-              p1 = (struct vec_pak *) rlist->data;
+              p1 = rlist->data;
               ARR3SET(vec1, vec2);
               ARR3ADD(vec1, p1->rx);
-              vecmat(acm, vec1);
               fprintf(fp, "< %f, %f, %f>", vec1[0], vec1[1], vec1[2]);
   
               rlist = g_slist_next(rlist);
@@ -735,146 +898,27 @@ for (list=data->spatial ; list ; list=g_slist_next(list))
               else
                 fprintf(fp, "\n");
               }
-          }
 
-/* FIXME - can we have individual colours at each vertex? */
-if (spatial->method != GL_TRIANGLE_STRIP)
-  {
-        g_assert(p1 != NULL);
+		/* FIXME - can we have individual colours at each vertex? */
         fprintf(fp, " texture { \n");
-        fprintf(fp, " pigment { color rgbt<%f, %f, %f, %f> } }\n",
+        fprintf(fp, " pigment { color rgbt<%f, %f, %f, %f> }\n",
                                                p1->colour[0],
                                                p1->colour[1],
                                                p1->colour[2],
                                       1.0-sysenv.render.transmit);
-
-/* NB: POVRay highlight size has reverse sense. */
+			/* NB: POVRay highlight size has reverse sense. */
+			fprintf(fp, " finish{Phong_Shiny}}}\n");
+		/*
         fprintf(fp, " finish { phong %f phong_size %d } }\n",
-          sysenv.render.shl_strength, (gint) sysenv.render.shl_size);
-  }
-
-/*
-if (spatial->method == GL_TRIANGLES)
-  fprintf(fp, "}\n");
-*/
-  
-/*
-        fprintf(fp, " finish { specular %d roughness %d } }\n",
-          sysenv.render.shl_strength, sysenv.render.shl_size);
-*/
+          sysenv.render.shl_strength, (gint) sysenv.render.shl_size); */
 
+          }
         }
       while (plist);
       break;
     }
   }
 
-/**************/
-/* MORPHOLOGY */
-/**************/
-if (data->num_vertices)
-  {
-/* morphology rendering style */
-  if (!sysenv.render.wire_surface)
-    {
-/* render via plane intersection */
-    fprintf(fp,"object\n{\nintersection{\n");
-
-    plist = data->planes;
-    while (plist != NULL)
-      {
-      plane = (struct plane_pak *) plist->data;
-      if (plane->present)
-        {
-/* rotate cartesian normals to match viewing angle */
-        ARR3SET(vec, plane->norm);
-        vecmat(data->rotmat, vec);
-/* apply axes conversion matrix */
-        vecmat(acm, vec);
-
-/* NEW - distance to facet */
-switch(data->morph_type)
-  {
-  case EQUIL_UN:
-    rad = plane->esurf[0];
-    break;
-  case GROWTH_UN:
-    rad = fabs(plane->eatt[0]);
-    break;
-  case EQUIL_RE:
-    rad = plane->esurf[1];
-    break;
-  case GROWTH_RE:
-    rad = fabs(plane->eatt[1]);
-    break;
-  case DHKL:
-  default:
-    rad = 1.0/plane->dhkl;
-    break;
-  }
-
-        fprintf(fp,"plane { <%f, %f, %f>,  %f }\n",vec[0],vec[1],vec[2], rad);
-        }
-      plist = g_slist_next(plist);
-      }
-/* crystal composition */
-    fprintf(fp,"} interior { ior %f } texture { finish { %s } "
-               " pigment { color rgbf <%f,%f,%f,%f> } }\n }\n", 
-               sysenv.render.ref_index,
-               sysenv.render.morph_finish,
-               sysenv.render.morph_colour[0],
-               sysenv.render.morph_colour[1],
-               sysenv.render.morph_colour[2],
-               1.0-sysenv.render.transmit);  
-    }
-  else
-    {
-    rad = sysenv.render.frame_thickness / p2a;
-    for (list1=data->vertices ; list1 ; list1=g_slist_next(list1))
-      {
-      vx1 = (struct vertex_pak *) list1->data;
-
-      for (list2=vx1->adj ; list2 ; list2=g_slist_next(list2))
-        {
-        vx2 = (struct vertex_pak *) list2->data;
-
-        if (!ridge_visible(vx1, vx2, data))
-          continue;
-
-        ARR3SET(vec, vx1->rx);
-        vecmat(acm, vec);
-        x1 = vec[0];
-        y1 = vec[1];
-        z1 = vec[2];
-
-        ARR3SET(vec, vx2->rx);
-        vecmat(acm, vec);
-        x2 = vec[0];
-        y2 = vec[1];
-        z2 = vec[2];
-
-        flag=0;
-        if (fabs(x2-x1) < 0.001)
-          flag++;
-        if (fabs(y2-y1) < 0.001)
-          flag++;
-        if (fabs(z2-z1) < 0.001)
-          flag++;
-        if (flag==3)
-          continue;
-
-        fprintf(fp,"cylinder { <%f,%f,%f>,<%f,%f,%f>,%f\n",
-                                  x1,y1,z1,x2,y2,z2,rad);
-        fprintf(fp," texture { pigment { color rgb<%f %f %f>} } }\n",
-                                          sysenv.render.morph_colour[0],
-                                          sysenv.render.morph_colour[1],
-                                          sysenv.render.morph_colour[2]);
-        }
-      }
-    }
-  }
-
-/* finished making the .pov file */
 fclose(fp);
 return(0);
 }
