@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -30,11 +30,14 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "file.h"
 #include "model.h"
 #include "parse.h"
+#include "render.h"
 #include "spatial.h"
 #include "matrix.h"
+#include "interface.h"
 
 /* main structures */
 extern struct sysenv_pak sysenv;
+extern struct elem_pak elements[];
 
 /* triggers */
 enum 
@@ -43,9 +46,7 @@ XML_INACTIVE,
 XML_SYSTEM, XML_SYMMETRY, XML_CRYSTAL, XML_CELL, XML_CORE, XML_SHELL,
 XML_CELL_A, XML_CELL_B, XML_CELL_C,
 XML_CELL_ALPHA, XML_CELL_BETA, XML_CELL_GAMMA,
-XML_SPATIAL, XML_VERTEX,
-XML_TRANSFORM, XML_MATRIX, XML_VECTOR, XML_SCALAR,
-XML_PICTURE
+XML_WAYPOINT, XML_SPATIAL, XML_VERTEX, XML_PICTURE
 };
 
 gint xml_context = XML_INACTIVE;
@@ -55,7 +56,6 @@ struct shel_pak *xml_shell=NULL;
 GHashTable *xml_system_table;
 
 
-
 /* TEMP - parse diffax sfc data */
 void parse_sfc(FILE *fp)
 {
@@ -174,19 +174,21 @@ while (*(names+i))
   {
   if (g_ascii_strncasecmp(*(names+i), "dictRef\0", 8) == 0)
     {
-/* CURRENT - only enter cell parsing context if it's the full cell's data */
     if (g_ascii_strncasecmp(*(values+i), "gulp:fullcell\0", 16) == 0)
       {
       xml_context = XML_CELL;
       xml_model->periodic = 3;
       }
-
     if (g_ascii_strncasecmp(*(values+i), "gulp:surface\0", 13) == 0)
       {
       xml_context = XML_CELL;
       xml_model->periodic = 2;
       }
-
+    if (g_ascii_strncasecmp(*(values+i), "gulp:polymer\0", 13) == 0)
+      {
+      xml_context = XML_CELL;
+      xml_model->periodic = 1;
+      }
     }
   i++;
   }
@@ -251,8 +253,9 @@ while (*(names+i))
   if (g_ascii_strncasecmp(*(names+i), "elementType\0", 12) == 0)
     {
 /* FIXME - potential overflow bug here, since label is an array */
-    strcpy(xml_core->label, *(values+i));
-    template_core(xml_core->label, xml_core, xml_model);
+    g_free(xml_core->atom_label);
+    xml_core->atom_label = g_strdup(*(values+i));     
+    core_init(xml_core->atom_label, xml_core, xml_model);
     }
   if (g_ascii_strncasecmp(*(names+i), "xyzf", 4) == 0)
     {
@@ -308,7 +311,7 @@ g_assert(xml_core != NULL);
 /* init structure */
 if (!xml_shell)
   {
-  xml_shell = new_shell(xml_core->label, xml_model);
+  xml_shell = new_shell(xml_core->atom_label, xml_model);
   xml_model->shels = g_slist_prepend(xml_model->shels, xml_shell);
   }
 
@@ -366,27 +369,86 @@ struct spatial_pak *xml_spatial;
 /************************/
 void xml_parse_spatial(const gchar **names, const gchar **values)
 {
-gint i;
+gint i, periodic=-1;
 
 g_assert(xml_model != NULL);
 
-xml_spatial = g_malloc(sizeof(struct spatial_pak));
-xml_spatial->type = -1;
-xml_spatial->method = -1;
-xml_spatial->data = NULL;
-xml_model->spatial = g_slist_prepend(xml_model->spatial, xml_spatial);
-
 i=0;
 while (*(names+i))
   {
-  if (g_ascii_strncasecmp(*(names+i), "type", 4) == 0)
-    xml_spatial->type = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "periodic", 8) == 0)
+    periodic = str_to_float(*(values+i));
+/* TODO - more details */
+  i++;
+  }
 
-  if (g_ascii_strncasecmp(*(names+i), "method", 6) == 0)
-    xml_spatial->method = str_to_float(*(values+i));
+xml_spatial = spatial_new(NULL, SPATIAL_GENERIC, 0, periodic, xml_model);
+}
+
+/***************************/
+/* create camera waypoints */
+/***************************/
+void xml_parse_waypoint(const gchar **names, const gchar **values)
+{
+gint i;
+struct camera_pak *camera;
+
+g_assert(xml_model != NULL);
+
+camera = camera_new();
+
+i=0;
+while (*(names+i))
+  {
+  if (g_ascii_strncasecmp(*(names+i), "mode", 4) == 0)
+    camera->mode = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "perspective", 11) == 0)
+    camera->perspective = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "fov", 3) == 0)
+    camera->fov = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "zoom", 4) == 0)
+    camera->zoom = str_to_float(*(values+i));
+
+  if (g_ascii_strncasecmp(*(names+i), "x0", 2) == 0)
+    camera->x[0] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "x1", 2) == 0)
+    camera->x[1] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "x2", 2) == 0)
+    camera->x[2] = str_to_float(*(values+i));
+
+  if (g_ascii_strncasecmp(*(names+i), "o0", 2) == 0)
+    camera->o[0] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "o1", 2) == 0)
+    camera->o[1] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "o2", 2) == 0)
+    camera->o[2] = str_to_float(*(values+i));
+
+  if (g_ascii_strncasecmp(*(names+i), "v0", 2) == 0)
+    camera->v[0] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "v1", 2) == 0)
+    camera->v[1] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "v2", 2) == 0)
+    camera->v[2] = str_to_float(*(values+i));
+
+  if (g_ascii_strncasecmp(*(names+i), "e0", 2) == 0)
+    camera->e[0] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "e1", 2) == 0)
+    camera->e[1] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "e2", 2) == 0)
+    camera->e[2] = str_to_float(*(values+i));
+
+  if (g_ascii_strncasecmp(*(names+i), "q0", 2) == 0)
+    camera->q[0] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "q1", 2) == 0)
+    camera->q[1] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "q2", 2) == 0)
+    camera->q[2] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "q3", 2) == 0)
+    camera->q[3] = str_to_float(*(values+i));
 
   i++;
   }
+xml_model->waypoint_list = g_slist_prepend(xml_model->waypoint_list, camera);
 }
 
 /********************/
@@ -395,64 +457,40 @@ while (*(names+i))
 void xml_parse_vertex(const gchar **names, const gchar **values)
 {
 gint i=0;
-struct vec_pak *vec;
+gdouble x[3], n[3], c[3];
 
 g_assert(xml_spatial != NULL);
 
-vec = g_malloc(sizeof(struct vec_pak));
-xml_spatial->data = g_slist_prepend(xml_spatial->data, vec);
+VEC3SET(x, 0.0, 0.0, 0.0);
+VEC3SET(n, 0.0, 0.0, 0.0);
+VEC3SET(c, 0.0, 0.0, 0.0);
 
 while (*(names+i))
   {
   if (g_ascii_strncasecmp(*(names+i), "red", 3) == 0)
-    vec->colour[0] = str_to_float(*(values+i));
+    c[0] = str_to_float(*(values+i));
   if (g_ascii_strncasecmp(*(names+i), "green", 5) == 0)
-    vec->colour[1] = str_to_float(*(values+i));
+    c[1] = str_to_float(*(values+i));
   if (g_ascii_strncasecmp(*(names+i), "blue", 4) == 0)
-    vec->colour[2] = str_to_float(*(values+i));
+    c[2] = str_to_float(*(values+i));
 
   if (g_ascii_strncasecmp(*(names+i), "x3", 2) == 0)
-    vec->x[0] = str_to_float(*(values+i));
+    x[0] = str_to_float(*(values+i));
   if (g_ascii_strncasecmp(*(names+i), "y3", 2) == 0)
-    vec->x[1] = str_to_float(*(values+i));
+    x[1] = str_to_float(*(values+i));
   if (g_ascii_strncasecmp(*(names+i), "z3", 2) == 0)
-    vec->x[2] = str_to_float(*(values+i));
+    x[2] = str_to_float(*(values+i));
 
   if (g_ascii_strncasecmp(*(names+i), "nx", 2) == 0)
-    vec->n[0] = str_to_float(*(values+i));
+    n[0] = str_to_float(*(values+i));
   if (g_ascii_strncasecmp(*(names+i), "ny", 2) == 0)
-    vec->n[1] = str_to_float(*(values+i));
+    n[1] = str_to_float(*(values+i));
   if (g_ascii_strncasecmp(*(names+i), "nz", 2) == 0)
-    vec->n[2] = str_to_float(*(values+i));
-
-  i++;
-  }
-}
-
-/**************************/
-/* transformation parsing */
-/**************************/
-struct transform_pak *xml_transform=NULL;
-
-/*******************************/
-/* create a new transformation */
-/*******************************/
-void xml_parse_transform(const gchar **names, const gchar **values)
-{
-gint i=0;
-
-g_assert(xml_model != NULL);
-
-xml_transform = g_malloc(sizeof(struct transform_pak));
-xml_model->transform_list = g_slist_append(xml_model->transform_list, xml_transform);
-
-while (*(names+i))
-  {
-  if (g_ascii_strncasecmp(*(names+i), "id", 2) == 0)
-    xml_transform->id = str_to_float(*(values+i));
+    n[2] = str_to_float(*(values+i));
 
   i++;
   }
+spatial_vnorm_add(x, n, c, xml_spatial);
 }
 
 /****************/
@@ -464,9 +502,6 @@ void xml_parse_text(GMarkupParseContext *context,
                     gpointer user_data,
                     GError **error)
 {
-gint i;
-gchar **buff;
-
 switch (xml_context)
   {
 /* specific cell contexts */
@@ -505,38 +540,6 @@ switch (xml_context)
     xml_model->picture_list = g_slist_append(xml_model->picture_list,
                                          g_strstrip(g_strdup(text)));
     break;
-
-  case XML_MATRIX:
-/* FIXME - bit of a hack here */
-    if (xml_transform)
-      {
-      buff = g_strsplit(text, " ", 10);
-      for (i=0 ; i<9 ; i++)
-        xml_transform->matrix[i] = str_to_float(*(buff+i));
-      g_strfreev(buff);
-      }
-    break;
- 
-  case XML_VECTOR:
-/* FIXME - bit of a hack here */
-    if (xml_transform)
-      {
-      buff = g_strsplit(text, " ", 4);
-      for (i=0 ; i<3 ; i++)
-        xml_transform->vector[i] = str_to_float(*(buff+i));
-      g_strfreev(buff);
-      }
-    break;
-
-  case XML_SCALAR:
-/* FIXME - bit of a hack here */
-    if (xml_transform)
-      {
-      buff = g_strsplit(text, " ", 2);
-      xml_transform->scalar = str_to_float(*(buff+0));
-      g_strfreev(buff);
-      }
-    break;
   }
 }
 
@@ -552,7 +555,7 @@ void xml_start_element(GMarkupParseContext *context,
 {
 struct model_pak *model;
 
-model = (struct model_pak *) current_model;
+model = current_model;
 
 /* context switching control */
 if (g_ascii_strncasecmp(element_name, "system\0", 7) == 0)
@@ -571,14 +574,8 @@ if (g_ascii_strncasecmp(element_name, "vertex\0", 7) == 0)
   xml_context = XML_VERTEX;
 if (g_ascii_strncasecmp(element_name, "picture\0", 8) == 0)
   xml_context = XML_PICTURE;
-if (g_ascii_strncasecmp(element_name, "transform\0", 10) == 0)
-  xml_context = XML_TRANSFORM;
-if (g_ascii_strncasecmp(element_name, "matrix\0", 7) == 0)
-  xml_context = XML_MATRIX;
-if (g_ascii_strncasecmp(element_name, "vector\0", 7) == 0)
-  xml_context = XML_VECTOR;
-if (g_ascii_strncasecmp(element_name, "scalar\0", 7) == 0)
-  xml_context = XML_SCALAR;
+if (g_ascii_strncasecmp(element_name, "waypoint\0", 9) == 0)
+  xml_context = XML_WAYPOINT;
 
 /* context attribute parsing */
 switch (xml_context)
@@ -615,8 +612,8 @@ switch (xml_context)
     xml_parse_vertex(attribute_names, attribute_values);
     return;
 
-  case XML_TRANSFORM:
-    xml_parse_transform(attribute_names, attribute_values);
+  case XML_WAYPOINT:
+    xml_parse_waypoint(attribute_names, attribute_values);
     return;
   }
 }
@@ -632,6 +629,10 @@ void xml_end_element(GMarkupParseContext *context,
 /* reset parsing context and related data */
 if (g_ascii_strncasecmp(element_name, "system\0", 7) == 0)
   {
+/* end of a model - prep? */
+  if (xml_model->waypoint_list)
+    xml_model->waypoint_list = g_slist_reverse(xml_model->waypoint_list);
+
   xml_model = NULL;
   xml_context = XML_INACTIVE;
   }
@@ -652,25 +653,14 @@ if (g_ascii_strncasecmp(element_name, "particle\0", 5) == 0)
 if (g_ascii_strncasecmp(element_name, "spatial\0", 8) == 0)
   {
   xml_context = XML_INACTIVE;
-  xml_spatial->data = g_slist_reverse(xml_spatial->data);
+  xml_spatial->list = g_slist_reverse(xml_spatial->list);
   xml_spatial = NULL;
   }
 if (g_ascii_strncasecmp(element_name, "vertex\0", 7) == 0)
   xml_context = XML_SPATIAL;
 if (g_ascii_strncasecmp(element_name, "picture\0", 8) == 0)
   xml_context = XML_INACTIVE;
-if (g_ascii_strncasecmp(element_name, "transform\0", 10) == 0)
-  {
-  xml_model->num_frames++;
-  xml_context = XML_INACTIVE;
-  xml_transform = NULL;
-  }
-
-if (g_ascii_strncasecmp(element_name, "matrix\0", 7) == 0)
-  xml_context = XML_INACTIVE;
-if (g_ascii_strncasecmp(element_name, "vector\0", 7) == 0)
-  xml_context = XML_INACTIVE;
-if (g_ascii_strncasecmp(element_name, "scalar\0", 7) == 0)
+if (g_ascii_strncasecmp(element_name, "waypoint\0", 9) == 0)
   xml_context = XML_INACTIVE;
 }
 
@@ -725,7 +715,7 @@ return(0);
 #define DEBUG_READ_XML 0
 gint read_xml(gchar *filename, struct model_pak *model)
 {
-gint i, n, num_models=0;
+gint /*i, n,*/ num_models=0;
 FILE *fp;
 
 fp = fopen(filename, "rt");
@@ -743,22 +733,7 @@ fclose(fp);
 printf("read_xml() : found %d models.\n", num_models);
 #endif
 
-/*
-n = sysenv.num_models;
-for (i=0 ; i<num_models ; i++)
-  {
-  n--;
-  model = model_ptr(n, RECALL);
-
-#if DEBUG_READ_XML
-printf("read_xml() : prep'ing model %p (%d)\n", model, n);
-#endif
-
-  prep_model(model);
-  }
-*/
-
-prep_model(model);
+model_prep(model);
 
 return(0);
 }
@@ -768,13 +743,11 @@ return(0);
 /*****************************/
 gint write_xml(gchar *filename, struct model_pak *model)
 {
-gint i;
 gdouble vec[3];
 GSList *list, *list2;
 struct core_pak *core;
 struct vec_pak *v;
 struct spatial_pak *spatial;
-struct transform_pak *transform;
 FILE *fp;
 
 fp = fopen(filename, "wt");
@@ -786,9 +759,12 @@ fprintf(fp, "<system>\n");
 /* periodicity */
 if (model->periodic)
   {
-/* TODO - 1D */
   switch (model->periodic)
     {
+    case 1:
+      fprintf(fp, "  <crystal dictRef=\"gulp:polymer\">\n");
+      break;
+
     case 2:
       fprintf(fp, "  <crystal dictRef=\"gulp:surface\">\n");
       break;
@@ -812,11 +788,18 @@ if (model->periodic)
 /* cores */
 for (list=model->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   ARR3SET(vec, core->x);
   vecmat(model->latmat, vec);
+/*
   fprintf(fp, "  <atom elementType=\"%s\" x3=\"%f\" y3=\"%f\" z3=\"%f\">",
-          core->label, vec[0], vec[1], vec[2]);
+          core->atom_label, vec[0], vec[1], vec[2]);
+*/
+
+  fprintf(fp, "  <atom elementType=\"%s\" x3=\"%f\" y3=\"%f\" z3=\"%f\">",
+          elements[core->atom_code].symbol, vec[0], vec[1], vec[2]);
+
+
   fprintf(fp, "</atom>\n");
 /* TODO - shells as particles */
   }
@@ -824,14 +807,14 @@ for (list=model->cores ; list ; list=g_slist_next(list))
 /* spatial data */
 for (list=model->spatial ; list ; list=g_slist_next(list))
   {
-  spatial = (struct spatial_pak *) list->data;
+  spatial = list->data;
 
-  fprintf(fp, "<spatial Type=\"%d\" Method=\"%d\">\n",
-                      spatial->type, spatial->method);
+  fprintf(fp, "<spatial Method=\"%d\" Periodic=\"%d\">\n",
+                      spatial->method, spatial->periodic);
 
-  for (list2=spatial->data ; list2 ; list2=g_slist_next(list2))
+  for (list2=spatial->list ; list2 ; list2=g_slist_next(list2))
     {
-    v = (struct vec_pak *) list2->data;
+    v = list2->data;
 
     fprintf(fp, "  <vertex red=\"%f\" green=\"%f\" blue=\"%f\"",
                  v->colour[0], v->colour[1], v->colour[2]);
@@ -846,36 +829,26 @@ for (list=model->spatial ; list ; list=g_slist_next(list))
   fprintf(fp, "</spatial>\n");
   }
 
-/* transform data */
-for (list=model->transform_list ; list ; list=g_slist_next(list))
+/* camera waypoints */
+for (list=model->waypoint_list ; list ; list=g_slist_next(list))
   {
-  transform = (struct transform_pak *) list->data;
+  struct camera_pak *camera = list->data;
 
-  fprintf(fp, "  <transform id=\"%d\">\n", transform->id);
-
-  fprintf(fp, "    <matrix>");
-  for (i=0 ; i<9 ; i++)
-    fprintf(fp, "%f ", transform->matrix[i]);
-  fprintf(fp, "</matrix>\n");
-
-  fprintf(fp, "    <vector>");
-  for (i=0 ; i<3 ; i++)
-    fprintf(fp, "%f ", transform->vector[i]);
-  fprintf(fp, "</vector>\n");
-
-  fprintf(fp, "    <scalar>");
-  fprintf(fp, "%f ", transform->scalar);
-  fprintf(fp, "</scalar>\n");
-  
-  fprintf(fp, "</transform>\n");
+  fprintf(fp, "<waypoint mode=\"%d\" perspective=\"%d\" fov=\"%f\" zoom=\"%f\"\n",
+              camera->mode, camera->perspective, camera->fov, camera->zoom);
+  fprintf(fp, "x0=\"%f\" x1=\"%f\" x2=\"%f\"\n",camera->x[0],camera->x[1],camera->x[2]);
+  fprintf(fp, "o0=\"%f\" o1=\"%f\" o2=\"%f\"\n",camera->o[0],camera->o[1],camera->o[2]);
+  fprintf(fp, "v0=\"%f\" v1=\"%f\" v2=\"%f\"\n",camera->v[0],camera->v[1],camera->v[2]);
+  fprintf(fp, "e0=\"%f\" e1=\"%f\" e2=\"%f\"\n",camera->e[0],camera->e[1],camera->e[2]);
+  fprintf(fp, "q0=\"%f\" q1=\"%f\" q2=\"%f\" q3=\"%f\">", 
+               camera->q[0], camera->q[1], camera->q[2], camera->q[3]);
+  fprintf(fp, "</waypoint>\n"); 
   }
 
-
 /* pictures */
 for (list=model->picture_list ; list ; list=g_slist_next(list))
   fprintf(fp, "<picture> %s </picture>\n", (gchar *) list->data);
 
-
 fprintf(fp, "</system>\n");
 
 fclose(fp);
