@@ -20,8 +20,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 The GNU GPL can also be found at http://www.gnu.org
 */
 #define JJM_DEBUG
-#define VERSION 0.89
+#define VERSION 0.90
 #define PATCH 0
+#define YEAR 2008
 
 /********************************************/
 /*** Debugging switches					 ****/
@@ -145,11 +146,11 @@ x+sysenv.subcenx[sub], y+sysenv.subceny[sub], w, h, a1, a2))
 enum 
 {
 E_SINGLE, E_OPTIMIZE, DOCK, FREE_ENERGY, MD, CONP, CONV,
-MOLE, MOLMEC, MOLQ, NOBUILD, NOFLAGS,
+MOLE, MOLMEC, MOLQ, NOBUILD, NOFLAGS, 
 NAME, CELL, CART, FRAC, SFRAC, PFRAC,
 SURFACE_CELL, SURFACE_VECTORS, POLYMER_CELL, POLYMER_VECTOR, LATTICE_VECTORS,
 SPACE, ORIGIN, SPECIES, MAXCYC, ENERGY, TOTAL_ENERGY, SBULK_ENERGY,
-GULP_POTENTIAL, GULP_SINGLE_LINE_POTENTIAL,
+GULP_POTENTIAL, GULP_SINGLE_LINE_POTENTIAL, GULP_NOAUTOBOND, GULP_CONNECT,
 SWITCH_OFF, SWITCH_ON, CYCLE, GNORM, BFGS_OPT, RFO_OPT, CONJ_OPT, UNIT_HESSIAN,
 TEMPERATURE, PRESSURE, ENSEMBLE,
 SHRINK, QEQ, ZSISA, COMPARE, NOSYM, FIX, PHONON, EIGEN,
@@ -167,7 +168,7 @@ NO_ESURF, NO_EATT, D_HKL,
 KEYWORD, 
 ELEMENT, SYMBOL, NUMBER, LABEL, WEIGHT, COVALENT, IONIC, VDW, CHARGE, COLOUR,
 GULP_VARIABLES,
-LABEL_NORMAL, LABEL_GHOST,
+LABEL_NORMAL, LABEL_GHOST, LABEL_FF,
 COORD_X, COORD_Y, COORD_Z,
 CORE_GROWTH_SLICE, CORE_REGION, CORE_TRANSLATE, CORE_FF,
 CIF_AUDIT, CIF_CREATION_DATE, CIF_DATA_START, CIF_LOOP_START,
