@@ -30,6 +30,10 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "matrix.h"
 #include "numeric.h"
 #include "opengl.h"
+#include "file.h"
+#include "parse.h"
+#include "dialog.h"
+#include "interface.h"
 #ifdef __APPLE__
 #include <OpenGL/gl.h>
 #else
@@ -554,3 +558,77 @@ fclose(fp);
 g_free(filename);
 }
 
+/*******************/
+/* graph importing */
+/*******************/
+#define GRAPH_SIZE_MAX 4
+void graph_read(gchar *filename)
+{
+gint i, j, n, num_tokens;
+gchar *fullpath, *line, **buff;
+gdouble xstart, xstop, x[GRAPH_SIZE_MAX], *y;
+GArray *garray;
+gpointer graph;
+struct model_pak *model;
+FILE *fp;
+
+/* TODO - close dialog */
+dialog_destroy_type(FILE_SELECT);
+
+/* read / create model / plot etc */
+model = sysenv.active_model;
+if (!model)
+  {
+  edit_model_create();
+  model = sysenv.active_model;
+  }
+
+garray = g_array_new(FALSE, FALSE, sizeof(gdouble));
+
+/* read */
+fullpath = g_build_filename(sysenv.cwd, filename, NULL);
+fp = fopen(fullpath, "rt");
+line = file_read_line(fp);
+n=0;
+while (line)
+  {
+  buff = tokenize(line, &num_tokens);
+  g_free(line);
+
+/* get all numbers */
+  j = 0;
+  for (i=0 ; i<num_tokens ; i++)
+    {
+    if (str_is_float(*(buff+i)))
+      {
+      if (j < GRAPH_SIZE_MAX)
+        x[j++] = str_to_float(*(buff+i));
+      }
+    }
+
+/* need x & y (2 items) minimum */
+  if (j > 1)
+    {
+    if (!n)
+      xstart = x[0];
+    g_array_append_val(garray, x[1]);
+    n++;
+    }
+
+  g_strfreev(buff);
+  line = file_read_line(fp);
+  }
+xstop = x[0];
+fclose(fp);
+
+y = g_malloc(n * sizeof(gdouble));
+for (i=0 ; i<n ; i++)
+  y[i] = g_array_index(garray, gdouble, i);
+
+graph = graph_new("Graph", model);
+graph_add_data(n, y, xstart, xstop, graph);
+
+g_array_free(garray, TRUE);
+g_free(y);
+}
+
