@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -28,8 +28,15 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "coords.h"
 #include "matrix.h"
 #include "numeric.h"
+#include "quaternion.h"
 #include "opengl.h"
+#include "render.h"
+#include "interface.h"
+#ifdef __APPLE__
+#include <OpenGL/gl.h>
+#else
 #include <GL/gl.h>
+#endif
 
 /* externals */
 extern struct sysenv_pak sysenv;
@@ -54,10 +61,15 @@ if (p->num_points)
 /* more efficient point generation */
 /***********************************/
 #define DEBUG_GLO_INIT_SPHERE 0
-void gl_init_sphere(struct point_pak *p)
+void gl_init_sphere(struct point_pak *p, struct model_pak *model)
 {
 gint i, j, n;
 gdouble layers, divisions, alpha, beta, phi, theta, sp, cp, st, ct;
+gdouble x[3], y[3], mat[9];
+struct camera_pak *camera;
+
+g_assert(model != NULL);
+g_assert(model->camera != NULL);
 
 /* number of layers - azimuthal sweep */
 layers = sysenv.render.sphere_quality+1.0;
@@ -76,9 +88,25 @@ p->x[2] = g_malloc(n * sizeof(gdouble));
 p->num_points = n;
 p->type = sysenv.render.sphere_quality;
 
-*(p->x[0]+0) = 0.0;
-*(p->x[1]+0) = 0.0;
-*(p->x[2]+0) = -1.0;
+/* TODO - make it use camera_init() as this is just the initial v->x vector */
+VEC3SET(x, 0.0, -1.0, 0.0);
+
+/* orient so hemisphere is facing the camera */
+camera = model->camera;
+if (camera->mode == LOCKED)
+  quat_rotate(x, camera->q);
+else
+  {
+/* in the case of a free camera - compute alignment transformation */
+  ARR3SET(y, camera->v);
+  VEC3MUL(y, -1.0);
+  matrix_v_alignment(mat, x, y);
+  vecmat(mat, x);
+  }
+
+*(p->x[0]+0) = x[0];
+*(p->x[1]+0) = x[1];
+*(p->x[2]+0) = x[2];
 
 n=1;
 
@@ -100,10 +128,19 @@ printf("[%d/%d]\n", i, (gint) layers);
     theta = j*beta;
     st = tbl_sin(theta);
     ct = tbl_cos(theta);
- 
-    *(p->x[0]+n) = ct*sp;
-    *(p->x[1]+n) = st*sp;
-    *(p->x[2]+n) = -cp;
+
+/* NEW - quat camera */
+    VEC3SET(x, st*sp, -cp, ct*sp);
+
+/* FIXME - FREE camera mode case */
+    if (camera->mode == LOCKED)
+      quat_rotate(x, camera->q);
+    else
+      vecmat(mat, x);
+
+    *(p->x[0]+n) = x[0]; 
+    *(p->x[1]+n) = x[1];
+    *(p->x[2]+n) = x[2];
     n++;
 
 #if DEBUG_GLO_INIT_SPHERE
@@ -114,21 +151,52 @@ printf("%7.4f %7.4f %7.4f\n", ct*sp, st*sp, -cp);
 g_assert(p->num_points == n);
 }
 
-/*********************************/
-/* more efficient sphere drawing */
-/*********************************/
-#define DEBUG_GLO_DRAW_SPHERE 0
-void gl_draw_sphere(struct point_pak *p, gdouble *x, gdouble rad)
+/**********************/
+/* draw half a sphere */
+/**********************/
+void gl_draw_hemisphere(struct point_pak *p, gdouble *x, gdouble rad)
 {
 gint i, j, div, m, n, mstart, nstart;
 gdouble vec[3];
 
 g_return_if_fail(p->num_points != 0);
 
-/* layer loop (phi) */
-n=1;
-mstart = 0;
-for (i=1 ; i<p->type+2 ; i++)
+/* top cap - split drawing to minimize vertices */
+glBegin(GL_TRIANGLE_FAN);
+vec[0] = *(p->x[0]);
+vec[1] = *(p->x[1]);
+vec[2] = *(p->x[2]);
+
+glNormal3dv(vec);
+VEC3MUL(vec, rad);
+
+ARR3ADD(vec, x);
+glVertex3dv(vec);
+for (i=6 ; i-- ; )
+  {
+  vec[0] = *(p->x[0]+i+1);
+  vec[1] = *(p->x[1]+i+1);
+  vec[2] = *(p->x[2]+i+1);
+
+  glNormal3dv(vec);
+  VEC3MUL(vec, rad);
+  ARR3ADD(vec, x);
+  glVertex3dv(vec);
+  }
+vec[0] = *(p->x[0]+6);
+vec[1] = *(p->x[1]+6);
+vec[2] = *(p->x[2]+6);
+
+glNormal3dv(vec);
+VEC3MUL(vec, rad);
+ARR3ADD(vec, x);
+glVertex3dv(vec);
+glEnd();
+
+/* remaining layers */
+n=7;
+mstart = 1;
+for (i=2 ; i<p->type+2 ; i++)
   {
   nstart = n;
   div = i*6.0;
@@ -150,6 +218,7 @@ printf("> : m,n = %d,%d\n", m, n);
     vec[0] = *(p->x[0]+n);
     vec[1] = *(p->x[1]+n);
     vec[2] = *(p->x[2]+n);
+
     glNormal3dv(vec);
     VEC3MUL(vec, rad);
     ARR3ADD(vec, x);
@@ -160,6 +229,7 @@ printf("> : m,n = %d,%d\n", m, n);
     vec[0] = *(p->x[0]+m);
     vec[1] = *(p->x[1]+m);
     vec[2] = *(p->x[2]+m);
+
     glNormal3dv(vec);
     VEC3MUL(vec, rad);
     ARR3ADD(vec, x);
@@ -181,6 +251,7 @@ printf("> : m,n = %d,%d\n", m, n);
   vec[0] = *(p->x[0]+nstart);
   vec[1] = *(p->x[1]+nstart);
   vec[2] = *(p->x[2]+nstart);
+
   glNormal3dv(vec);
   VEC3MUL(vec, rad);
   ARR3ADD(vec, x);
@@ -189,6 +260,7 @@ printf("> : m,n = %d,%d\n", m, n);
   vec[0] = *(p->x[0]+mstart);
   vec[1] = *(p->x[1]+mstart);
   vec[2] = *(p->x[2]+mstart);
+
   glNormal3dv(vec);
   VEC3MUL(vec, rad);
   ARR3ADD(vec, x);
@@ -204,45 +276,78 @@ printf("n : %d\n", n);
 #endif
 }
 
+/*********************************/
+/* more efficient sphere drawing */
+/*********************************/
+#define DEBUG_GLO_DRAW_SPHERE 0
+void gl_draw_sphere(struct point_pak *p, gdouble *x, gdouble rad)
+{
+gl_draw_hemisphere(p, x, rad);
+}
+
 /***************************/
 /* create digitized circle */
 /***************************/
-void gl_init_circle(struct point_pak *circle, gint segments)
+void gl_init_circle(struct point_pak *circle, gint segments, struct model_pak *model)
 {
 gint i;
 gdouble da, theta;
-gdouble vec[3];
+gdouble n[3], x[3], y[3], mat[9];
+struct camera_pak *camera;
 
+g_assert(model != NULL);
+g_assert(model->camera != NULL);
 g_return_if_fail(segments != 0);
 
+camera = model->camera;
+
 /* allocate for digitized circle */
 circle->x[0] = g_malloc((segments+1) * sizeof(gdouble));
 circle->x[1] = g_malloc((segments+1) * sizeof(gdouble));
 circle->x[2] = g_malloc((segments+1) * sizeof(gdouble));
 circle->num_points = segments + 1;
 
-/* first point (theta=0) */
-*(circle->x[0]) = 1.0;
-*(circle->x[1]) = 0.0;
-*(circle->x[2]) = 0.0;
-/* sweep */
-vec[2] = 0.0;
+/* first point */
+VEC3SET(x, 1.0, 0.0, 0.0);
+if (camera->mode == LOCKED)
+  quat_rotate(x, camera->q);
+else
+  {
+/* in the case of a free camera - compute alignment transformation */
+  VEC3SET(n, 0.0, -1.0, 0.0);
+  ARR3SET(y, camera->v);
+  matrix_v_alignment(mat, n, y);
+  vecmat(mat, x);
+  }
+
+*(circle->x[0]) = x[0];
+*(circle->x[1]) = x[1];
+*(circle->x[2]) = x[2];
+
+/* sweep in x,z */
 da = 2.0 * PI / (gdouble) segments;
 for (i=1 ; i<segments ; i++) 
   {
   theta = (gdouble) i * da;
 /* store point on rim (doubles as the normal) */
-  vec[0] = tbl_cos(theta);
-  vec[1] = tbl_sin(theta);
-  normalize(vec, 2);
-  *(circle->x[0]+i) = vec[0];
-  *(circle->x[1]+i) = vec[1];
-  *(circle->x[2]+i) = vec[2];
+  x[0] = tbl_cos(theta);
+  x[1] = 0.0;
+  x[2] = tbl_sin(theta);
+
+  if (camera->mode == LOCKED)
+    quat_rotate(x, camera->q);
+  else
+    vecmat(mat, x);
+
+  *(circle->x[0]+i) = x[0];
+  *(circle->x[1]+i) = x[1];
+  *(circle->x[2]+i) = x[2];
   }
+
 /* duplicate first point, so the circle is closed */
-*(circle->x[0]+segments) = 1.0;
-*(circle->x[1]+segments) = 0.0;
-*(circle->x[2]+segments) = 0.0;
+*(circle->x[0]+segments) = *(circle->x[0]);
+*(circle->x[1]+segments) = *(circle->x[1]);
+*(circle->x[2]+segments) = *(circle->x[2]);
 }
 
 /*******************************/
@@ -301,107 +406,11 @@ for (i=0 ; i<circle->num_points ; i++)
 glEnd();
 }
 
-/*********************************************/
-/* draw cylinder from a pre-digitized circle */
-/*********************************************/
-#define DEBUG_GL_DRAW_CYLINDER 0
-void gl_draw_cylinder(struct point_pak *circle, gdouble *pa, gdouble *pb, gdouble rad)
-{
-gint i, flag=0;
-gdouble len1, len2;
-gdouble p1[3], p2[3], p[3], vec[3], mat[9], mop[9];
-
-g_return_if_fail(circle->num_points != 0);
-
-/* force ordering (for correct quad normals) */
-if (pa[2] > pb[2])
-  {
-  ARR3SET(p1, pb);
-  ARR3SET(p2, pa);
-  }
-else
-  {
-  ARR3SET(p1, pa);
-  ARR3SET(p2, pb);
-  }
-
-/* get vector from point 1 to 2 */
-ARR3SET(vec, p2);
-ARR3SUB(vec, p1);
-
-/* normalize x */
-len1 = VEC3MAG(vec);
-vec[0] /= len1;
-/* normalize y,z (projection) */
-len2 = sqrt(vec[1]*vec[1] + vec[2]*vec[2]);
-if (len2 > FRACTION_TOLERANCE)
-  {
-  vec[1] /= len2;
-  vec[2] /= len2;
-  len2 /= len1;
-  }
-else
-  flag++;
-
-#if DEBUG_GL_DRAW_CYLINDER
-printf("--------------------\n");
-P3VEC("input: ", vec);
-#endif
-
-/* z axis coincidence */
-VEC3SET(&mat[0], len2, 0.0, vec[0]);
-VEC3SET(&mat[3], 0.0, 1.0, 0.0);
-VEC3SET(&mat[6], -vec[0], 0.0, len2);
-/* yz plane coincidence (if not already satisfied) */
-if (!flag)
-  {
-  VEC3SET(&mop[0], 1.0, 0.0, 0.0);
-  VEC3SET(&mop[3], 0.0, vec[2], vec[1]);
-  VEC3SET(&mop[6], 0.0, -vec[1], vec[2]);
-/* construct transformation matrix */
-  matmat(mop, mat);
-  }
-
-/* check */
-#if DEBUG_GL_DRAW_CYLINDER
-P3MAT("matrix: ", mat);
-VEC3SET(vec, 0.0, 0.0, 1.0);
-P3VEC("before: ", vec);
-vecmat(mat, vec);
-P3VEC("after: ", vec);
-printf("--------------------\n");
-#endif
-
-glBegin(GL_QUAD_STRIP);
-for (i=0 ; i<circle->num_points ; i++) 
-  {
-/* get point on rim (doubles as the normal) */
-  vec[0] = *(circle->x[0]+i);
-  vec[1] = *(circle->x[1]+i);
-  vec[2] = *(circle->x[2]+i);
-/* rotate the digitized circle */
-  vecmat(mat, vec);
-/* submit the normal */
-  glNormal3dv(vec);
-/* get points at required radius */
-  VEC3MUL(vec, rad);
-/* point on disk 2 */
-  ARR3SET(p, p2);
-  ARR3ADD(p, vec);
-  glVertex3dv(p);
-/* point on disk 1 */
-  ARR3SET(p, p1);
-  ARR3ADD(p, vec);
-  glVertex3dv(p);
-  }
-glEnd();
-}
-
 /******************************************/
-/* draw a cylinder between points v1 & v2 */
+/* draw a cylinder between points va & vb */
 /******************************************/
 /* NB: used for drawing vectors */
-void draw_cylinder(gdouble *va, gdouble *vb, gdouble rad, guint segments)
+void gl_draw_cylinder(gdouble *va, gdouble *vb, gdouble rad, guint segments)
 {
 gint i;
 gdouble theta, st, ct;
@@ -425,22 +434,26 @@ else
 ARR3SET(v12, v2);
 ARR3SUB(v12, v1);
 
-/* create vectors p and q, co-planar with the cylinder's cross-sectional disk */
-ARR3SET(p, v12);
-if (p[0] == 0.0 && p[1] == 0.0)
-  p[0] += 1.0;
-else
-  p[2] += 1.0;
+/* make a guess at a vector with some orthogonal component to v12 */
+VEC3SET(p, 1.0, 1.0, 1.0);
 crossprod(q, p, v12);
+
+/* our guess was bad - so fix it */
+if (VEC3MAGSQ(q) < FRACTION_TOLERANCE)
+  {
+  VEC3SET(p, 0.0, 1.0, 0.0);
+  crossprod(q, p, v12);
+  }
 crossprod(p, v12, q);
-/* do the normalization outside the segment loop */
+
+/* p and q are orthonormal and in the plane of the cylinder's cross section */
 normalize(p, 3);
 normalize(q, 3);
 
 /* build the cylinder from rectangular segments */
 /* TODO - see if vertex arrays give any speedup here */
 glBegin(GL_QUAD_STRIP);
-for (i=0 ; i<=segments ; i++) 
+for (i=segments+1 ; i-- ; )
   {
 /* sweep out a circle */
   theta = (gdouble) i * 2.0 * PI / (gdouble) segments;
@@ -472,51 +485,40 @@ glEnd();
 void draw_cone(gdouble *va, gdouble *vb, gdouble base, gint segments)
 {
 gint i;
-gdouble theta, st, ct;
-gdouble v1[3], v2[3], v[3];
-gdouble p[3], q[3], v12[3];
+gdouble theta, st, ct, da;
+gdouble v[3], p[3], q[3], v12[3];
 
 g_return_if_fail(segments != 0);
 
-/* force ordering of v1 & v2 to get correct quad normals */
-if (va[2] > vb[2])
-  {
-  ARR3SET(v1, vb);
-  ARR3SET(v2, va);
-  }
-else
-  {
-  ARR3SET(v1, va);
-  ARR3SET(v2, vb);
-  }
+/* vector from va to vb */
+ARR3SET(v12, vb);
+ARR3SUB(v12, va);
 
-/* vector from v1 to v2 */
-ARR3SET(v12, v2);
-ARR3SUB(v12, v1);
+/* create normal vectors p and q, co-planar with the cylinder's cross-sectional disk */
 
-/* create vectors p and q, co-planar with the cylinder's cross-sectional disk */
-ARR3SET(p, v12);
-if (p[0] == 0.0 && p[1] == 0.0)
-  p[0] += 1.0;
-else
-  p[2] += 1.0;
+/* make a guess at a vector with some orthogonal component to v12 */
+VEC3SET(p, 1.0, 1.0, 1.0);
 crossprod(q, p, v12);
+
+/* if our guess was bad - fix it */
+if (VEC3MAGSQ(q) < FRACTION_TOLERANCE)
+  {
+  VEC3SET(p, 0.0, 1.0, 0.0);
+  crossprod(q, p, v12);
+  }
 crossprod(p, v12, q);
 
 normalize(p, 3);
 normalize(q, 3);
 
-/* build the cone from triangular segments */
+/* build the cone */
+da = 2.0 * PI / (gdouble) segments;
 glBegin(GL_TRIANGLE_FAN);
-
-/* tip */
 glVertex3dv(vb);
-
-/* base */
-for (i=0 ; i<=segments ; i++) 
+for (i=segments+1 ; i-- ; )
   {
 /* sweep out a circle */
-  theta = (gdouble) i * 2.0 * PI / (gdouble) segments;
+  theta = (gdouble) i * da;
   st = tbl_sin(theta);
   ct = tbl_cos(theta);
 /* construct normal */
@@ -541,7 +543,7 @@ void draw_vector(gdouble *va, gdouble *vb, gdouble size)
 {
 gdouble vec[3];
 
-draw_cylinder(va, vb, size, 5);
+gl_draw_cylinder(va, vb, size, 5);
 
 ARR3SET(vec, vb);
 ARR3SUB(vec, va);
