@@ -84,16 +84,6 @@ glClearColor(sysenv.render.bg_colour[0], sysenv.render.bg_colour[1],
              sysenv.render.bg_colour[2], 1.0);
 glClearStencil(0x4);
 
-
-/* HACK - force perspective for windowed mode */
-if (!sysenv.stereo_fullscreen && camera)
-  {
-/*
-  camera->perspective = TRUE;
-*/
-  }
-
-
 #if DEBUG_STEREO_REDRAW
 printf(" --- STEREO ---\n");
 printf("x+y = %d + %d\n", stereo_x, stereo_y);
@@ -107,7 +97,6 @@ printf(" r  = %f\n", sysenv.rsize);
 printf("eye = %f\n", eye);
 #endif
 
-
 #ifdef __sgi
 /* TOP/BOTTOM viewports */
 x1 = SGI_STEREO_X;
@@ -119,9 +108,8 @@ height = SGI_STEREO_HEIGHT;
 #else
 if (sysenv.stereo_fullscreen && !sysenv.render.stereo_quadbuffer)
   {
-
 /* fullscreen with no quad buffer -> assume dual head */
-/* CURRENT - this mode seems broken */
+
   dim = stereo_width/2 < stereo_height ? stereo_width/2 : stereo_height;
 
   x1 = (stereo_width/2 - dim)/2;
@@ -132,18 +120,6 @@ if (sysenv.stereo_fullscreen && !sysenv.render.stereo_quadbuffer)
 
   width = dim;
   height = dim;
-
-/*
-  x1 = stereo_x;
-  y1 = stereo_y;
-
-  x2 = stereo_width/2 + x1;
-  y2 = stereo_y;
-
-  width = stereo_width/2.0;
-  height = stereo_height;
-*/
-
   }
 else
   {
@@ -196,21 +172,15 @@ off = 0.01 * sysenv.render.stereo_parallax * dist;
 /* large eye sep - small parallax -> +ve para -> everything BEHIND */
 /* small eye sep - large parallax -> -ve papa -> some stuff in FRONT */
 
-
-/* always clear the canvas first (even if we're not displaying anything */
-glDrawBuffer(GL_BACK_LEFT);
+/* always clear the canvas first (eliminates left over images if left/right eyes are turned off) */
+glDrawBuffer(GL_BACK);
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
 
 if (sysenv.render.stereo_left)
   {
-if (model)
-  {
-/* is this offset needed? */
-/*
-  glViewport(x1, y1, width, height);
-*/
-
-  gl_init_projection(&left, model);
+  if (model)
+    {
+    gl_init_projection(&left, model);
 
 /* CURRENT - with the new camera code things have been messed up a bit */
 /* easy solution - just stick to perspective view in the standard */
@@ -237,35 +207,26 @@ if (sysenv.render.stereo_use_frustum)
   glTranslatef(0.0, 0.0, -sysenv.rsize*0.5);
   }
 
+// CURRENT - reverse eye
+  glTranslatef(+eye, 0.0, 0.0);
 
-  glTranslatef(-eye, 0.0, 0.0);
+  glDrawBuffer(GL_BACK_LEFT);
+  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
   draw_objs(NULL, model);
   }
-glFlush();
   }
 
 /* RIGHT */
 if (sysenv.render.stereo_right)
   {
 
-/* only draw to right buffer if it exists, otherwise stay with same (left) one */
-if (sysenv.render.stereo_quadbuffer)
-  {
-  glDrawBuffer(GL_BACK_RIGHT);
-  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
-  }
-
 if (model)
   {
-/* is this offset needed? */
-/*
-  glViewport(x2, y2, width, height);
-*/
+/* only draw to right buffer if it exists, otherwise stay with same (left) one */
 
   gl_init_projection(&right, model);
 
-
 if (sysenv.render.stereo_use_frustum)
   {
   gdouble r;
@@ -287,14 +248,19 @@ if (sysenv.render.stereo_use_frustum)
   glTranslatef(0.0, 0.0, -sysenv.rsize*0.5);
   }
 
+// CURRENT - reverse eye
+  glTranslatef(-eye, 0.0, 0.0);
 
-  glTranslatef(+eye, 0.0, 0.0);
+  if (sysenv.render.stereo_quadbuffer)
+    glDrawBuffer(GL_BACK_RIGHT);
+
+  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
   draw_objs(NULL, model);
   }
-glFlush();
   }
 
+
 /* NEW - store matrices for proj/unproj operations */
 /* FIXME - doesn't seem to work */
 /*
@@ -348,19 +314,23 @@ stereo_expose_event(stereo_glarea, NULL);
 }
 
 /*****************************/
-/* init windowed stereo mode */
+/* init WINDOWED stereo mode */
 /*****************************/
 void stereo_init_window(struct canvas_pak *canvas)
 {
+if (sysenv.stereo_fullscreen)
+  return;
+
 stereo_x = canvas->x;
 stereo_y = canvas->y;
 stereo_width = canvas->width;
 stereo_height = canvas->height;
 }
 
-/********************/
-/* init stereo mode */
-/********************/
+/*******************************/
+/* init FULLSCREEN stereo mode */
+/*******************************/
+#define DEBUG_FULLSCREEN 0
 void stereo_open_window(void)
 {
 GdkWindow *win;
@@ -369,8 +339,6 @@ GdkGLConfig *glconfig=NULL;
 if (!sysenv.stereo_fullscreen)
   return;
 
-printf("fs mode\n");
-
 #ifdef __sgi
 if (system("/usr/gfx/setmon -n STR_RECT") != 0)
   {
@@ -405,8 +373,6 @@ if (!glconfig)
   return;
   }
 
-
-
 /* get root window dimensions so we can create a fullscreen window */
 win = gdk_get_default_root_window();
 gdk_window_get_geometry(win, &stereo_x, &stereo_y,
@@ -415,7 +381,11 @@ gdk_window_get_geometry(win, &stereo_x, &stereo_y,
 stereo_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 gtk_window_set_decorated(GTK_WINDOW(stereo_window), FALSE);
 
-gtk_window_set_default_size(GTK_WINDOW(stereo_window), stereo_width, stereo_height);
+#if DEBUG_FULLSCREEN
+printf("canvas x,y = %d,%d : w,h = %d,%d\n", stereo_x, stereo_y, stereo_width, stereo_height);
+#endif
+
+gtk_window_set_default_size(GTK_WINDOW(stereo_window), stereo_width-stereo_x, stereo_height-stereo_y);
 
 g_signal_connect(GTK_OBJECT(stereo_window), "key_press_event",
                 (GtkSignalFunc) cb_key_press, NULL);
