@@ -32,14 +32,11 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "analysis.h"
 #include "dialog.h"
 #include "interface.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 
 extern struct elem_pak elements[];
 extern struct sysenv_pak sysenv;
 
-gint do_rdf=FALSE, do_meas=FALSE, do_vacf=FALSE, do_temp=FALSE, do_ke=FALSE, do_pe=FALSE;
-GtkWidget *rdf_vbox;
-
 /******************************/
 /* export graphs as text data */
 /******************************/
@@ -66,6 +63,7 @@ else
 /******************************/
 void analysis_export_dialog(void)
 {
+/* FIXME - dialog + model are linked */
 if (sysenv.active_model)
   file_dialog("Export graph as text", NULL, FILE_SAVE, 
               (gpointer) analysis_export, MD_ANALYSIS);
@@ -74,25 +72,60 @@ if (sysenv.active_model)
 /**********************/
 /* submit an rdf task */
 /**********************/
-void exec_analysis_task(GtkWidget *w, struct model_pak *model)
+void exec_analysis_task(GtkWidget *w, gpointer dialog)
 {
-if (do_rdf)
-  task_new("RDF", &analysis_plot_rdf, model, &analysis_show, model, model);
+const gchar *type;
+gpointer gui_menu_calc;
+struct analysis_pak *analysis;
+struct model_pak *model;
 
-if (do_meas)
-  task_new("Measure", &analysis_plot_meas, model, &analysis_show, model, model);
+/* duplicate the analysis structure at time of request */
+g_assert(dialog != NULL);
+model = dialog_model(dialog);
+g_assert(model != NULL);
+analysis = analysis_dup(model->analysis);
 
-if (do_vacf)
-  task_new("VACF", &analysis_plot_vacf, model, &analysis_show, model, model);
+/* get the task type requested from the dialog */
+gui_menu_calc = dialog_child_get(dialog, "gui_menu_calc");
+type = gui_pulldown_text(gui_menu_calc);
 
-if (do_temp)
-  task_new("Temp", &analysis_plot_temp, model, &analysis_show, model, model);
-
-if (do_ke)
-  task_new("Energy", &analysis_plot_ke, model, &analysis_show, model, model);
-
-if (do_pe)
-  task_new("Energy", &analysis_plot_pe, model, &analysis_show, model, model);
+if (g_ascii_strncasecmp(type, "RDF", 3) == 0)
+  {
+  analysis->rdf_normalize = TRUE;
+  task_new("RDF", &analysis_plot_rdf, analysis, &analysis_show, model, model);
+  return;
+  }
+if (g_ascii_strncasecmp(type, "Pair", 4) == 0)
+  {
+  analysis->rdf_normalize = FALSE;
+  task_new("Pair", &analysis_plot_rdf, analysis, &analysis_show, model, model);
+  return;
+  }
+if (g_ascii_strncasecmp(type, "VACF", 4) == 0)
+  {
+  task_new("VACF", &analysis_plot_vacf, analysis, &analysis_show, model, model);
+  return;
+  }
+if (g_ascii_strncasecmp(type, "Temp", 4) == 0)
+  {
+  task_new("Temp", &analysis_plot_temp, analysis, &analysis_show, model, model);
+  return;
+  }
+if (g_ascii_strncasecmp(type, "Pot", 3) == 0)
+  {
+  task_new("Energy", &analysis_plot_pe, analysis, &analysis_show, model, model);
+  return;
+  }
+if (g_ascii_strncasecmp(type, "Kin", 3) == 0)
+  {
+  task_new("Energy", &analysis_plot_ke, analysis, &analysis_show, model, model);
+  return;
+  }
+if (g_ascii_strncasecmp(type, "Meas", 4) == 0)
+  {
+  task_new("Measure", &analysis_plot_meas, analysis, &analysis_show, model, model);
+  return;
+  }
 }
 
 /**********************/
@@ -109,24 +142,6 @@ if (*atom)
 *atom = g_strdup(gtk_entry_get_text(GTK_ENTRY(w)));
 }
 
-/************************/
-/* set sensitive states */
-/************************/
-void gui_analysis_refresh(GtkWidget *w)
-{
-if (do_rdf)
-  gtk_widget_set_sensitive(rdf_vbox, TRUE);
-else
-  gtk_widget_set_sensitive(rdf_vbox, FALSE);
-
-/*
-if (do_vacf)
-  gtk_widget_set_sensitive(vacf_vbox, TRUE);
-else
-  gtk_widget_set_sensitive(vacf_vbox, FALSE);
-*/
-}
-
 /*******************************/
 /* multi-frame analysis dialog */
 /*******************************/
@@ -135,8 +150,9 @@ void gui_analysis_dialog(void)
 gpointer dialog;
 GtkWidget *window, *combo, *label;
 GtkWidget *frame, *main_hbox, *main_vbox, *hbox, *vbox;
+GtkWidget *gui_menu_calc, *gui_vbox_rdf;
 GSList *list, *item;
-GList *match_list=NULL;
+GList *match_list=NULL, *calc_list=NULL;
 struct model_pak *model;
 struct analysis_pak *analysis;
 
@@ -149,7 +165,7 @@ if (!model->animation)
 if (analysis_init(model))
   return;
 
-analysis = (struct analysis_pak *) model->analysis;
+analysis = model->analysis;
 
 /* create new dialog */
 dialog = dialog_request(MD_ANALYSIS, "MD analysis", NULL, NULL, model);
@@ -165,65 +181,60 @@ gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), main_hbox, TRUE, TRUE, 0);
 main_vbox = gtk_vbox_new(FALSE, 0);
 gtk_box_pack_start(GTK_BOX(main_hbox), main_vbox, TRUE, TRUE, 0);
 
-/* --- calculation setup */
+/* --- analysis dialogs are specific to the active model when initiated */
+frame = gtk_frame_new("Model");
+gtk_box_pack_start(GTK_BOX(main_vbox), frame, FALSE, FALSE, 0);
+gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+hbox = gtk_hbox_new(TRUE, PANEL_SPACING);
+gtk_container_add(GTK_CONTAINER(frame), hbox);
+label = gtk_label_new(model->basename);
+gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+
+/* --- calculation menu */
 frame = gtk_frame_new("Calculate");
 gtk_box_pack_start(GTK_BOX(main_vbox), frame, FALSE, FALSE, 0);
 gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
 vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
 
-gui_direct_check("RDF", &do_rdf, gui_analysis_refresh, NULL, vbox);
-
-/* --- time plot setup */
-frame = gtk_frame_new("Time Plots");
-gtk_box_pack_start(GTK_BOX(main_vbox), frame, FALSE, FALSE, 0);
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-
+/* --- menu items */
 if (g_file_test(model->gulp.trj_file, G_FILE_TEST_EXISTS))
   {
-  gui_direct_check("VACF/power spectrum", &do_vacf, gui_analysis_refresh, NULL, vbox);
-
-/* temp */
-  gui_direct_check("Temperature", &do_temp, gui_analysis_refresh, NULL, vbox);
-/* ke & pe */
-  gui_direct_check("Kinetic E", &do_ke, gui_analysis_refresh, NULL, vbox);
-  gui_direct_check("Potential E", &do_pe, gui_analysis_refresh, NULL, vbox);
+  calc_list = g_list_prepend(calc_list, "Potential E");
+  calc_list = g_list_prepend(calc_list, "Kinetic E");
+  calc_list = g_list_prepend(calc_list, "Temperature");
+  calc_list = g_list_prepend(calc_list, "VACF");
   }
 else
   {
-  hbox = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-  label = gtk_label_new("VACF: no trajectory file.");
-  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-/* CURRENT */
-  gui_direct_check("Measurements", &do_meas, gui_analysis_refresh, NULL, vbox);
+  calc_list = g_list_prepend(calc_list, "Measurements");
   }
+calc_list = g_list_prepend(calc_list, "RDF");
+calc_list = g_list_prepend(calc_list, "Pair count");
+gui_menu_calc = gui_pulldown_new("Perform ", calc_list, FALSE, vbox);
+dialog_child_set(dialog, "gui_menu_calc", gui_menu_calc);
 
 /* --- right pane */
-rdf_vbox = gtk_vbox_new(FALSE, 0);
-gtk_box_pack_start(GTK_BOX(main_hbox), rdf_vbox, TRUE, TRUE, 0);
+gui_vbox_rdf = gtk_vbox_new(FALSE, 0);
+dialog_child_set(dialog, "gui_vbox_rdf", gui_vbox_rdf);
+gtk_box_pack_start(GTK_BOX(main_hbox), gui_vbox_rdf, TRUE, TRUE, 0);
 
 /* --- interval setup */
-frame = gtk_frame_new("RDF Interval");
-gtk_box_pack_start(GTK_BOX(rdf_vbox), frame, FALSE, FALSE, 0);
+frame = gtk_frame_new("Analysis Interval");
+gtk_box_pack_start(GTK_BOX(gui_vbox_rdf), frame, FALSE, FALSE, 0);
 gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
 vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
 
 /* TODO - some warning about the RDF being valid only for < L/2? */
 /* see Allan & Tildesley pg 199 */
-gui_direct_spin("Start", &analysis->start, 0.0, model->rmax, 0.1, NULL, NULL, vbox);
-gui_direct_spin("Stop", &analysis->stop, 0.1, model->rmax, 0.1, NULL, NULL, vbox);
-/*
+gui_direct_spin("Start", &analysis->start, 0.0, 10.0*model->rmax, 0.1, NULL, NULL, vbox);
+gui_direct_spin("Stop", &analysis->stop, 0.1, 10.0*model->rmax, 0.1, NULL, NULL, vbox);
 gui_direct_spin("Step", &analysis->step, 0.1, model->rmax, 0.1, NULL, NULL, vbox);
-*/
 
 /* --- RDF atom setup */
-frame = gtk_frame_new("RDF Atoms");
-gtk_box_pack_start(GTK_BOX(rdf_vbox), frame, FALSE, FALSE, 0);
+frame = gtk_frame_new("Analysis Atoms");
+gtk_box_pack_start(GTK_BOX(gui_vbox_rdf), frame, FALSE, FALSE, 0);
 gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
 vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
@@ -254,15 +265,12 @@ g_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry), "changed",
                  GTK_SIGNAL_FUNC(md_atom_changed), (gpointer) &analysis->atom2);
 
 /* terminating buttons */
-gui_stock_button(GTK_STOCK_EXECUTE, exec_analysis_task, model,
+gui_stock_button(GTK_STOCK_EXECUTE, exec_analysis_task, dialog,
                    GTK_DIALOG(window)->action_area);
 
 gui_stock_button(GTK_STOCK_CLOSE, dialog_destroy, dialog,
                    GTK_DIALOG(window)->action_area);
 
 gtk_widget_show_all(window);
-
-/* init widget sensitive states */
-gui_analysis_refresh(NULL);
 }
 
