@@ -38,7 +38,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "measure.h"
 #include "numeric.h"
 #include "opengl.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
 
@@ -120,34 +120,6 @@ g_assert(model->animation == TRUE);
 return(1);
 }
 
-/*************************************************/
-/* store the current position of the file stream */
-/*************************************************/
-#define DEBUG_ADD_FRAME_OFFSET 0
-gint add_frame_offset(FILE *fp, struct model_pak *data)
-{
-fpos_t *offset;
-
-g_assert(fp != NULL);
-g_assert(data != NULL);
-
-offset = g_malloc(sizeof(fpos_t));
-/* prepend & reverse? */
-if (!fgetpos(fp, offset))
-  data->frame_list = g_list_append(data->frame_list, offset); 
-else
-  {
-  g_free(offset);
-  return(1);
-  }
-
-#if DEBUG_ADD_FRAME_OFFSET
-printf("stored: %p\n", offset);
-#endif
-
-return(0);
-}
-
 /***********************************/
 /* retrieve the nth transformation */
 /***********************************/
@@ -280,13 +252,9 @@ gchar *text, *name;
 
 g_assert(model != NULL);
 
-/* test if should we return to the start */
-/*
-model->cur_frame++;
-*/
+/* increment and test if should we return to the start */
 model->cur_frame += model->anim_step;
-
-if (model->cur_frame == model->num_frames)
+if (model->cur_frame >= model->num_frames)
   if (model->anim_loop)
     model->cur_frame = 1;
 
@@ -374,12 +342,15 @@ if (sysenv.render.animate && !sysenv.render.no_povray_exec)
 /* cleanup */
 if (sysenv.render.no_keep_tempfiles)
   {
+#ifndef __WIN32
   text = g_strdup_printf("rm -rf %s_*.pov", sysenv.render.animate_file);
   system(text);
   g_free(text);
   text = g_strdup_printf("rm -rf %s_*.tga", sysenv.render.animate_file);
   system(text);
   g_free(text);
+#endif
+/* TODO - windows equivalents */
   }
 
 /* done - return FALSE to terminate the timer */
@@ -396,7 +367,7 @@ struct model_pak *model;
 model = dialog_model(data);
 g_assert(model != NULL);
 
-model->cur_frame = 1;
+model->cur_frame = 0;
 
 gui_relation_update(model);
 }
@@ -426,7 +397,7 @@ struct model_pak *model;
 model = dialog_model(data);
 g_assert(model != NULL);
 
-if (model->cur_frame > 1)
+if (model->cur_frame > 0)
   {
   model->cur_frame--;
   gui_relation_update(model);
@@ -596,6 +567,7 @@ label = gtk_label_new("Number of frames:");
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 tmp = g_strdup_printf("%9d", data->num_frames);
 label = gtk_label_new(tmp);
+g_free(tmp);
 gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
 /* desired delay */
