@@ -38,7 +38,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "file.h"
 #include "matrix.h"
 #include "parse.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
 
@@ -227,6 +227,16 @@ while (list)
   }
 }
 
+/*****************************/
+/* sanity check for a dialog */
+/*****************************/
+gint dialog_valid(gpointer dialog)
+{
+if (g_slist_find(sysenv.dialog_list, dialog))
+  return(TRUE);
+return(FALSE);
+}
+
 /********************************************/
 /* test if a specific type of dialog exists */
 /********************************************/
@@ -265,7 +275,7 @@ for (list=sysenv.dialog_list ; list ; list=g_slist_next(list))
   {
   dialog = list->data;
   if (dialog->refresh)
-    dialog->refresh(NULL);
+    dialog->refresh(dialog);
   }
 }
 
@@ -283,7 +293,7 @@ for (list=sysenv.dialog_list ; list ; list=g_slist_next(list))
 
   if (dialog->type == type)
     if (dialog->refresh)
-      dialog->refresh(NULL);
+      dialog->refresh(dialog);
   }
 }
 
@@ -301,7 +311,7 @@ for (list=sysenv.dialog_list ; list ; list=g_slist_next(list))
 
   if (dialog->model == model)
     if (dialog->refresh)
-      dialog->refresh(NULL);
+      dialog->refresh(dialog);
   }
 }
 
@@ -423,7 +433,7 @@ free_slist(dir_list);
 
 /* get directory listing */
 /* NB: success will always return something, even if only ".." */
-dir_list = get_dir_list(sysenv.cwd, TRUE);
+dir_list = file_dir_list(sysenv.cwd, TRUE);
 if (!dir_list)
   {
   gui_text_show(ERROR, "No directory listing; check your permissions.\n");
@@ -492,87 +502,6 @@ printf("[%s] : %d\n",full,buff.st_mode);
 return(TRUE);
 }
 
-/***********************************/
-/* change to a specified directory */
-/***********************************/
-#define DEBUG_SET_PATH 0
-gint set_path(const gchar *txt)
-{
-gint i, n, status=0;
-gchar **buff, *text;
-GString *path;
-
-/* split by directory separator */
-text = g_strdup(txt);
-g_strstrip(text);
-buff = g_strsplit(text, DIR_SEP, 0);
-path = g_string_new(NULL);
-
-/* find the number of tokens */
-n=0;
-while(*(buff+n))
-  {
-#if DEBUG_SET_PATH
-printf("[%s] ", *(buff+n));
-#endif
-  n++;
-  }
-#if DEBUG_SET_PATH
-printf(": found %d tokens.\n", n);
-#endif
-
-/* truncate token list if parent directory selected */
-if (n > 1)
-  if (g_ascii_strncasecmp("..",*(buff+n-1),2) == 0)
-    n -= 2;
-
-/* build new path */
-/* this is a bit fiddly, as we don't want a dir_sep on the end */
-/* EXCEPT if it's the root directory (blame windows for this) */
-g_string_sprintf(path, "%s", *buff);
-i=1;
-while (i < n)
-  {
-  g_string_sprintfa(path, "%s%s", DIR_SEP, *(buff+i));
-  i++;
-  }
-if (n < 2)
-  g_string_sprintfa(path, "%s", DIR_SEP);
-
-#if DEBUG_SET_PATH
-printf("testing path [%s] ... \n", path->str); 
-#endif
-
-if (g_file_test(path->str, G_FILE_TEST_IS_DIR))
-  {
-  if (g_file_test(path->str, G_FILE_TEST_IS_EXECUTABLE))
-    {
-    g_free(sysenv.cwd);
-    sysenv.cwd = g_strdup(path->str);
-    }
-  else
-    {
-#if DEBUG_SET_PATH
-printf(" Insufficient access permission.\n");
-#endif
-    status++;
-    }
-  }
-else
-  {
-#if DEBUG_SET_PATH
-printf(" Not a directory.\n");
-#endif
-  status++;
-  }
-
-g_free(text);
-g_strfreev(buff);
-g_string_free(path, TRUE);
-
-return(status);
-}
-
 /*****************************/
 /* file type/filter handlers */
 /*****************************/
