@@ -40,7 +40,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "numeric.h"
 #include "interface.h"
 #include "dialog.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "opengl.h"
 
 /* main pak structures */
