@@ -47,7 +47,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "space.h"
 #include "spatial.h"
 #include "surface.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "type.h"
 #include "interface.h"
 #include "dialog.h"
@@ -388,189 +388,9 @@ redraw_canvas(SINGLE);
 */
 }
 
-/*********************************************/
-/* find and record the maximum region number */
-/*********************************************/
-gint region_max(struct model_pak *mdata)
-{
-gint max;
-GSList *list;
-struct core_pak *core;
-
-max = 0;
-for (list=mdata->cores ; list ; list=g_slist_next(list))
-  {
-  core = list->data;
-  if (core->region > max)
-    max = core->region;
-  }
-return(max);
-}
-
-/*************************************************************/
-/* region changing routine - for dealing with polar surfaces */
-/*************************************************************/
-/* can now go both ways via the direction flag (UP or DOWN) */
-#define DEBUG_REGION_SWITCH_ATOM 0
-gint region_move_atom(struct core_pak *core, gint direction,
-                                    struct model_pak *data)
-{
-gint flag, primary, secondary, mov[2];
-gdouble vec[3], tmp[3], d[3];
-GSList *list;
-struct core_pak *comp;
-
-#if DEBUG_REGION_SWITCH_ATOM
-printf("       model: %s\n", data->basename);
-printf(" periodicity: %d\n", data->periodic);
-printf("         hkl: %f %f %f\n", data->surface.miller[0],
-          data->surface.miller[1], data->surface.miller[2]);
-printf("        dhkl: %f\n", data->surface.dspacing);
-printf("region sizes: %f %f\n", data->surface.region[0], data->surface.region[1]);
-printf("      moving: ");
-if (direction == UP)
-  printf("UP\n");
-else
-  printf("DOWN\n");
-#endif
-
-/* checks */
-g_return_val_if_fail(data != NULL, 1);
-g_return_val_if_fail(data->periodic == 2, 1);
-if (data->surface.region[0] < 1)
-  {
-  gui_text_show(ERROR, "region 1 is empty.\n");
-  return(1);
-  }
-
-/* setup region switching labels */
-if (direction == UP)
-  {
-  primary = REGION1A;
-  secondary = REGION2A;
-  }
-else
-  {
-  primary = REGION2A;
-  secondary = REGION1A;
-  }
-
-/* get fractional depth translation vector */
-ARR3SET(vec, data->surface.depth_vec);
-vecmat(data->ilatmat, vec);
-
-/* calculate offset to region boundary */
-ARR3SET(tmp, vec);
-if (direction == DOWN)
-  {
-  VEC3MUL(tmp, data->surface.region[0]);
-  VEC3MUL(tmp, -1.0);
-  }
-else
-  {
-  if (data->surface.region[1] == 0)
-    {
-    VEC3MUL(tmp, data->surface.region[0]);
-    }
-  else
-    {
-    VEC3MUL(tmp, data->surface.region[1]);
-    }
-  }
-
-/* if region 2 is empty, just move core to the bottom */
-if (data->surface.region[1] == 0.0)
-  {
-  ARR3ADD(core->x, tmp);
-  if (core->shell)
-    {
-    ARR3ADD((core->shell)->x, tmp);
-    }
-  atom_colour_scheme(data->colour_scheme, core, data);
-  return(0);
-  }
-
-/* get coordinates of target atom */
-ARR3ADD(tmp, core->x);
-
-#if DEBUG_REGION_SWITCH_ATOM
-P3VEC("    translation: ", vec);
-P3VEC("  target coords: ", tmp);
-#endif
-
-/* find the target */
-flag=0;
-for (list=data->cores ; list ; list=g_slist_next(list))
-  {
-  comp = list->data;
-
-/* only atoms of the same type need apply */
-  if (core->atom_code != comp->atom_code)
-    continue;
-
-/* get difference vector */
-  ARR3SET(d, comp->x);
-  ARR3SUB(d, tmp);
-
-/* pbc constraint */
-  while(d[0] < -FRACTION_TOLERANCE)
-    d[0] += 1.0;
-  while(d[0] > 0.5)
-    d[0] -= 1.0;
-  while(d[1] < -FRACTION_TOLERANCE)
-    d[1] += 1.0;
-  while(d[1] > 0.5)
-    d[1] -= 1.0;
-
-/* test difference vector's magnitude */
-  if (VEC3MAGSQ(d) < FRACTION_TOLERANCE)
-    {
-/* change its labelling */
-#if DEBUG_REGION_SWITCH_ATOM
-printf("Matched core: %p\n", comp);
-#endif
-    comp->region = secondary;
-    if (comp->shell)
-      {
-      (comp->shell)->region = secondary;
-      }
-    atom_colour_scheme(data->colour_scheme, comp, data);
-    flag++;
-    break;
-    }
-  }
-
-if (!flag)
-  {
-  gui_text_show(ERROR, "Failed to find a boundary image.\n");
-  return(1);
-  }
-
-/* now move selected atom to bottom of region 2 */
-ARR3SET(tmp, vec);
-VEC3MUL(tmp, (data->surface.region[0] + data->surface.region[1]));
-if (direction == UP)
-  VEC3MUL(tmp, -1.0);
-ARR3SUB(core->x, tmp);
-core->region = primary;
-/* pbc constrain */
-fractional_clamp(core->x, mov, 2);
-
-if (core->shell)
-  {
-  ARR3SUB((core->shell)->x, tmp);
-  ARR2ADD((core->shell)->x, mov);
-  (core->shell)->region = primary;
-  }
-
-atom_colour_scheme(data->colour_scheme, core, data);
-
-return(0);
-}
-
-/********************************************/
-/* move a selection using the above routine */
-/********************************************/
+/****************************/
+/* move the whole selection */
+/****************************/
 #define DEBUG_REGION_MOVE 0
 void region_move(GtkWidget *w, gint direction)
 {
@@ -741,148 +561,6 @@ calc_emp(data);
 g_free(elem);
 }
 
-/*******************************************************/
-/* permanently remove all deleted cores from all lists */
-/*******************************************************/
-#define DEBUG_DELETE_COMMIT 0
-void delete_commit(struct model_pak *data)
-{
-gint flag1=FALSE, flag2=FALSE;
-gpointer m;
-GSList *list1, *list2;
-struct core_pak *core;
-struct shel_pak *shell;
-struct bond_pak *bond;
-
-g_assert(data != NULL);
-
-/* delete flaged cores in list */
-list1 = data->cores;
-while (list1)
-  {
-  core = list1->data;
-  list1 = g_slist_next(list1);
-
-  if (core->status & DELETED)
-    {
-#if DEBUG_DELETE_COMMIT
-printf("Deleting %s core [%p] ...\n", core->label, core);
-#endif
-    flag1 = TRUE;
-
-/* flag assoc. shell */
-    if (core->shell)
-      {
-      shell = core->shell;
-      shell->status |= DELETED;
-      }
-
-/* update connectivity */
-    connect_atom_clear(core, data);
-    list2 = data->ubonds;
-    while (list2)
-      {
-      bond = list2->data;
-      list2 = g_slist_next(list2);
-
-      if (bond->atom1 == core || bond->atom2 == core)
-        {
-        data->ubonds = g_slist_remove(data->ubonds, bond);
-        g_free(bond);
-        }
-      }
-
-/* update selection */
-    data->selection = g_slist_remove(data->selection, core);
-
-/* delete any labels that reference the deleted core */
-    list2 = data->measure_list;
-    while (list2)
-      {
-      m = list2->data;
-      list2 = g_slist_next(list2);
-      if (measure_has_core(core, m))
-        measure_free(m, data);
-      }
-/* update main list */
-    data->cores = g_slist_remove(data->cores, core);
-    g_free(core);
-    }
-  }
-
-/* delete flaged shells in list */
-list1 = data->shels;
-while (list1)
-  {
-  shell = list1->data;
-  list1 = g_slist_next(list1);
-
-  if (shell->status & DELETED)
-    {
-    flag2 = TRUE;
-/* update main list */
-    data->shels = g_slist_remove(data->shels, shell);
-    g_free(shell);
-    }
-  }
-
-/* refresh totals */
-data->num_atoms = g_slist_length(data->cores);
-data->num_shells = g_slist_length(data->shels);
-
-/* refresh spatial partitioning */
-/* it's probably best to refresh the partitioning here, rather than */
-/* incrementally as it's speedups for large models we're targeting */
-zone_init(data);
-
-/* cope with deleted bonds - expensive, so only do if required */
-if (flag1)
-  {
-  connect_bonds(data);
-  connect_molecules(data);
-  }
-
-/* refresh net charge calc */
-calc_emp(data);
-
-/* refresh unique atom list */
-g_slist_free(data->unique_atom_list);
-data->unique_atom_list = find_unique(ELEMENT, data);
-
-/* refresh widgets */
-meas_graft_model(data);
-gui_refresh(GUI_MODEL_PROPERTIES);
-
-/* reset functions with static pointers to cores */
-data->state = 0;
-}
-
-/**********************************/
-/* flag core and associated shell */
-/**********************************/
-void delete_core(struct core_pak *core)
-{
-core->status |= DELETED;
-if (core->shell)
-  (core->shell)->status |= DELETED;
-}
-
-/*********************************/
-/* delete atom by pixel position */
-/*********************************/
-void delete_atom_at(gdouble *r, struct model_pak *data)
-{
-struct core_pak *core;
-
-/* attempt to match point with an atom */
-core = seek_coord3d(r, data);     
-if (core)
-  {
-  delete_core(core);
-  delete_commit(data);
-  }
-}
-
 /*****************************/
 /* change the lattice matrix */
 /*****************************/
@@ -1298,8 +976,8 @@ struct model_pak *model = sysenv.active_model;
 
 if (model)
   {
-  model->build_molecules ^= 1;
-  connect_refresh(model);
+  model->show_bonds ^= 1;
+
   redraw_canvas(SINGLE);
   }
 }
@@ -1310,62 +988,40 @@ if (model)
 /* TODO - molecule fragments (library?) */
 void build_page(GtkWidget *box)
 {
-GtkWidget *frame, *vbox, *hbox;
+GtkWidget *vbox, *hbox;
 GtkWidget *label, *entry, *spin;
 
 g_return_if_fail(box != NULL);
 
 /* Frame */
-frame = gtk_frame_new ("Atoms");
-gtk_box_pack_start(GTK_BOX(box),frame,FALSE,TRUE,0); 
-gtk_container_set_border_width (GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width (GTK_CONTAINER(vbox), PANEL_SPACING);
+vbox = gui_frame_vbox("Atoms", FALSE, FALSE, box);
+
 gui_button_x("Add atoms", edit_atom_add, NULL, vbox);
 gui_button_x("Confine atoms to cell", edit_confine, (gpointer) CORE, vbox);
 gui_button_x("Confine molecules to cell", edit_confine, (gpointer) MOL, vbox);
-
-/* NEW */
 gui_button_x("Add shells to selected cores", edit_shells_add, NULL, vbox);
 
-
-
 /* Frame */
 /* FIXME - fix up user bonds then re-introduce */
-frame = gtk_frame_new ("Connectivity");
-gtk_box_pack_start(GTK_BOX(box),frame,FALSE,TRUE,0); 
-gtk_container_set_border_width (GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width (GTK_CONTAINER(vbox), PANEL_SPACING);
+vbox = gui_frame_vbox("Bonds", FALSE, FALSE, box);
+
 gui_button_x("Add single bonds ", gtk_mode_switch, (gpointer) BOND_SINGLE, vbox);
 /*
 gui_button_x("Add double bonds ", gtk_mode_switch, (gpointer) BOND_DOUBLE, vbox);
 gui_button_x("Add triple bonds ", gtk_mode_switch, (gpointer) BOND_TRIPLE, vbox);
 */
-
 gui_button_x("Delete bonds ", gtk_mode_switch, (gpointer) BOND_DELETE, vbox);
 gui_button_x("Toggle bonding ", gui_connect_toggle, NULL, vbox);
 
 /* Frame */
-frame = gtk_frame_new ("Structural");
-gtk_box_pack_start(GTK_BOX(box),frame,FALSE,TRUE,0); 
-gtk_container_set_border_width (GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width (GTK_CONTAINER(vbox), PANEL_SPACING);
+vbox = gui_frame_vbox("Structural", FALSE, FALSE, box);
+
 gui_button_x("Make new model", edit_model_create, NULL, vbox);
 gui_button_x("Make supercell", edit_make_supercell, NULL, vbox);
 gui_button_x("Force structure to P1", edit_make_p1, NULL, vbox);
 
-/* NEW - nanotube setup */
-frame = gtk_frame_new ("Nanotube");
-gtk_box_pack_start(GTK_BOX(box),frame,FALSE,TRUE,0); 
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(FALSE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(vbox), PANEL_SPACING);
+/* nanotube setup */
+vbox = gui_frame_vbox("Nanotube", FALSE, FALSE, box);
 
 /* chirality */
 hbox = gtk_hbox_new(FALSE,0);
@@ -1626,7 +1282,7 @@ gchar *titles[] = {" Label ", " Size ", " Primitive "};
 GtkCellRenderer *r;
 GtkTreeViewColumn *c;
 GtkTreeSelection *select;
-GtkWidget *swin, *frame, *hbox, *vbox1, *vbox2, *vbox;
+GtkWidget *swin, *hbox, *vbox1, *vbox2, *vbox;
 
 /* initialize - nothing selected */
 spatial_selected = NULL;
@@ -1640,12 +1296,7 @@ vbox2 = gtk_vbox_new(FALSE, PANEL_SPACING);
 gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
 
 /* Frame */
-frame = gtk_frame_new ("Adding");
-gtk_box_pack_start(GTK_BOX(vbox1),frame,FALSE,FALSE,0); 
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(vbox), PANEL_SPACING);
+vbox = gui_frame_vbox("Adding", FALSE, FALSE, vbox1);
 
 /* mode buttons */
 gui_button_x("Add vectors ", 
@@ -1661,12 +1312,7 @@ gui_button_x("Add ribbons ",
                vbox);
 
 /* Frame */
-frame = gtk_frame_new ("Deleting");
-gtk_box_pack_start(GTK_BOX(vbox1),frame,FALSE,TRUE,0); 
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(vbox), PANEL_SPACING);
+vbox = gui_frame_vbox("Deleting", FALSE, FALSE, vbox1);
 
 gui_button_x("Delete all vectors ", gui_spatial_delete, "vector", vbox);
 gui_button_x("Delete all planes ", gui_spatial_delete, "plane", vbox);
@@ -1674,12 +1320,7 @@ gui_button_x("Delete all ", gui_spatial_delete_all, NULL, vbox);
 gui_button_x("Delete selected ", gui_spatial_delete_selected, NULL, vbox);
 
 /* Frame */
-frame = gtk_frame_new (NULL);
-gtk_box_pack_start(GTK_BOX(vbox1),frame,FALSE,TRUE,0); 
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(vbox), PANEL_SPACING);
+vbox = gui_frame_vbox(NULL, FALSE, FALSE, vbox1);
 
 /* TODO - apply to selection */
 gui_colour_box("Spatial fill colour  ", edit_spatial_colour, vbox);
@@ -1689,12 +1330,7 @@ gui_button_x("Apply to selected ",
                gui_spatial_colour_select, edit_spatial_colour, vbox);
 
 /* Frame */
-frame = gtk_frame_new (NULL);
-gtk_box_pack_start(GTK_BOX(vbox1),frame,FALSE,TRUE,0); 
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(vbox), PANEL_SPACING);
+vbox = gui_frame_vbox(NULL, FALSE, FALSE, vbox1);
 
 /* TODO - apply to selection */
 gui_colour_box("Spatial label colour ", edit_label_colour, vbox);
@@ -1704,12 +1340,7 @@ gui_button_x("Apply to selected ",
                gui_spatial_colour_label_select, edit_label_colour, vbox);
 
 /* spatial list */
-frame = gtk_frame_new(NULL);
-gtk_box_pack_start(GTK_BOX(vbox2),frame,TRUE,TRUE,0); 
-gtk_container_set_border_width (GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(vbox), PANEL_SPACING);
+vbox = gui_frame_vbox(NULL, TRUE, TRUE, vbox2);
 
 /* scrolled window */
 swin = gtk_scrolled_window_new(NULL, NULL);
@@ -1749,17 +1380,12 @@ void trans_page(GtkWidget *box)
 {
 gint i, j;
 GList *list;
-GtkWidget *frame, *table, *hbox, *vbox, *label;
+GtkWidget *table, *hbox, *vbox, *label;
 
 g_assert(box != NULL);
 
 /* transformation construction */
-frame = gtk_frame_new("Construction");
-gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
-gtk_container_set_border_width(GTK_CONTAINER (frame), PANEL_SPACING);
-vbox = gtk_vbox_new(FALSE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(GTK_BOX(vbox)), PANEL_SPACING);
+vbox = gui_frame_vbox("Construction", FALSE, FALSE, box);
 
 /* TODO - use spinners instead of gtk entries for these numbers */
 /* axes order */
@@ -1843,12 +1469,7 @@ for (j=0 ; j<3 ; j++)
   }
 
 /* control buttons */
-frame = gtk_frame_new("Coordinate transformations");
-gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
-gtk_container_set_border_width(GTK_CONTAINER (frame), PANEL_SPACING);
-vbox = gtk_vbox_new(FALSE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(GTK_BOX(vbox)), PANEL_SPACING);
+vbox = gui_frame_vbox("Coordinate transformations", FALSE, FALSE, box);
 
 gui_button_x("Apply to all atoms", 
                apply_transmat, GINT_TO_POINTER(AT_ANY),
@@ -1869,12 +1490,7 @@ gui_button_x(NULL, apply_n_transmat, GINT_TO_POINTER(AT_ANY), hbox);
 */
 
 /* control buttons */
-frame = gtk_frame_new("Lattice transformations");
-gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
-gtk_container_set_border_width(GTK_CONTAINER (frame), PANEL_SPACING);
-vbox = gtk_vbox_new(FALSE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(GTK_BOX(vbox)), PANEL_SPACING);
+vbox = gui_frame_vbox("Lattice transformations", FALSE, FALSE, box);
 
 gui_button_x("Apply to lattice matrix", edit_transform_latmat, NULL, vbox);
 
@@ -2474,7 +2090,8 @@ GtkWidget *obj;
 struct model_pak *model;
 
 model = sysenv.active_model;
-g_assert(model != NULL);
+if (!model)
+  return;
 
 obj = dialog_child_get(dialog, "FF_label");
 ff_type = gtk_entry_get_text(GTK_ENTRY(obj));
@@ -2497,21 +2114,55 @@ type_apply(type, model->cores);
 type_free(type);
 }
 
+/**********************************/
+/* read in a set of FF parameters */
+/**********************************/
+void gui_import_ff(GtkWidget *w, gpointer dialog)
+{
+const gchar *name;
+GSList *list;
+GtkWidget *obj;
+struct model_pak *model;
+
+model = sysenv.active_model;
+if (!model)
+  return;
+
+obj = dialog_child_get(dialog, "FF_import");
+name = gtk_entry_get_text(GTK_ENTRY(obj));
+
+list = gromacs_read_ff(name);
+
+if (list)
+  {
+  gui_text_show(NORMAL, "Imported Forcefield.\n");
+
+/* remove any previous GULP FF set */
+  if (model->gulp.potentials)
+    {
+    g_free(model->gulp.potentials);
+    model->gulp.potentials = NULL;
+    }
+/* remove any previous internal FF set */
+  if (model->ff_list)
+    free_slist(model->ff_list);
+
+  model->ff_list = list;
+  }
+else
+  gui_text_show(WARNING, "Empty or missing Forcefield.\n");
+}
+
 /*******************************/
 /* core labelling manipulation */
 /*******************************/
-/* TODO - not enough options now - link to surface dialog somehow? */
 void labelling_page(GtkWidget *box, gpointer dialog)
 {
-GtkWidget *w, *frame, *vbox;
+GtkWidget *w, *vbox, *hbox;
 
 /* Surface options */
-frame = gtk_frame_new("Regions");
-gtk_box_pack_start(GTK_BOX (box), frame, FALSE, FALSE, 0);
-gtk_container_set_border_width(GTK_CONTAINER (frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(GTK_BOX(vbox)), PANEL_SPACING);
+vbox = gui_frame_vbox("Regions", FALSE, FALSE, box);
+
 gui_button_x("Move selection up", region_move, (gpointer) UP, vbox);
 gui_button_x("Move selection down", region_move, (gpointer) DOWN, vbox);
 
@@ -2522,12 +2173,7 @@ GList *list;
 GtkWidget *combo, *hbox, *label;
 
 /* EXP - forcefield assignment */
-frame = gtk_frame_new("Typing");
-gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
-gtk_container_set_border_width(GTK_CONTAINER (frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(GTK_BOX(vbox)), PANEL_SPACING);
+vbox = gui_frame_vbox("Typing", FALSE, FALSE, box);
 
 /* typing setup */
 hbox = gtk_hbox_new(FALSE, PANEL_SPACING);
@@ -2538,9 +2184,13 @@ gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
 list = NULL;
 list = g_list_prepend(list, "QEq charges");
-list = g_list_prepend(list, "CVFF labels");
-list = g_list_prepend(list, "Dreiding labels");
 list = g_list_prepend(list, "Gasteiger charges");
+list = g_list_prepend(list, "Dreiding labels");
+
+/* experimental */
+/*
+list = g_list_prepend(list, "CVFF labels");
+*/
 
 combo = gtk_combo_new();
 gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(combo)->entry), FALSE);
@@ -2554,11 +2204,7 @@ gui_button_x(NULL, cb_type_model, GTK_COMBO(combo)->entry, hbox);
 #if DIFFAX
 /* DIFFAX stuff - shunted here */
 /* frame */
-frame = gtk_frame_new("DIFFAX");
-gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(FALSE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
+vbox = gui_frame_vbox("DIFFAX", FALSE, FALSE, box);
 
 /* source */
 hbox = gtk_hbox_new(FALSE, 0);
@@ -2600,11 +2246,7 @@ gui_icon_button(GTK_STOCK_SAVE, "Save  ",
 #endif
 
 /* frame */
-frame = gtk_frame_new("Experimental typing");
-gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(FALSE, PANEL_SPACING);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
+vbox = gui_frame_vbox("Experimental typing", FALSE, FALSE, box);
 
 w = gui_text_entry("FF label ", NULL, TRUE, FALSE, vbox);
 dialog_child_set(dialog, "FF_label", w);
@@ -2618,9 +2260,26 @@ dialog_child_set(dialog, "FF_level", w);
 w = gui_text_entry("Neighbour Count ", NULL, TRUE, FALSE, vbox);
 dialog_child_set(dialog, "FF_count", w);
 
+/* CURRENT - experimental */
+/*
+gtk_widget_set_sensitive(GTK_WIDGET(vbox), FALSE);
+*/
 
 /* apply single rule */
 gui_button_x("Apply rule ", gui_make_rule, dialog, vbox);
+
+vbox = gui_frame_vbox("Experimental - Import Forcefield", FALSE, FALSE, box);
+
+hbox = gtk_hbox_new(FALSE, 0);
+gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+w = gui_text_entry("Import GROMACS FF ", NULL, TRUE, FALSE, hbox);
+dialog_child_set(dialog, "FF_import", w);
+
+/* FIXME - for convenience during testing */
+gtk_entry_set_text(GTK_ENTRY(w), "ffoplsaabon.itp");
+
+gui_button_x(NULL, gui_import_ff, dialog, hbox);
 }
 
 /**************************/
@@ -2702,3 +2361,537 @@ for (i=0 ; i<12; i++)
                    GTK_SIGNAL_FUNC(change_transmat), (gpointer) i);
 }
 
+/* globals for the atom properties dialog */
+GtkWidget *apd_label, *apd_type, *apd_charge, *apd_x, *apd_y, *apd_z;
+GtkWidget *apd_growth, *apd_region, *apd_translate;
+struct model_pak *apd_data=NULL;
+struct core_pak *apd_core=NULL;
+
+/***********************************************/
+/* change the properties of all selected atoms */
+/***********************************************/
+/* NB: this is a bit dangerous as you change everything in the */
+/* selection to the specified value - even (eg) incompatible elements */
+void selection_properties_change(gint type)
+{
+gint n, growth, region, translate;
+gdouble charge;
+const gchar *text;
+GSList *list;
+struct elem_pak edata;
+struct model_pak *model;
+struct core_pak *core;
+
+model = sysenv.active_model;
+if (!model)
+  return;
+if (!model->selection)
+  return;
+
+switch (type)
+  {
+  case NAME:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_label));
+    n = elem_symbol_test(text);
+    if (n)
+      {
+      get_elem_data(n, &edata, model);
+/* make sure we alow enough space for the string and the \0 */
+      n = (LABEL_SIZE-1 > strlen(text)) ? strlen(text) : LABEL_SIZE-1;
+      for (list=model->selection ; list ; list=g_slist_next(list))
+        {
+        core = list->data;
+        g_free(core->atom_label);
+        core->atom_label = g_strdup(text);
+/* update atttached shell */
+        if (core->shell)
+          {
+          struct shel_pak *shell = core->shell;
+          g_free(shell->shell_label);
+          shell->shell_label = g_strdup(text);
+          }
+/* NEW - don't update element specific data if the element type was not */
+/* changed - ie the user has just made a labelling change (eg C -> C1) */
+        if (edata.number != core->atom_code)
+          {
+          core->atom_code = edata.number;
+          core->bond_cutoff = edata.cova;
+          }
+        init_atom_colour(core, model);
+        init_atom_charge(core, model);
+        }
+/* model updates */
+      g_slist_free(model->unique_atom_list);
+      model->unique_atom_list = find_unique(ELEMENT, model);
+      calc_emp(model);
+      }
+    break;
+
+  case CHARGE:
+    charge = str_to_float(gtk_entry_get_text(GTK_ENTRY(apd_charge)));
+    for (list=model->selection ; list ; list=g_slist_next(list))
+      {
+      core = list->data;
+/* core updates */
+      core->charge = charge;
+      core->lookup_charge = FALSE;
+      }
+    calc_emp(model);
+    break;
+
+  case CORE_GROWTH_SLICE:
+    growth = str_to_float(gtk_entry_get_text(GTK_ENTRY(apd_growth)));
+    growth = CLAMP(growth, 0, 1);
+    for (list=model->selection ; list ; list=g_slist_next(list))
+      {
+      core = list->data;
+      core->growth = growth;
+      if (model->colour_scheme == GROWTH_SLICE)
+        atom_colour_scheme(GROWTH_SLICE, core, model);
+      }
+    break;
+
+  case CORE_REGION:
+    region = str_to_float(gtk_entry_get_text(GTK_ENTRY(apd_region)));
+    if (region > model->region_max)
+      model->region_max = region;
+
+    for (list=model->selection ; list ; list=g_slist_next(list))
+      {
+      core = list->data;
+      core->region = region;
+      if (core->shell)
+        (core->shell)->region = region;
+
+      if (model->colour_scheme == REGION)
+        atom_colour_scheme(REGION, core, model);
+      }
+    break;
+
+  case CORE_TRANSLATE:
+    translate = str_to_float(gtk_entry_get_text(GTK_ENTRY(apd_translate)));
+    translate = CLAMP(translate, 0, 1);
+    for (list=model->selection ; list ; list=g_slist_next(list))
+      {
+      core = list->data;
+      core->translate = translate;
+      if (core->shell)
+        (core->shell)->translate = translate;
+
+      if (model->colour_scheme == TRANSLATE)
+        atom_colour_scheme(TRANSLATE, core, model);
+      }
+    break;
+
+  case CORE_FF:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_type));
+    for (list=model->selection ; list ; list=g_slist_next(list))
+      {
+      core = list->data;
+      if (core->atom_type)
+        g_free(core->atom_type);
+      core->atom_type = g_strdup(text);
+      }
+    break;
+  }
+
+gui_refresh(GUI_MODEL_PROPERTIES);
+gui_refresh(GUI_CANVAS);
+}
+
+/*****************************/
+/* commit changes to an atom */
+/*****************************/
+void atom_properties_change(GtkWidget *w, gint type)
+{
+gint n, growth, region, translate;
+const gchar *text;
+struct elem_pak edata;
+struct model_pak *model;
+
+model = sysenv.active_model;
+if (!model)
+  return;
+
+/* act on multiple atoms? */
+if (g_slist_length(model->selection) > 1)
+  {
+  selection_properties_change(type);
+  return;
+  }
+
+if (!apd_core)
+  return;
+
+switch(type)
+  {
+  case NAME:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_label));
+    g_free(apd_core->atom_label);
+    apd_core->atom_label = g_strdup(text);
+    n = elem_symbol_test(text);
+
+/* update atttached shell */
+if (apd_core->shell)
+  {
+  struct shel_pak *shell = apd_core->shell;
+  g_free(shell->shell_label);
+  shell->shell_label = g_strdup(text);
+  }
+
+/* if recognized -> update */
+    if (n)
+      {
+      get_elem_data(n, &edata, model);
+
+/* NEW - don't update element specific data if the element type was not */
+/* changed - ie the user has just made a labelling change (eg C -> C1) */
+      if (n != apd_core->atom_code)
+        {
+        apd_core->atom_code = n;
+        apd_core->bond_cutoff = edata.cova;
+        }
+      init_atom_colour(apd_core, model);
+      init_atom_charge(apd_core, model);
+
+      g_slist_free(model->unique_atom_list);
+      model->unique_atom_list = find_unique(ELEMENT, model);
+      calc_emp(model);
+
+/* REFRESH */
+      gui_refresh(GUI_MODEL_PROPERTIES);
+      }
+    break;
+
+  case CORE_FF:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_type));
+    if (apd_core->atom_type)
+      g_free(apd_core->atom_type);
+    apd_core->atom_type = g_strdup(text);
+    break;
+
+  case CHARGE:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_charge));
+    apd_core->charge = str_to_float(text);
+    apd_core->lookup_charge = FALSE;
+    calc_emp(model);
+    break;
+
+  case COORD_X:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_x));
+    apd_core->x[0] = str_to_float(text);
+    coords_compute(model);
+    break;
+
+  case COORD_Y:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_y));
+    apd_core->x[1] = str_to_float(text);
+    coords_compute(model);
+    break;
+
+  case COORD_Z:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_z));
+    apd_core->x[2] = str_to_float(text);
+    coords_compute(model);
+    break;
+
+  case CORE_GROWTH_SLICE:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_growth));
+    growth = CLAMP(str_to_float(text), 0, 1);
+    apd_core->growth = growth;
+    if (model->colour_scheme == GROWTH_SLICE)
+      atom_colour_scheme(GROWTH_SLICE, apd_core, model);
+    break;
+
+  case CORE_REGION:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_region));
+    region = str_to_float(text);
+    if (region > model->region_max)
+      model->region_max = region;
+    apd_core->region = region;
+    if (apd_core->shell)
+      (apd_core->shell)->region = region;
+    if (model->colour_scheme == REGION)
+      atom_colour_scheme(REGION, apd_core, model);
+    break;
+
+  case CORE_TRANSLATE:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_translate));
+    translate = CLAMP(str_to_float(text), 0, 1);
+    apd_core->translate = translate;
+    if (apd_core->shell)
+      (apd_core->shell)->translate = translate;
+    if (model->colour_scheme == TRANSLATE)
+      atom_colour_scheme(TRANSLATE, apd_core, model);
+    break;
+
+  default:
+    printf("Not yet modifiable...\n");
+  }
+gui_refresh(GUI_CANVAS);
+}
+
+/*************************************/
+/* updates the dialog for a new atom */
+/*************************************/
+void gui_refresh_selection(void)
+{
+gint n, cflag=FALSE;
+gdouble q, centroid[3];
+gchar *element, *label, *type, *charge, *x, *y, *z, *growth, *region, *translate;
+struct core_pak *core;
+struct model_pak *model;
+GSList *list;
+
+model = sysenv.active_model;
+core = NULL;
+if (model)
+  {
+  list = model->selection;
+  n = g_slist_length(list);
+  switch (n)
+    {
+    case -1:
+      g_assert_not_reached();
+    case 0:
+      break;
+    case 1:
+      core = list->data;
+      break;
+
+    default:
+      VEC3SET(centroid, 0.0, 0.0, 0.0);
+      for (list=model->selection ; list ; list=g_slist_next(list))
+        {
+        core = list->data;
+        ARR3ADD(centroid, core->x);
+        }
+      VEC3MUL(centroid, 1.0 / (gdouble) n);
+/* special print case - centroid display */
+      cflag = TRUE;
+      core = NULL;
+    }
+  }
+
+if (core && model)
+  {
+/* data available */
+  element = g_strdup(elements[core->atom_code].symbol);
+  label = g_strdup(core->atom_label);
+
+  if (core->atom_type)
+    type = g_strdup(core->atom_type);
+  else
+    type = g_strdup("");
+
+  q = atom_charge(core); /* Replaced by C. Fisher 2004 */
+  charge = g_strdup_printf("%9.4f", q);
+
+  x = g_strdup_printf("%9.4f", core->x[0]);
+  y = g_strdup_printf("%9.4f", core->x[1]);
+  z = g_strdup_printf("%9.4f", core->x[2]);
+
+  growth = g_strdup_printf("%d", core->growth);
+  region = g_strdup_printf("%d", core->region);
+  translate = g_strdup_printf("%d", core->translate);
+
+  apd_core = core;
+  }
+else
+  {
+/* otherwise defaults */
+  element = g_strdup("");
+  type = g_strdup("");
+  charge = g_strdup("");
+
+  if (cflag)
+    {
+    label = g_strdup("centroid");
+    x = g_strdup_printf("%9.4f", centroid[0]);
+    y = g_strdup_printf("%9.4f", centroid[1]);
+    z = g_strdup_printf("%9.4f", centroid[2]);
+    }
+  else
+    {
+    label = g_strdup("");
+    x = g_strdup("");
+    y = g_strdup("");
+    z = g_strdup("");
+    }
+
+  growth = g_strdup("");
+  region = g_strdup("");
+  translate = g_strdup("");
+  }
+
+/* prevent changes from messing up the atom_properties_change() callback */
+apd_data = NULL;
+
+/* entry updates */
+gtk_entry_set_text(GTK_ENTRY(apd_label), label);
+gtk_entry_set_text(GTK_ENTRY(apd_type), type);
+gtk_entry_set_text(GTK_ENTRY(apd_charge), charge);
+gtk_entry_set_text(GTK_ENTRY(apd_x), x);
+gtk_entry_set_text(GTK_ENTRY(apd_y), y);
+gtk_entry_set_text(GTK_ENTRY(apd_z), z);
+gtk_entry_set_text(GTK_ENTRY(apd_growth), growth);
+gtk_entry_set_text(GTK_ENTRY(apd_region), region);
+gtk_entry_set_text(GTK_ENTRY(apd_translate), translate);
+
+apd_data = model;
+
+/* cleanup */
+g_free(element);
+g_free(label);
+g_free(type);
+g_free(charge);
+g_free(x);
+g_free(y);
+g_free(z);
+g_free(growth);
+g_free(region);
+g_free(translate);
+}
+
+/*******************************************/
+/* display the properties of a single atom */
+/*******************************************/
+void gui_edit_widget(GtkWidget *box)
+{
+GtkWidget *frame, *hbox, *vbox, *entry;
+
+/* checks */
+g_return_if_fail(box != NULL);
+
+/* two column element data display */
+hbox = gtk_hbox_new(FALSE, 0);
+gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, TRUE, 0);
+
+/* left vbox - titles */
+vbox = gtk_vbox_new(TRUE, 0);
+gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
+
+/* TODO - put in a for loop? */
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "Label");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "FF Type");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "X");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "Y");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "Z");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "Charge");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "Growth");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "Region");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "Translate");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+/* right vbox - data */
+vbox = gtk_vbox_new(TRUE, 0);
+gtk_box_pack_end(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
+
+apd_label = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_label, FALSE, FALSE, 0);
+
+apd_type = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_type, FALSE, FALSE, 0);
+
+apd_x = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_x, FALSE, FALSE, 0);
+
+apd_y = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_y, FALSE, FALSE, 0);
+
+apd_z = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_z, FALSE, FALSE, 0);
+
+apd_charge = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_charge, FALSE, FALSE, 0);
+
+apd_growth = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_growth, FALSE, FALSE, 0);
+
+apd_region = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_region, FALSE, FALSE, 0);
+
+apd_translate = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_translate, FALSE, FALSE, 0);
+
+/* attach callbacks (NB: set initial data first) */
+g_signal_connect(GTK_OBJECT(apd_label), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(NAME));
+g_signal_connect(GTK_OBJECT(apd_type), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(CORE_FF));
+
+g_signal_connect(GTK_OBJECT(apd_x), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(COORD_X));
+g_signal_connect(GTK_OBJECT(apd_y), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(COORD_Y));
+g_signal_connect(GTK_OBJECT(apd_z), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(COORD_Z));
+
+g_signal_connect(GTK_OBJECT(apd_charge), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(CHARGE));
+
+g_signal_connect(GTK_OBJECT(apd_growth), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change),
+                 GINT_TO_POINTER(CORE_GROWTH_SLICE));
+
+g_signal_connect(GTK_OBJECT(apd_region), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(CORE_REGION));
+g_signal_connect(GTK_OBJECT(apd_translate), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(CORE_TRANSLATE));
+
+/* CURRENT */
+frame = gtk_frame_new(NULL);
+gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
+gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+vbox = gtk_vbox_new(TRUE, 1);
+gtk_container_add(GTK_CONTAINER(frame), vbox);
+
+gui_button_x("Add atoms", gtk_mode_switch, (gpointer) ATOM_ADD, vbox);
+gui_button_x("Add bonds", gtk_mode_switch, (gpointer) BOND_SINGLE, vbox);
+gui_button_x("Delete bonds", gtk_mode_switch, (gpointer) BOND_DELETE, vbox);
+gui_button_x("Normal mode", gtk_mode_switch, (gpointer) FREE, vbox);
+
+frame = gtk_frame_new(NULL);
+gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
+gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+vbox = gtk_vbox_new(TRUE, 1);
+gtk_container_add(GTK_CONTAINER(frame), vbox);
+
+gui_button_x("Mark as ghost", select_flag_ghost, NULL, vbox);
+gui_button_x("Mark as normal", select_flag_normal, NULL, vbox);
+}
