@@ -33,43 +33,21 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "coords.h"
 #include "file.h"
 #include "task.h"
+#include "grid.h"
 #include "matrix.h"
 #include "parse.h"
 #include "spatial.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "dialog.h"
 #include "interface.h"
 #include "opengl.h"
 
-struct basis_pak basis_sets[] = {
-{"User Defined", GMS_USER, 0},
-{"MNDO", GMS_MNDO, 0},
-{"AM1", GMS_AM1, 0},
-{"PM3", GMS_PM3, 0},
-{"MINI", GMS_MINI, 0},
-{"MIDI", GMS_MIDI, 0},
-{"STO-2G", GMS_STO, 2},
-{"STO-3G", GMS_STO, 3},
-{"STO-4G", GMS_STO, 4},
-{"STO-5G", GMS_STO, 5},
-{"STO-6G", GMS_STO, 6},
-{"3-21G", GMS_N21, 3},
-{"6-21G", GMS_N21, 6},
-{"4-31G", GMS_N31, 4},
-{"5-31G", GMS_N31, 5},
-{"6-31G", GMS_N31, 6},
-{"6-311G", GMS_N311, 6},
-{"DZV", GMS_DZV, 0},
-{"DH", GMS_DH, 0},
-{"TZV", GMS_TZV, 0},
-{"MC", GMS_MC, 0},
-{NULL, 0, 0}
-};
-
 #define CONTROL_TAB_LABEL " Control "
 #define BASIS_TAB_LABEL " Basis Set "
 #define OPTIMISATION_TAB_LABEL " Optimisation "
 
+extern struct basis_pak basis_sets[];
+
 enum {
   TIMLIM, MWORDS, WIDE_OUTPUT
 };
@@ -131,7 +109,7 @@ gchar *basename, *bat, *fubar, *fname, *temp, *tmp, *dir, *inp, *out;
 FILE *fp;
 
 /* setup variables */
-basename = strdup_basename(input);
+basename = parse_strip(input);
 fname = g_strdup_printf("%s.F05", basename);
 fubar = g_build_filename(sysenv.gamess_path, "scratch", fname, NULL);
 temp = g_build_filename(sysenv.gamess_path, "temp", basename, NULL);
@@ -811,14 +789,18 @@ gui_direct_spin("Maximum cycles", &data->gamess.nstep,
                   0, 500, 10, NULL, NULL, hbox);
 
 
-/* setup the filename entry */
-
 /* setup some gulp file name defaults */
 if (g_ascii_strncasecmp(data->gamess.temp_file, "none", 4) == 0)
   {
   g_free(data->gamess.temp_file);
   data->gamess.temp_file = g_strdup_printf("%s.inp", data->basename); 
   }
+if (g_ascii_strncasecmp(data->gamess.out_file, "none", 4) == 0)
+  {
+  g_free(data->gamess.out_file);
+  data->gamess.out_file = g_strdup_printf("%s.gmot", data->basename); 
+  }
+
 
 frame = gtk_frame_new("Files");
 gtk_box_pack_start(GTK_BOX(box),frame,FALSE,FALSE,0);
@@ -904,6 +886,36 @@ else
 g_string_free(line, TRUE);
 }
 
+/***********************/
+/* grid job submission */
+/***********************/
+void gui_gms_grid_submit(GtkWidget *w, gpointer data)
+{
+#ifdef WITH_GRISU
+gchar *tmp;
+struct model_pak *model = data;
+struct grid_pak *grid;
+
+g_assert(model != NULL);
+
+grid = grid_new();
+grid->local_cwd = g_strdup(sysenv.cwd);
+grid->local_input = g_strdup(model->gamess.temp_file);
+
+/* immediately write to disk so job is preserved even if user deletes model */
+tmp = g_build_filename(grid->local_cwd, grid->local_input, NULL);
+write_gms(tmp, model);
+g_free(tmp);
+
+/* TODO - deal with gamess vs gamess-us problems that could arise */
+if (!grid_job_submit("gamess-us", grid))
+  {
+/* TODO - failed message */
+  grid_free(grid);
+  }
+#endif
+}
+
 /****************************************/
 /* run the approriate energetics method */
 /****************************************/
@@ -947,6 +959,10 @@ gui_gamess_widget(vbox, model);
 gui_stock_button(GTK_STOCK_EXECUTE, gamess_execute, model,
                    GTK_DIALOG(window)->action_area);
 
+#ifdef WITH_GRISU
+gui_icon_button(GTK_STOCK_APPLY, "Grid Submit", gui_gms_grid_submit, model, GTK_DIALOG(window)->action_area);
+#endif
+
 gui_stock_button(GTK_STOCK_CLOSE, dialog_destroy, dialog,
                    GTK_DIALOG(window)->action_area);
 
