@@ -34,7 +34,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "matrix.h"
 #include "render.h"
 #include "gperiodic.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
 #include "opengl.h"
