@@ -37,10 +37,11 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "parse.h"
 #include "scan.h"
 #include "task.h"
+#include "grid.h"
 #include "matrix.h"
 #include "surface.h"
 #include "spatial.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
 #include "opengl.h"
@@ -68,44 +69,10 @@ if (model->num_phonons > 0)
   gtk_range_set_range(GTK_RANGE(model->phonon_slider), 1, model->num_phonons);
 }
 
-/*******************/
-/* run a gulp file */
-/*******************/
-#define DEBUG_EXEC_GULP 0
-gint exec_gulp(const gchar *input, const gchar *output)
-{
-gint status=0;
-gchar *cmd;
-
-/* checks */
-if (!sysenv.gulp_path)
-  return(-1);
-
-/* delete the old file to be sure output is only data from current run */
-#if _WIN32
-unlink("gulp.got");
-/* NB: must enclose paths with single quotes */
-cmd = g_strdup_printf("'%s'", sysenv.gulp_path);
-#else
-unlink(output);
-cmd = g_strdup_printf("%s < %s > %s", sysenv.gulp_path, input, output);
-#endif
-
-#if DEBUG_EXEC_GULP
-printf("executing: [%s]\n",cmd);
-#endif
-
-task_sync(cmd);
-
-/* done */
-g_free(cmd);
-return(status);
-}
-
 /*****************************/
 /* execute a gulp run (task) */
 /*****************************/
-#define DEBUG_EXEC_GULP_TASK 0 
+#define DEBUG_EXEC_GULP_TASK 0
 void exec_gulp_task(gpointer ptr, gpointer data)
 {
 gchar *inpfile;
@@ -117,13 +84,8 @@ g_assert(model != NULL);
 g_assert(task != NULL);
 
 /* construct fullpath input filename - required for writing */
-#if _WIN32
-inpfile = g_build_filename(sysenv.cwd, "gulp.gin", NULL);
-task->status_file = g_build_filename(sysenv.cwd, "gulp.got", NULL);
-#else
 inpfile = g_build_filename(sysenv.cwd, model->gulp.temp_file, NULL);
 task->status_file = g_build_filename(sysenv.cwd, model->gulp.out_file, NULL);
-#endif
 
 #if DEBUG_EXEC_GULP_TASK
 printf(" input file: %s\n", inpfile);
@@ -168,11 +130,7 @@ if (data->gulp.no_exec)
 inp = g_build_filename(sysenv.cwd, data->gulp.temp_file, NULL);
 res = g_build_filename(sysenv.cwd, data->gulp.dump_file, NULL);
 
-#if __WIN32
-out = g_build_filename(sysenv.cwd, "gulp.got", NULL);
-#else
 out = g_build_filename(sysenv.cwd, data->gulp.out_file, NULL);
-#endif
 
 switch (data->gulp.run)
   {
@@ -274,7 +232,7 @@ if (!sysenv.gulp_path)
 printf("output file: %s\n", data->gulp.out_file);
 #endif
 
-task_new("Gulp", &exec_gulp_task, data, &proc_gulp_task, data, data);
+task_new("gulp", &exec_gulp_task, data, &proc_gulp_task, data, data);
 }
 
 /*************************/
@@ -419,6 +377,12 @@ model->gulp.dump_file = g_strdup_printf("%s.res", text);
 model->gulp.trj_file = g_strdup_printf("%s.trg", text);
 model->gulp.out_file = g_strdup_printf("%s.got", text);
 
+/* disallow spaces (bad filename) */
+parse_space_replace(model->gulp.temp_file, '_');
+parse_space_replace(model->gulp.dump_file, '_');
+parse_space_replace(model->gulp.trj_file, '_');
+parse_space_replace(model->gulp.out_file, '_');
+
 gui_relation_update_widget(&model->gulp.dump_file);
 gui_relation_update_widget(&model->gulp.temp_file);
 gui_relation_update_widget(&model->gulp.trj_file);
@@ -760,6 +724,10 @@ GSList *list;
 
 g_assert(model != NULL);
 
+/* checks */
+if (model->num_phonons < 1)
+  return;
+
 spectrum = g_malloc(model->num_phonons * sizeof(gdouble));
 
 /* extract data into array */
@@ -1057,6 +1025,8 @@ g_assert(model != NULL);
 
 solvation = dialog_child_get(dialog, "solvation_model");
 text = gui_pd_text(solvation);
+if (!text)
+  return;
 
 if (g_ascii_strncasecmp(text, "none", 4) == 0)
   {
@@ -1436,7 +1406,7 @@ g_object_set_data(G_OBJECT(button), "ptr", (gpointer) model);
 if (model->gulp.coulomb == NOBUILD)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 gui_direct_check("Fix the initial connectivity", &model->gulp.fix, NULL, NULL, vbox);
-
+gui_direct_check("Automatic connectivity off", &model->gulp.noautobond, NULL, NULL, vbox);
 
 /* temperature & pressure */
 frame = gtk_frame_new(NULL);
@@ -1525,46 +1495,30 @@ vbox = gtk_vbox_new(FALSE, PANEL_SPACING/2);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
 
 /* GULP file labels */
-/*
-hbox = gtk_hbox_new(FALSE, 0);
-gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-label = gtk_label_new("Input file");
-gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-label = gui_auto_text_label(&model->gulp.temp_file);
-gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-hbox = gtk_hbox_new(FALSE, 0);
-gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-label = gtk_label_new("Dump file");
-gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-label = gui_auto_text_label(&model->gulp.dump_file);
-gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-hbox = gtk_hbox_new(FALSE, 0);
-gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-label = gtk_label_new("Trajectory file");
-gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-label = gui_auto_text_label(&model->gulp.trj_file);
-gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-*/
-
 gui_text_entry("Input file", &model->gulp.temp_file, FALSE, FALSE, vbox);
 gui_text_entry("Dump file", &model->gulp.dump_file, FALSE, FALSE, vbox);
 gui_text_entry("Trajectory file  ", &model->gulp.trj_file, FALSE, FALSE, vbox);
 
+/*
 gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, FALSE, 0);
 
 hbox = gtk_hbox_new(FALSE, 0);
 gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-label = gtk_label_new("Job name  ");
+label = gtk_label_new("Customize job name  ");
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 entry = gtk_entry_new();
 gtk_box_pack_end(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
+*/
 
-gtk_entry_set_text(GTK_ENTRY(entry), model->basename);
+/* FIXME - stopped using basename, because it may have spaces in it - which */
+/* cause problems reading/writing filenames (ie not quoted) */
+//gtk_entry_set_text(GTK_ENTRY(entry), model->basename);
+/*
+gtk_entry_set_text(GTK_ENTRY(entry), "");
 g_signal_connect(GTK_OBJECT(entry), "changed",
                  GTK_SIGNAL_FUNC(gulp_jobname_changed), (gpointer) model);
 
+*/
 
 /* CURRENT - advanced options */
 frame = gtk_frame_new("Options");
@@ -1678,11 +1632,7 @@ if (model->gulp.optimiser2 == RFO_OPT)
 
 
 /* optimiser to use */
-frame = gtk_frame_new("Switching critera");
-gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(FALSE, 0);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
+vbox = gui_frame_vbox("Switching criteria", TRUE, TRUE, hbox);
 
 /* do the first (DEFAULT MODE) radio button */
 button = gtk_radio_button_new_with_label(NULL, "cycle");
@@ -1716,23 +1666,18 @@ g_signal_connect(GTK_OBJECT(entry), "changed",
 g_object_set_data(G_OBJECT(entry), "ptr", (gpointer) model);
 
 /* frame */
-frame = gtk_frame_new(NULL);
-gtk_box_pack_start(GTK_BOX(page), frame, FALSE, FALSE, 0);
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-hbox = gtk_hbox_new(FALSE, 0);
-gtk_container_add(GTK_CONTAINER(frame),hbox);
-gtk_container_set_border_width(GTK_CONTAINER(hbox), PANEL_SPACING);
+hbox = gui_frame_hbox(NULL, FALSE, FALSE, page);
 
 /* optimization cycles */
 label = gtk_label_new(" Maximum cycles ");
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 gui_direct_spin(NULL, &model->gulp.maxcyc, 0, 500, 10, NULL, NULL, hbox);
 
-
 /* NEW - text box for potentials */
 vbox = gtk_vbox_new(FALSE,0);
 label = gtk_label_new(" Potentials ");
 gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);
+
 frame = gtk_frame_new(NULL);
 gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
 swin = gui_text_window(&model->gulp.potentials, TRUE);
@@ -1755,29 +1700,20 @@ gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
 swin = gui_text_window(&model->gulp.species, TRUE);
 gtk_container_add(GTK_CONTAINER(frame), swin);
 
-
-/* NEW - text box for "pass through" GULP data */
+/* text box for "pass through" GULP data */
 vbox = gtk_vbox_new(FALSE,0);
 label = gtk_label_new(" Unprocessed ");
 gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);
 
 /* unprocessed keywords */
-frame = gtk_frame_new(NULL);
-gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox1 = gtk_vbox_new(FALSE,0);
-gtk_container_add(GTK_CONTAINER(frame), vbox1);
+vbox1 = gui_frame_vbox(NULL, TRUE, TRUE, vbox);
+
 hbox = gtk_hbox_new(TRUE, 0);
 gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
 gui_direct_check("Pass keywords through to GULP", &model->gulp.output_extra_keywords, NULL, NULL, hbox);
 gui_text_entry("Keywords  ", &model->gulp.extra_keywords, TRUE, TRUE, vbox1);
 
 /* unprocessed options */
-frame = gtk_frame_new(NULL);
-gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox1 = gtk_vbox_new(FALSE,0);
-gtk_container_add(GTK_CONTAINER(frame), vbox1);
 hbox = gtk_hbox_new(TRUE, 0);
 gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
 gui_direct_check("Pass options through to GULP", &model->gulp.output_extra, NULL, NULL, hbox);
@@ -1790,20 +1726,39 @@ label = gtk_label_new (" Vibrational ");
 gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
 gulp_phonon_box(page, model);
 
+
 /* CURRENT - solvation (cosmic) */
 /* TODO - only available on some versions of GULP */
+/* FIXME - windows version generates GTK CRITICALS when trying to invoke this */
+#ifndef _WIN32
 page = gtk_vbox_new(FALSE,0);
 label = gtk_label_new (" Solvation ");
 gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
 gulp_solvation_box(page, dialog);
+#endif
 
 
+/* job input filename */
+hbox = gui_frame_hbox("Files", FALSE, FALSE, box);
+
+label = gtk_label_new("Job file name  ");
+gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
+entry = gtk_entry_new();
+gtk_box_pack_end(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+
+//gtk_entry_set_text(GTK_ENTRY(entry), model->gulp.temp_file);
+//gtk_entry_set_text(GTK_ENTRY(entry), "");
+
+/* NB: GULP job file name entry has extension removed */
+text = parse_strip_extension(model->gulp.temp_file);
+gtk_entry_set_text(GTK_ENTRY(entry), text);
+g_free(text);
+
+g_signal_connect(GTK_OBJECT(entry), "changed",
+                 GTK_SIGNAL_FUNC(gulp_jobname_changed), (gpointer) model);
+
 /* summary details for the model */
-frame = gtk_frame_new("Details");
-gtk_box_pack_start(GTK_BOX(box),frame,FALSE,FALSE,0);
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-hbox = gtk_hbox_new(FALSE, 0);
-gtk_container_add (GTK_CONTAINER(frame),hbox);
+hbox = gui_frame_hbox("Details", FALSE, FALSE, box);
 
 /* left vbox */
 vbox = gtk_vbox_new(TRUE, 2);
@@ -1911,6 +1866,35 @@ g_assert(model != NULL);
 gui_gulp_task(NULL, model);
 }
 
+/***********************/
+/* grid job submission */
+/***********************/
+void gui_gulp_grid_submit(GtkWidget *w, gpointer data)
+{
+#ifdef WITH_GRISU
+gchar *tmp;
+struct model_pak *model = data;
+struct grid_pak *grid;
+
+g_assert(model != NULL);
+
+grid = grid_new();
+grid->local_cwd = g_strdup(sysenv.cwd);
+grid->local_input = g_strdup(model->gulp.temp_file);
+
+/* immediately write to disk so job is preserved even if user deletes model */
+tmp = g_build_filename(grid->local_cwd, grid->local_input, NULL);
+write_gulp(tmp, model);
+g_free(tmp);
+
+if (!grid_job_submit("gulp", grid))
+  {
+/* TODO - failed message */
+  grid_free(grid);
+  }
+#endif
+}
+
 /*********************************/
 /* cleanup for energetics dialog */
 /*********************************/
@@ -1930,7 +1914,7 @@ model->gulp.eatt_entry = NULL;
 void gulp_dialog(void)
 {
 gpointer dialog;
-GtkWidget *window, *frame, *vbox;
+GtkWidget *window, *vbox;
 struct model_pak *model;
 
 model = sysenv.active_model;
@@ -1945,18 +1929,18 @@ if (!dialog)
   return;
 window = dialog_window(dialog);
 
-frame = gtk_frame_new(NULL);
-gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, FALSE, FALSE, 0);
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+vbox = gui_frame_vbox(NULL, FALSE, FALSE, GTK_DIALOG(window)->vbox);
 
-vbox = gtk_vbox_new(FALSE,0);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
 gui_gulp_widget(vbox, dialog);
 
 /* terminating button */
 gui_stock_button(GTK_STOCK_EXECUTE, gulp_execute, model,
                    GTK_DIALOG(window)->action_area);
 
+#ifdef WITH_GRISU
+gui_icon_button(GTK_STOCK_APPLY, "Grid Submit", gui_gulp_grid_submit, model, GTK_DIALOG(window)->action_area);
+#endif
+
 gui_stock_button(GTK_STOCK_CLOSE, dialog_destroy, dialog,
                    GTK_DIALOG(window)->action_area);
 
