@@ -30,7 +30,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #endif
 
 #include "gdis.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "dialog.h"
 #include "scan.h"
 #include "parse.h"
