@@ -27,6 +27,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include <time.h>
 #include <unistd.h>
 #include <gtk/gtk.h>
+#include <gtk/gtkgl.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -48,11 +49,12 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "opengl.h"
 #include "quaternion.h"
 #include "surface.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
 #include "zmatrix.h"
 #include "gui_image.h"
+#include "undo.h"
 
 #include "logo_left.xpm"
 #include "logo_right.xpm"
@@ -194,13 +196,18 @@ else
 /* selection stuff */
     default:
       select_clear(data);
-/* NEW - don't select if a core is under the mouse */
+/* don't select if a core is under the mouse */
 /* instead we allow the selection box to be drawn */
       update_box(x,y,data,START);
       refresh++;
       break; 
     }
   }
+
+/* CURRENT */
+if (refresh)
+  gui_active_refresh();
+
 return(FALSE);
 }
 
@@ -231,12 +238,10 @@ switch (event->button)
   {
   case 1:
 
-
 /* HACK */
 if (sysenv.stereo)
   return(FALSE);
 
-
 /* clean up after move operations */
   switch(data->mode)
     {
@@ -532,7 +537,7 @@ printf("(%d,%d)\n",dx,dy);
       da = D2R * ROTATE_SCALE * dx;
       if (ctrl)
         {
-        matrix_relative_rotation(mat, da, YAW, data);
+        matrix_relative_rotation(mat, -da, YAW, data);
         rotate_select(data, mat);
         }
       else
@@ -1275,6 +1280,11 @@ void gui_import_project(void)
 file_dialog("Import Project", NULL, FILE_LOAD, (gpointer) import_pcf, PROJECT);
 }
 
+void gui_import_graph(void)
+{
+file_dialog("Import Graph", NULL, FILE_LOAD, (gpointer) graph_read, TEXT);
+}
+
 /*****************/
 /* mode switches */
 /*****************/
@@ -1306,6 +1316,7 @@ static GtkItemFactoryEntry menu_items[] =
   { "/File/Import",              NULL, NULL, 0, "<Branch>" },
   { "/File/Import/Geomview...",  NULL, gui_import_geomview, 1, NULL },
   { "/File/Import/Project...",   NULL, gui_import_project, 1, NULL },
+  { "/File/Import/Graph...",     NULL, gui_import_graph, 1, NULL },
 
   { "/File/Export",                     NULL, NULL, 0, "<Branch>" },
   { "/File/Export/Canvas snapshot...",  NULL, image_export_dialog, 1, NULL },
@@ -1318,8 +1329,13 @@ static GtkItemFactoryEntry menu_items[] =
   { "/Edit/_Copy",          NULL, select_copy, 0, NULL },
   { "/Edit/_Paste",         NULL, select_paste, 0, NULL },
   { "/Edit/sep1",           NULL, NULL, 0, "<Separator>" },
-  { "/Edit/Colour...",      NULL, select_colour, 0, NULL },
+
   { "/Edit/Delete",         NULL, select_delete, 0, NULL },
+  { "/Edit/Undo",           NULL, undo_active, 0, NULL },
+
+  { "/Edit/sep1",           NULL, NULL, 0, "<Separator>" },
+
+  { "/Edit/Colour...",      NULL, select_colour, 0, NULL },
   { "/Edit/Hide",           NULL, select_hide, 0, NULL },
   { "/Edit/Unhide all",     NULL, unhide_atoms, 0, NULL },
   { "/Edit/sep1",           NULL, NULL, 0, "<Separator>" },
@@ -1360,6 +1376,11 @@ static GtkItemFactoryEntry menu_items[] =
   { "/View/Recording mode",         NULL, gui_mode_record, 0, NULL},
   { "/View/sep1",                   NULL, NULL, 0, "<Separator>"},
   { "/View/Task manager...",        NULL, task_dialog, 0, NULL},
+
+#ifdef WITH_GRISU
+  { "/View/Grid manager...",      NULL, gui_grid_dialog, 0, NULL},
+#endif
+
   { "/View/Executable paths...",    NULL, gui_setup_dialog, 0, NULL},
 
   { "/_Help",                  NULL, NULL, 0, "<Branch>"},
@@ -1378,6 +1399,11 @@ gint cb_key_press(GtkWidget *w, GdkEventKey *event, gpointer dummy)
 {
 switch(event->keyval)
   {
+/* selection delete */
+  case GDK_Insert:
+    undo_active();
+    break;
+
 /* selection delete */
   case GDK_Delete:
     select_delete();
@@ -1655,7 +1681,7 @@ active_list = g_list_append(active_list, active);
 /*********/
 /* SETUP */
 /*********/
-void gui_init(void)
+void gui_init(int argc, char *argv[])
 {
 gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
 gchar *text;
@@ -1672,6 +1698,9 @@ GtkStyle *style;
 GtkItemFactory *item;
 GdkColor colour;
 
+gtk_init(&argc, &argv);
+gtk_gl_init(&argc, &argv);
+
 /* enforce true colour (fixes SG problems) */
 sysenv.visual = gdk_visual_get_best_with_type(GDK_VISUAL_TRUE_COLOR);
 if (!sysenv.visual)
@@ -2067,7 +2096,7 @@ text = g_strdup_printf("This is free software, distributed under the terms of th
 gui_text_show(WARNING, text);
 g_free(text);
 
-text = g_strdup_printf("Welcome to GDIS version %4.2f.%d, Copyright (C) 2004 by Sean Fleming\n",VERSION,PATCH); 
+text = g_strdup_printf("Welcome to GDIS version %4.2f.%d, Copyright (C) %d by Sean Fleming\n",VERSION,PATCH,YEAR); 
 gui_text_show(STANDARD, text);
 g_free(text);
 
