@@ -30,12 +30,18 @@ The GNU GPL can also be found at http://www.gnu.org
 #endif
 
 #include "gdis.h"
+#include "file.h"
 #include "coords.h"
+#include "edit.h"
+#include "zone.h"
 #include "matrix.h"
 #include "model.h"
-#include "shortcuts.h"
+#include "render.h"
+#include "select.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
+#include "mdi_pak.h"
 
 #define DEBUG 0
 
@@ -142,6 +148,53 @@ g_free(mdi_data.comp_req);
 g_free(mdi_data.comp_done);
 }
 
+/****************************/
+/* GROMACS genbox solvation */
+/****************************/
+/* return true on succes */
+gint gromacs_genbox(void)
+{
+gchar *genbox, *cmd, *inp, *out;
+struct model_pak *model;
+
+model = sysenv.active_model;
+if (!model)
+  return(FALSE);
+
+genbox = file_find_program("genbox");
+if (!genbox)
+  return(FALSE);
+
+printf("using: %s\n", genbox);
+
+inp = gun("gro");
+out = gun("gro");
+
+write_gromacs(inp, model);
+
+cmd = g_strdup_printf("%s -cs -cp %s -o %s", genbox, inp, out);
+
+system(cmd);
+
+/* TODO - delete old atoms */
+core_delete_all(model);
+
+read_gromacs_gro(out, model);
+
+zone_init(model);
+
+core_render_mode_set(STICK, model->cores);
+
+g_free(cmd);
+
+g_free(inp);
+g_free(out);
+
+g_free(genbox);
+
+return(TRUE);
+}
+
 /********************/
 /* MDI input dialog */
 /********************/
@@ -155,6 +208,10 @@ GString *frame_label;
 GSList *list;
 struct model_pak *data;
 
+/* NEW - hijack if genbox (GROMACS solvator) is found */
+if (gromacs_genbox())
+  return;
+
 /* we need some models to work with */
 if (g_slist_length(sysenv.mal) < 2)
   {
@@ -192,7 +249,6 @@ spinner[0] = gtk_spin_button_new (adj, 0, 0);
 gtk_spin_button_set_wrap(GTK_SPIN_BUTTON (spinner[0]), FALSE);
 gtk_box_pack_start(GTK_BOX (vbox), spinner[0], FALSE, TRUE, 0);
 
-
 g_string_sprintf(frame_label,"Model: %s",data->basename);
 
 frame = gtk_frame_new(frame_label->str);
@@ -354,13 +410,12 @@ struct box_pak *site;
 
 /* allocate for sites - ie solvent not included XCPT corners */
 max_sites = pow(mdi_data.box_dim,3) - mdi_data.comp_req[0] + 8;
-site = (struct box_pak *) g_malloc(max_sites*sizeof(struct box_pak));
+site = g_malloc(max_sites*sizeof(struct box_pak));
 
 r2_min = (gint *) g_malloc(pow(mdi_data.box_dim,3)*sizeof(gint));
 
 /* max out the number of candidate sites */
-cand = (struct cand_pak *) g_malloc(pow(mdi_data.box_dim,3)
-                                   *sizeof(struct cand_pak));
+cand = g_malloc(pow(mdi_data.box_dim,3) *sizeof(struct cand_pak));
 
 /* NB: initially every point in box should be 0 => solvent */
 #if DEBUG
@@ -568,7 +623,7 @@ if (pos >= pow(mdi_data.box_dim,3))
 
       for (list=data->cores ; list ; list=g_slist_next(list))
         {
-        core = (struct core_pak *) list->data;
+        core = list->data;
         if (core->status & DELETED)
           continue;
  
