@@ -39,7 +39,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "render.h"
 #include "spatial.h"
 #include "select.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
 #include "measure.h"
@@ -546,114 +546,6 @@ switch (model->state)
 redraw_canvas(SINGLE);
 }
 
-/************************/
-/* match pairs of atoms */
-/************************/
-#define DEBUG_PAIR_MATCH 0
-gint pair_match(const gchar *label1, const gchar *label2,
-                struct core_pak *core1, struct core_pak *core2)
-{
-gint a, b, i, j, mask;
-
-#if DEBUG_PAIR_MATCH
-printf("[%s,%s] : [%s,%s]\n", label1, label2, core1->label, core2->label);
-#endif
-
-/* if a or b = 0 => any i or j is accepted */
-/* otherwise it must match either i or j */
-/* if a and b are non zero, both i & j must match both a & b */
-a = elem_symbol_test(label1);
-b = elem_symbol_test(label2);
-i = core1->atom_code;
-j = core2->atom_code;
-
-/* fill out the mask */
-mask = 0;
-if (i == a)
-  {
-/* if input label doesn't match the element symbol length - it means the */
-/* user has put in something like H1 - compare this with the atom label */
-  if (g_ascii_strcasecmp(label1, elements[i].symbol) != 0)
-    {
-    if (g_ascii_strcasecmp(core1->atom_label, label1) == 0)
-      mask |= 1;
-    }
-  else
-    mask |= 1; 
-  }
-
-if (j == a)
-  {
-  if (g_ascii_strcasecmp(label1, elements[j].symbol) != 0)
-    {
-    if (g_ascii_strcasecmp(core2->atom_label, label1) == 0)
-      mask |= 2;
-    }
-  else
-    mask |= 2; 
-  }
-
-if (i == b)
-  {
-  if (g_ascii_strcasecmp(label2, elements[i].symbol) != 0)
-    {
-    if (g_ascii_strcasecmp(core1->atom_label, label2) == 0)
-      mask |= 4;
-    }
-  else
-    mask |= 4; 
-  }
-
-if (j == b)
-  {
-  if (g_ascii_strcasecmp(label2, elements[j].symbol) != 0)
-    {
-    if (g_ascii_strcasecmp(core2->atom_label, label2) == 0)
-      mask |= 8;
-    }
-  else
-    mask |= 8; 
-  }
-
-#if DEBUG_PAIR_MATCH
-printf("mask = %d\n", mask);
-#endif
-
-/* if both types must match - only two possibilities (a,b) or (b,a) */
-/* but we can get further matches (than the required 2) when labels are compared */
-if (a && b)
-  {
-  switch(mask)
-    {
-/* single valid pair match */
-    case 6:
-    case 9:
-/* valid pair match plus one extra */
-    case 7:
-    case 11:
-    case 13:
-    case 14:
-/* pair match in all possible combinations */
-    case 15:
-      break;
-/* bad match - exit */
-    default:
-      return(0);
-    }
-  }
-
-/* if only one type to match - any match at all will do */
-if (a || b)
-  if (!mask)
-    return(0);
-
-#if DEBUG_PAIR_MATCH
-printf("accepted [%d][%d] as a match.\n",i,j);
-#endif
-
-return(1);
-}
-
 /***************************/
 /* geometry search - bonds */
 /***************************/
