@@ -42,7 +42,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "surface.h"
 #include "sginfo.h"
 #include "task.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
 #include "opengl.h"
