@@ -40,7 +40,7 @@
 #include "matrix.h"
 #include "surface.h"
 #include "spatial.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
 #include "opengl.h"
