@@ -352,7 +352,9 @@ relation->variable = value;
 relation->model = model;
 relation->widget = widget;
 
-/* if this happens, sysenv.active is not correct  */
+/* this can happen if gui_auto_check() is used on a variable that is not in a model */
+/* eg in sysenv ... in this case use gui_direct_check() */
+/* FIXME - a better way to fail gracefully? better still - rewrite old code */
 if (!direct)
   g_assert(relation->offset > 0);
 
@@ -1191,10 +1193,30 @@ gtk_box_pack_start(GTK_BOX(box), frame, p1, p2, 0);
 gtk_container_set_border_width(GTK_CONTAINER(frame), 0.5*PANEL_SPACING);
 vbox = gtk_vbox_new(FALSE, PANEL_SPACING);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
+gtk_container_set_border_width(GTK_CONTAINER(vbox), 0.5*PANEL_SPACING);
 
 return(vbox);
 }
 
+/**********************************/
+/* hbox in a frame layout shorcut */
+/**********************************/
+GtkWidget *gui_frame_hbox(const gchar *label, gint p1, gint p2, GtkWidget *box)
+{
+GtkWidget *frame, *hbox;
+
+g_assert(box != NULL);
+
+frame = gtk_frame_new(label);
+gtk_box_pack_start(GTK_BOX(box), frame, p1, p2, 0); 
+gtk_container_set_border_width(GTK_CONTAINER(frame), 0.5*PANEL_SPACING);
+hbox = gtk_hbox_new(FALSE, PANEL_SPACING);
+gtk_container_add(GTK_CONTAINER(frame), hbox);
+gtk_container_set_border_width(GTK_CONTAINER(hbox), 0.5*PANEL_SPACING);
+
+return(hbox);
+}
+
 /***************************************************************/
 /* horizontal packing function for label plus some data widget */
 /***************************************************************/
@@ -1219,8 +1241,11 @@ if (text)
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
   gtk_box_pack_end(GTK_BOX(hbox), w, FALSE, FALSE, 0);
   }
-if (w)
-  gtk_box_pack_end(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+else
+  {
+  if (w)
+    gtk_box_pack_end(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+  }
 }
 
 /**************************************/
@@ -1260,7 +1285,12 @@ return(w);
 /*******************************************************/
 gchar *gui_pd_text(gpointer pd)
 {
+#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 6
 return(gtk_combo_box_get_active_text(GTK_COMBO_BOX(pd)));
+#else
+/* you're screwed */
+return(NULL);
+#endif
 }
 
 /**************************************/
