@@ -31,7 +31,7 @@
 #include "edit.h"
 #include "matrix.h"
 #include "sginfo.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
 #include "opengl.h"
@@ -1993,6 +1993,7 @@ if (model->periodic)
       msb_list = g_slist_prepend(msb_list, value);
 */
       }
+
     i++;
     }
   }
@@ -2021,5 +2022,9 @@ else
   }
 
   }
+
+/* fixes data hiding that can occur when columns get resized between models */
+gtk_tree_view_columns_autosize(GTK_TREE_VIEW(tv));
+
 }
 
