@@ -39,7 +39,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "task.h"
 #include "file.h"
 #include "parse.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
 #include "dialog.h"
 
@@ -157,6 +157,11 @@ struct tm start_tm;
 time_t oldest_time;
 FILE *fp;
 
+#ifdef _WIN32
+/* it's just not worth it ... */
+return;
+#endif
+
 /* dispose of task list if it exists */
 if (psarray != NULL)
   g_array_free(psarray, TRUE);
@@ -717,11 +722,14 @@ if (!dialog)
 window = dialog_window(dialog);
 gtk_window_set_default_size(GTK_WINDOW(window), 600, 600);
 
+/* CURRENT - locking this to 1 so grid stuff is limited to 1 task */
+/* this is because Im using global vars (esp soap struct) in the */
+/* grisu_client code - so the whole thing isn't thread safe */
 /* task thread setup */
-hbox = gtk_hbox_new(FALSE, 0);
-gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), hbox, FALSE, FALSE, 0); 
-gtk_container_set_border_width(GTK_CONTAINER(hbox), PANEL_SPACING);
-gui_direct_spin("Maximum number of running tasks", &max_threads, 1, 32, 1, NULL, NULL, hbox);
+//hbox = gtk_hbox_new(FALSE, 0);
+//gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), hbox, FALSE, FALSE, 0); 
+//gtk_container_set_border_width(GTK_CONTAINER(hbox), PANEL_SPACING);
+//gui_direct_spin("Maximum number of running tasks", &max_threads, 1, 32, 1, NULL, NULL, hbox);
 
 #if _WIN32
 gtk_widget_set_sensitive(GTK_WIDGET(hbox), FALSE);
