@@ -33,9 +33,8 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "select.h"
 #include "matrix.h"
 #include "project.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "interface.h"
-#include "opengl.h"
 #include "dialog.h"
 
 #include "methane.xpm"
