@@ -32,7 +32,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "matrix.h"
 #include "zmatrix.h"
 #include "zmatrix_pak.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "dialog.h"
 #include "interface.h"
 #include "zone.h"
