@@ -30,6 +30,8 @@ struct keyword_pak gulp_keyword[] =
 {"conj", CONJ_OPT},
 {"conp", CONP},
 {"conv", CONV},
+/* technically, cellonly */
+{"cell", CELL},
 {"noflag", NOFLAGS},
 {"mole", MOLE},
 {"molm", MOLMEC},
@@ -40,6 +42,7 @@ struct keyword_pak gulp_keyword[] =
 {"comp", COMPARE},
 {"nosym", NOSYM},
 {"fix", FIX},
+{"noauto", GULP_NOAUTOBOND},
 {"cosmi", GULP_SOLVATION_COSMIC},
 {"cosmo", GULP_SOLVATION_COSMO},
 {"phon", PHONON},
@@ -81,6 +84,7 @@ struct keyword_pak gulp_option[] =
 {"equi", EQUILIBRATION},
 {"prod", PRODUCTION},
 {"samp", SAMPLE},
+{"conn", GULP_CONNECT},
 
 /* NEW */
 {"cosmoshape", GULP_COSMO_SHAPE},
