@@ -33,7 +33,6 @@ struct keyword_pak keywords[] =
 {"cell", CELL},
 {"cart", CART},
 {"frac", FRAC},
-{"sfrac", S_FRAC},
 {"spac", SPACE},
 {"orig", ORIGIN},
 {"spec", SPECIES},
@@ -51,16 +50,6 @@ struct keyword_pak keywords[] =
 /*
 {"shrink", SHRINK},
 */
-{"harm", HARMONIC},
-{"mors", MORSE},
-{"buck", BUCKINGHAM},
-{"lenn", LENNARD},
-{"thre", THREE},
-{"urey", UREYBRADLEY},
-{"four", FOUR},
-{"tors", TORSION},
-{"outofp", OUTOFPLANE},
-{"spri", SPRING},
 {"switch", SWITCH_ON},
 {"gnorm", GNORM},
 {"cycle", CYCLE},
