@@ -203,7 +203,7 @@ if (fsetpos(fp, offset))
 
 /* atom counter */
 n=-1;
-new = first = data->number;
+new = first = 0;
 data->id = -1;
 for(;;)
   {
@@ -261,7 +261,7 @@ goto cif_done;
           }
         
 #if DEBUG_LOAD_CIF
-printf("Start of new model %d: %s.\n",data->number,data->basename);
+printf("Start of new model: %s.\n", data->basename);
 #endif
         break;
 
