@@ -33,15 +33,16 @@ The GNU GPL can also be found at http://www.gnu.org
 #ifndef __WIN32
 #include <sys/wait.h>
 #endif
-#include <gtk/gtkgl.h>
 
 #include "gdis.h"
+#include "command.h"
 #include "file.h"
 #include "parse.h"
 #include "task.h"
+#include "host.h"
 #include "render.h"
 #include "matrix.h"
-#include "opengl.h"
+#include "model.h"
 #include "numeric.h"
 #include "module.h"
 #include "library.h"
@@ -63,14 +64,22 @@ free_slist((GSList *) sfc_list);
 /******************/
 /* system cleanup */
 /******************/
-/* TODO - other things to free? */
 void sys_free(void)
 {
+
+file_free();
+
+free_slist(sysenv.render.light_list);
+
 g_hash_table_destroy(sysenv.sfc_table);
 g_hash_table_destroy(sysenv.library);
 g_hash_table_destroy(sysenv.manual);
 g_hash_table_destroy(sysenv.image_table);
 module_free();
+
+#ifndef _WIN32
+host_free_all();
+#endif
 }
 
 /**********************************/
@@ -154,10 +163,8 @@ for (;;)
 
 /* povray executable */
   if (g_ascii_strncasecmp("povray_p", *buff, 8) == 0)
-    {
-    if (num_tokens > 1)
-      sysenv.povray_path = g_strdup(*(buff+1));
-    }
+    sysenv.povray_path = parse_strip_newline(&line[12]);
+
   if (g_ascii_strncasecmp("povray_e", *buff, 8) == 0)
     {
     if (num_tokens > 1)
@@ -166,12 +173,11 @@ for (;;)
       sysenv.povray_exe = g_strdup(*(buff+1));
       }
     }
+
 /* animation creation tool */
   if (g_ascii_strncasecmp("convert_p", *buff, 9) == 0)
-    {
-    if (num_tokens > 1)
-      sysenv.convert_path = g_strdup(*(buff+1));
-    }
+    sysenv.convert_path = parse_strip_newline(&line[13]);
+
   if (g_ascii_strncasecmp("convert_e", *buff, 9) == 0)
     {
     if (num_tokens > 1)
@@ -180,12 +186,11 @@ for (;;)
       sysenv.convert_exe = g_strdup(*(buff+1));
       }
     }
+
 /* image viewing */
   if (g_ascii_strncasecmp("viewer_p", *buff, 8) == 0)
-    {
-    if (num_tokens > 1)
-      sysenv.viewer_path = g_strdup(*(buff+1));
-    }
+    sysenv.viewer_path = parse_strip_newline(&line[12]);
+
   if (g_ascii_strncasecmp("viewer_e", *buff, 8) == 0)
     {
     if (num_tokens > 1)
@@ -194,12 +199,11 @@ for (;;)
       sysenv.viewer_exe = g_strdup(*(buff+1));
       }
     }
-/* gulp */
+
+/* GULP */
   if (g_ascii_strncasecmp("gulp_p", *buff, 6) == 0)
-    {
-    if (num_tokens > 1)
-      sysenv.gulp_path = g_strdup(*(buff+1));
-    }
+    sysenv.gulp_path = parse_strip_newline(&line[10]);
+
   if (g_ascii_strncasecmp("gulp_e", *buff, 6) == 0)
     {
     if (num_tokens > 1)
@@ -371,6 +375,7 @@ FILE *fp;
 /* top level structure initialization */
 sysenv.max_threads = 1;
 sysenv.task_list = NULL;
+sysenv.host_list = NULL;
 sysenv.dialog_list = NULL;
 sysenv.glconfig = NULL;
 sysenv.stereo = FALSE;
@@ -421,6 +426,7 @@ sysenv.render.width = 600;
 sysenv.render.height = 600;
 sysenv.render.vp_dist = 5.0;
 sysenv.render.zone_size = 10.0;
+sysenv.render.show_energy = FALSE;
 
 /* stereo defaults */
 sysenv.render.stereo_quadbuffer = FALSE;
@@ -625,9 +631,15 @@ if (!sysenv.gulp_path)
   sysenv.gulp_path = g_find_program_in_path(sysenv.gulp_exe);
 if (!sysenv.povray_path)
   sysenv.povray_path = g_find_program_in_path(sysenv.povray_exe);
+
+/* display program */
 if (!sysenv.viewer_path)
   sysenv.viewer_path = g_find_program_in_path(sysenv.viewer_exe);
 
+/* alternative (FIXME - starting to get ugly - redo) */
+if (!sysenv.viewer_path)
+  sysenv.viewer_path = g_find_program_in_path("eog");
+
 /* HACK FIX - GAMESS path contains the path, not the path + file */
 temp = g_find_program_in_path(sysenv.gamess_exe);
 
@@ -654,7 +666,9 @@ sysenv.surfaces = NULL;
 
 /* module */
 file_init();
+#ifdef WITH_GUI
 help_init();
+#endif
 init_math();
 library_init();
 }
@@ -664,8 +678,6 @@ library_init();
 /********/
 gint main(int argc, char *argv[])
 {
-gint i;
-
 /* NBL read this 1st as it affects canvas type, but command */
 /* line arguments should be able to overide */
 sys_init(argc, argv);
@@ -677,17 +689,24 @@ if (argc > 1)
   if (g_ascii_strncasecmp(argv[1],"-c",3) == 0)
     sysenv.canvas = FALSE;
 
+
+#ifdef WITH_GUI
 /* set up main window and event handlers (or start non-gui mode) */
 if (sysenv.canvas)
   {
+  gint i;
+
 /* initialize threads and set up the queue */
   task_queue_init();
 
+/* CURRENT - absorb these into gui_init() */
+/*
   gtk_init(&argc, &argv);
   gtk_gl_init(&argc, &argv);
+*/
 
 /* main interface */
-  gui_init();
+  gui_init(argc, argv);
 
 /* task update timer */
 /* TODO - put this in gui_widget_handler? */
@@ -709,7 +728,74 @@ if (sysenv.canvas)
   gtk_main();
   gdk_threads_leave();
   }
+else
+#else
+sysenv.canvas = FALSE;
+#endif
+  command_main_loop(argc, argv);
 
 return(0);
 }
 
+/* CURRENT */
+/* routines that are not cleanly seperable when we build with no GUI */
+#ifndef WITH_GUI
+void gui_text_show(gint type, gchar *msg)
+{
+printf("%s", msg);
+}
+
+void gui_refresh(gint dummy)
+{
+}
+
+void tree_select_delete(void)
+{
+}
+void tree_select_active(void)
+{
+}
+void tree_select_model(struct model_pak *m)
+{
+}
+void tree_model_add(struct model_pak *m)
+{
+}
+void tree_model_refresh(struct model_pak *m)
+{
+}
+
+void dialog_destroy_type(gint dummy)
+{
+}
+
+gpointer graph_new(const gchar *dummy, struct model_pak *m)
+{
+return(NULL);
+}
+
+void graph_add_data(gint a, gdouble *b, gdouble c, gdouble d, gpointer e)
+{
+}
+
+void graph_set_yticks(gint a, gint b, gpointer c)
+{
+}
+
+void graph_free_list(struct model_pak *m)
+{
+}
+
+void meas_graft_model(struct model_pak *m)
+{
+}
+
+void meas_prune_model(struct model_pak *m)
+{
+}
+
+/* FIXME - this should actually be replaced by gui_refresh() */
+void redraw_canvas(gint dummy)
+{
+}
+#endif
