@@ -810,11 +810,24 @@ printf("constructing latmat...\n");
 /* this basically works by using the cosine rule in conjunction */
 /* with a few geometric constraints (eg normalized vectors) */
 
+/* FIXME - correction needed for 1D case as well? */
+if (data->periodic == 2)
+  {
+/* lattice code requires non existent c parameter to be 1 */
+  data->pbc[2] = 1.0;
+  }
+
 /* compute the translation vector for b */
   b1 = cos(data->pbc[5]);
   b2 = sin(data->pbc[5]);
   b3 = 0.0;              /* constrain a,b to remain on the x,y plane */
 
+  if (b2 == 0.0)
+    {
+    printf("matrix_lattice_init(): bad cell parameters.\n");
+    b2 = 1.0;
+    }
+
 /* compute the translation vector for c */
   c1 = cos(data->pbc[4]);
   c2 = (2.0*cos(data->pbc[3]) + b1*b1 + b2*b2 - 2.0*b1*c1 - 1.0)/(2.0*b2);
