@@ -27,6 +27,7 @@ enum {IDENTITY, INVERSION, PLANE, PAXIS, IAXIS, REFLECTION, ALIGNMENT, LATMAT};
 
 /* dim 4 */
 #define VEC4SET(a,x,y,z,t) {*a = x; *(a+1) = y; *(a+2) = z; *(a+3) = t;}
+#define VEC4MUL(v,m) {*v *= m; *(v+1) *= m; *(v+2) *= m; *(v+3) *= m;}
 #define ARR4SET(a,b) {*a=*b; *(a+1)=*(b+1); *(a+2)=*(b+2); *(a+3)=*(b+3);}
 
 #define P4MAT(s,m) {printf("%s\n|%f, %f, %f, %f|\n|%f, %f, %f, %f|\n\
@@ -36,22 +37,28 @@ enum {IDENTITY, INVERSION, PLANE, PAXIS, IAXIS, REFLECTION, ALIGNMENT, LATMAT};
 
 
 /* matrix/vector manipulation */
-gint invmat(gdouble *);
 void vecmat(gdouble *, gdouble *);
 void vectmat(gdouble *, gdouble *);
 void vec4mat(gdouble *, gdouble *);
 void matmat(gdouble *, gdouble *);
 void mat4mat(gdouble *, gdouble *);
-void transpose(gdouble *);
-void transpose4(gdouble *);
+gint matrix_invert(gdouble *);
+void matrix_transpose(gdouble *);
+void matrix_transpose_44(gdouble *);
 
 void calc_norm(gdouble *, gdouble *, gdouble *, gdouble *);
 
+void matrix_lattice_init(struct model_pak *);
+void matrix_lattice_new(gdouble *, struct model_pak *);
+void matrix_identity(gdouble *);
+void matrix_rotation(gdouble *, gdouble , gint);
 
-/* TODO - replace with make_symmat */
-void get_rot_matrix(gdouble *, gint, gdouble *, gint);
+void matrix_z_alignment(gdouble *, gdouble *);
+void matrix_v_alignment(gdouble *, gdouble *, gdouble *);
+void matrix_v_rotation(gdouble *, gdouble *, gdouble);
+void matrix_v_reflection(gdouble *, gdouble *);
 
-void make_symmat(gdouble *, gdouble *, gpointer *, gint, gdouble);
+void matrix_relative_rotation(gdouble *, gdouble, gint, struct model_pak *);
 
 void crossprod(gdouble *, gdouble *, gdouble *);
 void proj_vop(gdouble *, gdouble *, gdouble *);
@@ -61,12 +68,11 @@ gdouble calc_volume(gdouble *);
 gint normalize(gdouble *, gint);
 gdouble magnitude(gdouble *, gint);
 gdouble via(gdouble *, gdouble *, gint);
-gdouble det(gdouble *);
 
-/* symmetry calls */
-gint is_inversion(gdouble *);
-gint is_identity(gdouble *);
-gint order(gdouble *);
+gdouble matrix_determinant(gdouble *);
 
-void new_lattice(gdouble *, struct model_pak *);
+gint matrix_is_inversion(gdouble *);
+gint matrix_is_identity(gdouble *);
+gint matrix_is_z_rotation(gdouble *);
+gint matrix_order(gdouble *);
 
