@@ -320,7 +320,7 @@ if (n)
   core2 = mp->core[n];
   ARR3SET(dx, core1->x);
   ARR3SUB(dx, core2->x);
-  fractional_minsq(dx, model->periodic);
+  fractional_min(dx, model->periodic);
   vecmat(model->latmat, dx);
   ARR3SUB(x, dx);
   }
@@ -507,6 +507,49 @@ ARR3SUB(b, x2);
 return(R2D*via(a,b,3)); 
 }
 
+/**********************************/
+/* dihedral calculation primitive */
+/**********************************/
+gdouble measure_dihedral(gdouble *x1, gdouble *x2, gdouble *x3, gdouble *x4)
+{
+gdouble len, sign;
+gdouble a[3], b[3], c[3], n[3];
+
+/* compute end atom vectors (from 1-2 axis) */
+ARR3SET(a, x1);
+ARR3SUB(a, x2);
+ARR3SET(b, x4);
+ARR3SUB(b, x3);
+
+/* compute normal to the plane in which the dihedral is to be calc'd */
+ARR3SET(n, x2);
+ARR3SUB(n, x3);
+normalize(n, 3);
+
+/* CURRENT - compute the direction sense (sign) */
+crossprod(c, a, b);
+if (via(c,n,3) < 0.5*PI)
+  sign = -1.0;
+else
+  sign = 1.0;
+
+/* n[0..2] is the normal of the plane */
+/* project 1-0 onto the normal (ie dist to plane) */
+len = a[0]*n[0] + a[1]*n[1] + a[2]*n[2];
+/* subtract vector to plane from 1-0 to get projection on plane */
+a[0] -= len*n[0];
+a[1] -= len*n[1];
+a[2] -= len*n[2];
+/* project 2-3 onto the normal */
+len = b[0]*n[0] + b[1]*n[1] + b[2]*n[2];
+/* subtract vector to plane from 2-3 to get projection on plane */
+b[0] -= len*n[0];
+b[1] -= len*n[1];
+b[2] -= len*n[2];
+/* compute angle between projected vectors */
+return(sign * R2D*via(a,b,3)); 
+}
+
 /*********************************************/
 /* recompute a measurement value for a model */
 /*********************************************/
@@ -727,7 +770,7 @@ P3VEC("xlat2:", &xlat[4]);
 
 /* test if distance satisfies the cutoffs */
 ARR3SUB(x1, x2);
-fractional_minsq(x1, model->periodic);
+fractional_min(x1, model->periodic);
 vecmat(model->latmat, x1);
 rsq = VEC3MAGSQ(x1);
 
@@ -1280,3 +1323,111 @@ if (angle >= a1 && angle <= a2)
   }
 return(m);
 }
+
+/************************/
+/* match pairs of atoms */
+/************************/
+#define DEBUG_PAIR_MATCH 0
+gint pair_match(const gchar *label1, const gchar *label2,
+                struct core_pak *core1, struct core_pak *core2)
+{
+gint a, b, i, j, mask;
+
+#if DEBUG_PAIR_MATCH
+printf("[%s,%s] : [%s,%s]\n", label1, label2, core1->label, core2->label);
+#endif
+
+/* if a or b = 0 => any i or j is accepted */
+/* otherwise it must match either i or j */
+/* if a and b are non zero, both i & j must match both a & b */
+a = elem_symbol_test(label1);
+b = elem_symbol_test(label2);
+i = core1->atom_code;
+j = core2->atom_code;
+
+/* fill out the mask */
+mask = 0;
+if (i == a)
+  {
+/* if input label doesn't match the element symbol length - it means the */
+/* user has put in something like H1 - compare this with the atom label */
+  if (g_ascii_strcasecmp(label1, elements[i].symbol) != 0)
+    {
+    if (g_ascii_strcasecmp(core1->atom_label, label1) == 0)
+      mask |= 1;
+    }
+  else
+    mask |= 1; 
+  }
+
+if (j == a)
+  {
+  if (g_ascii_strcasecmp(label1, elements[j].symbol) != 0)
+    {
+    if (g_ascii_strcasecmp(core2->atom_label, label1) == 0)
+      mask |= 2;
+    }
+  else
+    mask |= 2; 
+  }
+
+if (i == b)
+  {
+  if (g_ascii_strcasecmp(label2, elements[i].symbol) != 0)
+    {
+    if (g_ascii_strcasecmp(core1->atom_label, label2) == 0)
+      mask |= 4;
+    }
+  else
+    mask |= 4; 
+  }
+
+if (j == b)
+  {
+  if (g_ascii_strcasecmp(label2, elements[j].symbol) != 0)
+    {
+    if (g_ascii_strcasecmp(core2->atom_label, label2) == 0)
+      mask |= 8;
+    }
+  else
+    mask |= 8; 
+  }
+
+#if DEBUG_PAIR_MATCH
+printf("mask = %d\n", mask);
+#endif
+
+/* if both types must match - only two possibilities (a,b) or (b,a) */
+/* but we can get further matches (than the required 2) when labels are compared */
+if (a && b)
+  {
+  switch(mask)
+    {
+/* single valid pair match */
+    case 6:
+    case 9:
+/* valid pair match plus one extra */
+    case 7:
+    case 11:
+    case 13:
+    case 14:
+/* pair match in all possible combinations */
+    case 15:
+      break;
+/* bad match - exit */
+    default:
+      return(0);
+    }
+  }
+
+/* if only one type to match - any match at all will do */
+if (a || b)
+  if (!mask)
+    return(0);
+
+#if DEBUG_PAIR_MATCH
+printf("accepted [%d][%d] as a match.\n",i,j);
+#endif
+
+return(1);
+}
