@@ -75,5 +75,8 @@ void measure_update_global(struct model_pak *);
 
 gdouble measure_distance(gdouble *, gdouble *);
 gdouble measure_angle(gdouble *, gdouble *, gdouble *);
+gdouble measure_dihedral(gdouble *, gdouble *, gdouble *, gdouble *);
+
+/* bonded case of dihedral */
 gdouble measure_torsion(struct core_pak **);
 
