@@ -41,15 +41,16 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "project.h"
 #include "analysis.h"
 #include "render.h"
+#include "opengl.h"
 #include "select.h"
 #include "space.h"
 #include "surface.h"
 #include "spatial.h"
-#include "opengl.h"
 #include "interface.h"
 #include "dialog.h"
 #include "zone.h"
 #include "zmatrix.h"
+#include "undo.h"
 
 /* the main pak structures */
 extern struct sysenv_pak sysenv;
@@ -107,6 +108,7 @@ data->ghosts = NULL;
 data->images = NULL;
 data->camera = NULL;
 data->camera_default = NULL;
+data->ff_list = NULL;
 data->layer_list = NULL;
 data->measure_list = NULL;
 data->graph_list = NULL;
@@ -124,8 +126,6 @@ data->zmatrix = zmat_new();
 data->show_names = FALSE;
 data->show_title = TRUE;
 data->show_frame_number = TRUE;
-data->show_hbonds = FALSE;
-data->show_energy = FALSE;
 data->show_charge = FALSE;
 data->show_atom_charges = FALSE;
 data->show_atom_labels = FALSE;
@@ -134,16 +134,27 @@ data->show_atom_index = FALSE;
 data->show_geom_labels = TRUE;
 data->show_cores = TRUE;
 data->show_shells = FALSE;
+data->show_bonds = TRUE;
+data->show_hbonds = FALSE;
+data->show_links = FALSE;
 data->show_axes = TRUE;
 data->show_cell = TRUE;
 data->show_cell_images = TRUE;
 data->show_cell_lengths = FALSE;
 data->show_waypoints = TRUE;
+data->show_selection_labels = FALSE;
+
+/*VZ*/
+data->show_nmr_shifts = FALSE;
+data->show_nmr_csa = FALSE;
+data->show_nmr_efg = FALSE;
 
 /* NEW */
 data->property_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, property_free);
 data->property_list = NULL;
 
+undo_init(data);
+
 data->ribbons = NULL;
 data->spatial = NULL;
 data->phonons = NULL;
@@ -158,6 +169,7 @@ data->num_phonons = 0;
 data->show_eigenvectors = FALSE;
 data->phonon_movie = FALSE;
 data->phonon_movie_name = g_strdup("phonon_movie");
+data->phonon_movie_type = g_strdup("gif");
 data->gulp.phonon = FALSE;
 data->gulp.kpoints = NULL;
 
@@ -223,7 +235,6 @@ data->region[REGION2B] = FALSE;
 data->morph_type = DHKL;
 data->num_vertices = 0;
 data->num_planes = 0;
-data->vertices = NULL;
 data->sculpt_shift_use = FALSE;
 
 data->basename = g_strdup_printf("model_%d", g_slist_index(sysenv.mal, data));
@@ -231,114 +242,11 @@ strcpy(data->filename, data->basename);
 n++;
 
 /* gulp template init */
-data->gulp.maxcyc=0;
-data->gulp.energy = 0.0 ;
-data->gulp.no_esurf=FALSE;
-data->gulp.esurf[0]=0.0;
-data->gulp.esurf[1]=0.0;
-data->gulp.esurf_units=g_strdup(" ");
-data->gulp.no_eatt=FALSE;
-data->gulp.eatt[0]=0.0;
-data->gulp.eatt[1]=0.0;
-data->gulp.eatt_units=g_strdup(" ");
-data->gulp.sbulkenergy=0.0;
-data->gulp.sdipole=999999999.0;
-data->gulp.gnorm = -1.0;
-data->gulp.qsum = 0.0;
-data->gulp.no_exec = FALSE;
-data->gulp.run = E_SINGLE;
-data->gulp.rigid = FALSE;
-data->gulp.rigid_move = g_strdup("z");
-data->gulp.qeq = FALSE;
-data->gulp.free = FALSE;
-data->gulp.zsisa = FALSE;
-data->gulp.compare = FALSE;
-data->gulp.prop = FALSE;
-data->gulp.nosym = FALSE;
-data->gulp.fix = FALSE;
-data->gulp.method = CONP;
-/* dynamics info */
-data->gulp.frame_time = 0.0;
-data->gulp.frame_ke = 0.0;
-data->gulp.frame_pe = 0.0;
-data->gulp.frame_temp = 0.0;
-/* NEW - original frame data (stored since reading in trajectory overwrites) */
-data->gulp.orig_fractional = FALSE;
-data->gulp.orig_construct_pbc = FALSE;
-data->gulp.orig_cores = NULL;
-data->gulp.orig_shells = NULL;
-data->gulp.potentials = NULL;
-data->gulp.elements = NULL;
-data->gulp.species = NULL;
-/* potgrid replacement */
-data->gulp.epot_vecs = NULL;
-data->gulp.epot_vals = NULL;
-data->gulp.epot_min = 999999999.9;
-data->gulp.epot_max = -999999999.9;
-
-data->gulp.optimiser = -1;
-data->gulp.optimiser2 = SWITCH_OFF;
-data->gulp.switch_type = -1;
-data->gulp.switch_value = 0.1;
-data->gulp.unit_hessian = FALSE;
-data->gulp.ensemble = -1;
-data->gulp.pressure = NULL;
-data->gulp.temperature = NULL;
-VEC3SET(data->gulp.super, 1, 1, 1);
-data->gulp.coulomb = NOBUILD;
-data->gulp.libfile = NULL;
-data->gulp.output_extra_keywords = TRUE;
-data->gulp.output_extra = FALSE;
-data->gulp.extra_keywords = NULL;
-data->gulp.extra = NULL;
-
-data->gulp.timestep = NULL;
-data->gulp.equilibration = NULL;
-data->gulp.production = NULL;
-data->gulp.sample = NULL;
-data->gulp.write = NULL;
-
 /* NEW - COSMO init ... TODO - include all GULP inits */
 gulp_init(&data->gulp);
 
-/* io filenames */
-data->gulp.temp_file = NULL;
-data->gulp.dump_file = NULL;
-data->gulp.out_file = NULL;
-data->gulp.trj_file = NULL;
-data->gulp.mov_file = NULL;
-
-/* io options */
-data->gulp.print_charge = TRUE;
-
 /* GAMESS defaults */
-data->gamess.units = GMS_ANGS;
-data->gamess.exe_type = GMS_RUN;
-data->gamess.run_type = GMS_ENERGY;
-data->gamess.scf_type = GMS_RHF;
-data->gamess.basis = GMS_MNDO;
-data->gamess.dft = FALSE;
-data->gamess.dft_functional = 0;
-data->gamess.ngauss = 0;
-data->gamess.opt_type = GMS_QA;
-data->gamess.total_charge = 0;
-data->gamess.multiplicity = 1;
-data->gamess.time_limit = 600;
-data->gamess.mwords = 1;
-data->gamess.num_p = 0;
-data->gamess.num_d = 0;
-data->gamess.num_f = 0;
-data->gamess.have_heavy_diffuse = FALSE;
-data->gamess.have_hydrogen_diffuse = FALSE;
-data->gamess.converged = FALSE;
-data->gamess.wide_output = FALSE;
-data->gamess.nstep = 20;
-data->gamess.maxit = 30;
-data->gamess.title = g_strdup("none");
-data->gamess.temp_file = g_strdup("none");
-data->gamess.out_file = g_strdup("none");
-data->gamess.energy = data->gamess.max_grad = data->gamess.rms_grad = 0.0;
-data->gamess.have_energy = data->gamess.have_max_grad = data->gamess.have_rms_grad = FALSE;
+gamess_init(&data->gamess);
 
 data->abinit.energy = data->abinit.max_grad = data->abinit.rms_grad = 0.0;
 
@@ -676,7 +584,7 @@ gint model_prep(struct model_pak *data)
 g_return_val_if_fail(data != NULL, 1);
 
 #if DEBUG_PREP_MODEL
-printf("start prep: %d\n", data->number);
+printf("start prep: %p\n", data);
 #endif
 
 error_table_clear();
@@ -823,6 +731,9 @@ if ((num_frames - data->num_frames) > FRAME_EPS)
   }
 fclose(fp);
 
+/* NEW - attempt to mark the start of all frames (for faster random access) */
+      mark_trj_frames(data);
+
       }
     g_free(filename);
     }
@@ -838,14 +749,14 @@ printf(" num atoms: %d\n", data->num_atoms);
 printf("num shells: %d\n", data->num_shells);
 #endif
 
-/* display coords & connectivity */
+/* initial center/coord calc */
 coords_init(INIT_COORDS, data);
 
+/* connectivity */
 connect_bonds(data);
-
 connect_molecules(data);
 
-/* init the colour scheme */
+/* assign colours */
 model_colour_scheme(data->colour_scheme, data);
 
 /* final periodicity setup */
@@ -853,17 +764,18 @@ if (data->periodic == 2)
   {
 /* order by z, unless input core order should be kept */
   if (!data->surface.keep_atom_order)
-    {
     sort_coords(data);
-    coords_init(CENT_COORDS, data);
-    }
   }
 
-/* set up the camera - should be after any coord centering etc. */
-camera_init(data);
+/* final recenter/coord calc (after molecule unfragmenting/surface sorting) */
+coords_init(CENT_COORDS, data);
+
+/* set up the camera - should be done after all coord centering etc. */
+if (sysenv.canvas)
+  camera_init(data);
 
 #if DEBUG_PREP_MODEL
-printf("end prep: %d\n", data->number);
+printf("end prep: %p\n", data);
 #endif
 
 error_table_print_all();
@@ -871,6 +783,34 @@ error_table_print_all();
 return(0);
 }
 
+/*************************************************/
+/* store the current position of the file stream */
+/*************************************************/
+#define DEBUG_ADD_FRAME_OFFSET 0
+gint add_frame_offset(FILE *fp, struct model_pak *data)
+{
+fpos_t *offset;
+
+g_assert(fp != NULL);
+g_assert(data != NULL);
+
+offset = g_malloc(sizeof(fpos_t));
+/* prepend & reverse? */
+if (!fgetpos(fp, offset))
+  data->frame_list = g_list_append(data->frame_list, offset); 
+else
+  {
+  g_free(offset);
+  return(1);
+  }
+
+#if DEBUG_ADD_FRAME_OFFSET
+printf("stored: %p\n", offset);
+#endif
+
+return(0);
+}
+
 /*********************************************************/
 /* convenience routine for duplicating a list of strings */
 /*********************************************************/
@@ -894,6 +834,9 @@ void free_slist(GSList *list)
 {
 GSList *item;
 
+if (!list)
+  return;
+
 for (item=list ; item ; item=g_slist_next(item))
   g_free(item->data);
 
@@ -966,6 +909,7 @@ free_slist(data->ir_list);
 free_slist(data->raman_list);
 free_slist(data->waypoint_list);
 free_slist(data->transform_list);
+free_slist(data->ff_list);
 
 /* camera */
 g_free(data->camera_default);
@@ -1000,9 +944,6 @@ g_slist_free(data->planes);
 data->planes = NULL;
 data->num_planes = 0;
 
-/* TODO - free adj list (not data) for each vertices */
-free_slist(data->vertices);
-
 /* spatial objects */
 spatial_destroy_all(data);
 
@@ -1020,7 +961,8 @@ if (data->afp)
   fclose(data->afp);
 */
 
-analysis_free(data);
+analysis_free(data->analysis);
+data->analysis = NULL;
 
 g_string_free(data->error_file_read, TRUE);
 }
@@ -1031,7 +973,6 @@ g_string_free(data->error_file_read, TRUE);
 void model_delete(struct model_pak *model)
 {
 GSList *list;
-struct model_pak *m;
 struct canvas_pak *canvas;
 
 /* checks */
@@ -1056,20 +997,14 @@ else
     if (canvas->model == model)
       canvas->model = NULL;
     }
-/* shuffle the data->number to account */
-/* for the deletion of model */
-  for (list=sysenv.mal ; list ; list=g_slist_next(list))
-    {
-    m = list->data;
-    if (m == model)
-      break;
-    m->number--; 
-    }
+
   sysenv.mal = g_slist_remove(sysenv.mal, model);
   }
 
+#ifdef WITH_GUI
 /* destroy any associated dialogs */
 dialog_destroy_model(model);
+#endif
 
 /* free the model's pointers */
 model_free(model);
@@ -1082,6 +1017,75 @@ canvas_shuffle();
 redraw_canvas(ALL);
 }
 
+/**********************************/
+/* update the model/canvas layout */
+/**********************************/
+/* TODO - rename - eg model_active_shuffle() */
+void canvas_shuffle(void)
+{
+gint c, m;
+GSList *clist, *mlist;
+struct canvas_pak *canvas;
+
+/* find active model in canvas list */
+c = 0;
+mlist = NULL;
+for (clist=sysenv.canvas_list ; clist ; clist=g_slist_next(clist))
+  {
+  canvas = clist->data;
+  if (sysenv.active_model)
+    {
+
+    if (!canvas->model &&!mlist)
+      {
+      canvas->model = sysenv.active_model;
+      }
+
+    if (canvas->model == sysenv.active_model)
+      {
+      m = g_slist_index(sysenv.mal, canvas->model);
+      if (m < c)
+        {
+/* not enough prior models to display active model at current canvas poisiton */
+/* start at active model */
+/* TODO - start at low enough model number so that active model is displayed */
+        mlist = g_slist_find(sysenv.mal, sysenv.active_model);
+        }
+      else
+        {
+/* we have enough prior models to display active model at current canvas poisiton */
+        mlist = g_slist_nth(sysenv.mal, m-c);
+        }
+      }
+
+    }
+  c++;
+  }
+
+/* active model not in the canvas list */
+if (sysenv.active_model && !mlist)
+  {
+/* start at active model */
+  mlist = g_slist_find(sysenv.mal, sysenv.active_model);
+  }
+
+/* fill the canvas list */
+if (mlist)
+  {
+  for (clist=sysenv.canvas_list ; clist ; clist=g_slist_next(clist))
+    {
+    canvas = clist->data;
+    if (mlist)
+      {
+      canvas->model = mlist->data;
+      mlist = g_slist_next(mlist);
+      }
+    else
+      canvas->model = NULL;
+    }
+  }
+}
+
 /*******************************/
 /* replacement for ASSIGN call */
 /*******************************/
