@@ -39,3 +39,6 @@ gchar *property_label(gpointer);
 gchar *property_value(gpointer);
 gchar *property_lookup(gchar *, struct model_pak *);
 
+void gulp_init(gpointer);
+void gamess_init(gpointer);
+
