@@ -140,7 +140,7 @@ printf("Attempting to load: %s\n", fullname);
 gmodule = g_module_open(fullname, 0);
 if (gmodule)
   {
-  basename = strdup_basename(name);
+  basename = parse_strip(name);
   symbol = g_strdup_printf("module_%s_config", basename);
   if (g_module_symbol(gmodule, symbol, &ptr))
     {
@@ -196,7 +196,7 @@ gchar *name, *ext;
 GSList *dir, *list;
 
 /* search gdis directory for valid plug-ins */
-dir = get_dir_list(sysenv.gdis_path, FALSE);
+dir = file_dir_list(sysenv.gdis_path, FALSE);
 for (list=dir ; list ; list=g_slist_next(list))
   {
   name = list->data;
