@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -31,3 +31,9 @@ gchar *module_label(gpointer);
 GSList *module_symbols(gpointer);
 gint module_invoke(gchar *, gpointer, gpointer);
 
+/* Hirshfeld surface prototypes */
+void hfs_init(void);
+gdouble hfs_calc_wf(gdouble *, struct model_pak *, GSList *);
+GSList *hfs_calc_env(GSList *, struct model_pak *);
+int hfs_calc_normals(GSList *, struct model_pak *, GSList *);
+int hfs_calulation_limits(GSList *selection, struct model_pak *, gdouble *min, gdouble *max);
