@@ -222,7 +222,7 @@ for (list=core_list ; list ; list=g_slist_next(list))
 /* get minimum distance from core's VDW surface to point */
   ARR3SET(x, core->x);
   ARR3SUB(x, f);
-  fractional_minsq(x, ms_data->periodic);
+  fractional_min(x, ms_data->periodic);
   vecmat(ms_data->latmat, x);
   a = elements[core->atom_code].vdw;
 
@@ -272,7 +272,7 @@ for (list=cores ; list ; list=g_slist_next(list))
 /* get minimium separation vector */
   ARR3SET(x, r);
   ARR3SUB(x, core->x);
-  fractional_minsq(x, ms_data->periodic);
+  fractional_min(x, ms_data->periodic);
   vecmat(ms_data->latmat, x);
   dx = VEC3MAG(x);
 
@@ -488,6 +488,13 @@ struct smv_pak *point;
 /* checks */
 g_assert(model != NULL);
 
+/* NEW - if we have an existing set of the same size - don't recalculate */
+/* TODO - make this an option */
+if (g_slist_length(list) == g_slist_length(model->gulp.epot_vecs))
+  {
+  return(0);
+  }
+
 /* setup the site coordinate list */
 if (model->gulp.epot_vecs)
   free_slist(model->gulp.epot_vecs);
