@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,7 +20,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 The GNU GPL can also be found at http://www.gnu.org
 */
 
-enum {DHKL, EQUIL_UN, EQUIL_RE, GROWTH_UN, GROWTH_RE};
+enum {DHKL, EQUIL_UN, EQUIL_RE, GROWTH_UN, GROWTH_RE, MORPH_BBPA};
 
 /**************/
 /* structures */
@@ -31,6 +31,7 @@ struct vertex_pak
 /* coords */
 gdouble x[3];
 gdouble rx[3];
+gdouble n[3];
 /* adjacencies */
 GSList *adj;
 };
@@ -39,14 +40,10 @@ GSList *adj;
 /* prototypes */
 /**************/
 
+gint morph_build(struct model_pak *);
 gint morph_create(gchar *);
-gint ridge_visible(struct vertex_pak *,
-                   struct vertex_pak *,
-                   struct model_pak *);
 gint facet_visible(struct model_pak *, struct plane_pak *);
 gint facet_equiv(struct model_pak *, gint *, gint *);
 
 gint calc_valid_shifts(struct model_pak *, struct plane_pak *);
 
-GSList *get_facet_equiv(struct model_pak *, gint *);
-
