@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
  
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@ The GNU GPL can also be found at http://www.gnu.org
 
 #include <math.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <sys/time.h>
 
 #include "gdis.h"
@@ -42,7 +43,7 @@ gint sqrt_method=0;
 /**********/
 gulong mytimer(void)
 {
-glong usec;
+gulong usec;
 struct timeval tv;
 
 #ifndef __WIN32
@@ -143,6 +144,30 @@ gdouble tbl_cos(gdouble angle)
 return(tbl_sin(angle + PI/2.0));
 }
 
+/*******************************************/
+/* get angle made with x axis of 2D vector */
+/*******************************************/
+gdouble angle_x_compute(gdouble x, gdouble y)
+{
+gdouble angle;
+
+if (x == 0.0)
+  x = G_MINDOUBLE;
+if (fabs(y) <= 0.0001)
+  y = G_MINDOUBLE;
+
+angle = atan(y/x);
+
+if (y >= 0.0 && x < 0.0)
+  angle += G_PI;
+else if (y < 0.0 && x < 0.0)
+  angle += G_PI;
+else if (y < 0.0 && x > 0.0)
+  angle += 2.0*G_PI;
+
+return(angle);
+}
+
 /***************************/
 /* constrain angle [0,2pi] */
 /***************************/
@@ -176,13 +201,13 @@ if (*angle < 0.0)
 /********************/
 /* SQRT replacement */
 /********************/
-/* NB: s [0.1, 1.0] */
 gdouble fast_sqrt(gdouble s)
 {
 gdouble ds, r;
 
 if (sqrt_method)
   {
+/* NB: applicable range [0.1, 1.0] */
   r = 0.188030699 + 1.48359853*s - 1.0979059*s*s + 0.430357353*s*s*s;
   do
     {
@@ -215,6 +240,20 @@ init_trig();
 init_sqrt();
 }
 
+/***************************************************/
+/* convert a floating point to the nearest integer */
+/***************************************************/
+gint nearest_int(gdouble x)
+{
+gint i;
+
+if (x > 0.0)
+  i = (x+0.5);
+else
+  i = (x-0.5);
+return(i);
+}
+
 /********************/
 /* rounding routine */
 /********************/
@@ -225,11 +264,9 @@ gdouble y, f;
 
 f = pow(10.0, dp);
 y = x * f;
-y += 0.5;
-i = y;
+i = nearest_int(y);
 y = i;
 y /= f;
-
 return(y);
 }
 
@@ -254,6 +291,17 @@ for (j=0 ; j<=5 ; j++)
 return -tmp+log(2.50662827465*ser);
 }
 
+/*******/
+/* gcd */
+/*******/
+gint gcd(gint p, gint q)
+{
+if (q == 0)
+  return(abs(p));
+else
+  return(gcd(q, p%q));
+}
+
 /*******/
 /* gcf */
 /*******/
