@@ -39,6 +39,7 @@ gint shadowless;
 gint fog;
 gint axes;
 gint atype;
+gint show_energy;
 gboolean wire_model;
 gboolean wire_surface;
 gboolean wire_show_hidden;
@@ -266,6 +267,10 @@ GSList *task_list;
 GtkWidget *task_label;
 gint max_threads;
 GThreadPool *thread_pool;
+
+/* CURRENT - task successor (remote job execution) */
+GSList *host_list;
+
 };
 
 /********************************/
@@ -448,6 +453,7 @@ gint eigen;
 gint compare;
 gint nosym;
 gint fix;
+gint noautobond;
 gint cycles;
 gint ensemble;
 gint coulomb;
@@ -473,10 +479,12 @@ gdouble sdipole;
 gdouble qsum;
 gdouble gnorm;
 /* dynamics info */
+/*
 gdouble frame_time;
 gdouble frame_ke;
 gdouble frame_pe;
 gdouble frame_temp;
+*/
 /* HACK - store original structure so we can recall after reading trajectory */
 gint orig_fractional;
 gint orig_construct_pbc;
@@ -994,7 +1002,6 @@ struct model_pak
 {
 /* main info */
 gint id;
-gint number;
 gboolean locked;         /* prevent deletion (eg by dependent child process) */
 gboolean protein;        /* cope with ugly PDB element labelling */
 gint mode;
@@ -1036,9 +1043,7 @@ gdouble zoom;
 /* display flags for the drawing area */
 gint show_names;
 gint show_title;
-gint show_hbonds;
 gint show_frame_number;
-gint show_energy;
 gint show_charge;
 gint show_atom_charges;
 gint show_atom_labels;
@@ -1046,12 +1051,21 @@ gint show_atom_types;
 gint show_atom_index;
 gint show_geom_labels;
 gint show_cores;
+gint show_bonds;
+gint show_hbonds;
+gint show_links;
 gint show_shells;
 gint show_axes;
 gint show_cell;
 gint show_cell_images;
 gint show_cell_lengths;
 gint show_waypoints;
+gint show_selection_labels;
+
+/*VZ*/
+gint show_nmr_shifts;
+gint show_nmr_csa;
+gint show_nmr_efg;
 
 /* ghosts */
 gint num_atoms;
@@ -1071,6 +1085,7 @@ GSList *images;
 GSList *selection;
 GSList *unique_atom_list;
 GSList *elements;
+GSList *ff_list;
 GSList *measure_list;
 GSList *layer_list;
 GSList *graph_list;
@@ -1078,6 +1093,7 @@ GSList *picture_list;
 GSList *transform_list;
 GSList *frame_data_list;
 GSList *waypoint_list;
+GSList *undo_list;
 gpointer graph_active;
 gpointer picture_active;
 gpointer project;
@@ -1103,6 +1119,7 @@ GtkWidget *phonon_slider;
 gboolean show_eigenvectors;
 gboolean phonon_movie;
 gchar *phonon_movie_name;
+gchar *phonon_movie_type;
 
 /* electrostatic scale */
 gint epot_autoscale;
@@ -1148,7 +1165,6 @@ gint num_planes;                /* total number of faces */
 gint num_facets;                /* number of present facets in plane list */
 gint sculpt_shift_use;          /* attempt to tailor sculpted morphology to shifts */
 /* morphology */
-GSList *vertices;
 GSList *planes;
 
 /* space group */
@@ -1199,6 +1215,13 @@ gdouble anim_speed;
 gdouble anim_step;
 gpointer analysis;
 GList *frame_list;
+
+/* current frame data */
+gdouble frame_time;
+gdouble frame_ke;
+gdouble frame_pe;
+gdouble frame_temp;
+
 FILE *afp;
 };
 
@@ -1243,37 +1266,6 @@ struct grid_config_pak
     gchar * queueing_system;
 };
 
-/******************/
-/* MDI structures */
-/******************/
-/* TODO - combine box_pak & cand_pak? */
-struct box_pak
-{
-gint component;
-gint x;
-gint y;
-gint z;
-};
-
-struct mdi_pak
-{
-gint box_dim;
-gdouble latt_sep;
-gint num_comp;
-gint *comp_idx;
-gint *comp_req;
-gint *comp_done;
-gint *array;
-};
-
-struct cand_pak
-{
-gint pos;
-gint x;
-gint y;
-gint z;
-};
-
 /*********************************/
 /* widget convenience structures */
 /*********************************/
