@@ -27,6 +27,11 @@ The GNU GPL can also be found at http://www.gnu.org
 #include <stdlib.h>
 #include <ctype.h>
 #include <time.h>
+
+#ifndef __WIN32
+#include <termios.h>
+#endif
+
 #include <unistd.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -46,10 +51,52 @@ The GNU GPL can also be found at http://www.gnu.org
 extern struct sysenv_pak sysenv;
 extern struct elem_pak elements[];
 
+/***********************/
+/* strip off extension */
+/***********************/
+gchar *parse_strip_extension(const gchar *name)
+{
+gint i;
+gchar *base, *temp;
+
+temp = g_strdup(name);
+
+/* get the rightmost '.' */
+for (i=strlen(temp) ; i-- ; )
+  if (*(temp+i) == '.')
+    {
+    base = g_strndup(temp, i);
+    g_free(temp);
+    return(base);
+    }
+
+/* not found - return the whole thing */
+return(temp);
+}
+
+/***************************/
+/* strip newline character */
+/***************************/
+gchar *parse_strip_newline(const gchar *text)
+{
+gint i;
+
+for (i=0 ; i<strlen(text) ; i++)
+  {
+  switch (text[i])
+    {
+    case '\n':
+      break;
+    }
+  }
+
+return(g_strndup(text, i-1));
+}
+
 /*****************************************/
 /* strip off path and extension (if any) */
 /*****************************************/
-gchar *strdup_basename(const gchar *name)
+gchar *parse_strip(const gchar *name)
 {
 gint i;
 gchar *base, *temp;
@@ -185,23 +232,29 @@ g_strstrip(text);
 #define DEBUG_STR_IS_FLOAT 0
 gint str_is_float(const gchar *text)
 {
-gint i;
+gint i, n;
 
 #if DEBUG_STR_IS_FLOAT
 printf("testing [%s] ", text);
 #endif
 
+n=0;
 for (i=0 ; i<strlen(text) ; i++)
   {
   switch(text[i])
     {
 /* exceptions */
-/* TODO - cope with exp notation chars */
     case '+':
     case '-':
     case '.':
       break;
 
+/* cope with exp notation chars */
+    case 'D':
+    case 'E':
+      n++;
+      break;
+
     default:
       if (g_ascii_isdigit(text[i]))
         break;
@@ -210,6 +263,14 @@ printf(" = not a number (contains %c)\n", text[i]);
 #endif
       return(FALSE);
     }
+
+  if (n > 1)
+    {
+#if DEBUG_STR_IS_FLOAT
+printf(" = not a number (contains more than one exponential character)\n");
+#endif
+    return(FALSE);
+    }
   }
 #if DEBUG_STR_IS_FLOAT
 printf(" = a number.\n");
@@ -591,12 +652,8 @@ return(buff);
 /*********************/
 /* tokenize a string */
 /*********************/
-/* replacement routine for copy_items */
-/* aim is to have one call, rather than multiple copy_item calls */
-/* trouble is, g_strsplit doesn't eliminate multiple separators */
-/* return number found -> check if enough in file parsing */
-/* ensure exactly num tokens returned!!! */
-/* strlen = 0 if token is empty */
+/* can be useful if you want a definite number of tokens */
+/* also, doesnt skip # lines (useful for processing GROMACS stuff) */
 #define DEBUG_GET_TOKENS 0
 gchar **get_tokens(gchar *src, gint num)
 {
@@ -672,3 +729,69 @@ for (n=0 ; n<=num ; n++)
 return(src+i);
 }
 
+#ifndef __WIN32
+
+/*****************************************************************/
+/* get a character from the terminal, preventing it being echoed */
+/*****************************************************************/
+gint parse_getchar_hidden(void)
+{
+struct termios old;
+struct termios new;
+gint c;
+
+/* get terminal stdin attributes */
+if (tcgetattr(0, &old) == -1)
+  return(-1);
+
+/* turn off echo */
+new = old;
+new.c_lflag &= ~(ICANON | ECHO);
+new.c_cc[VMIN] = 1;
+/* don't timeout? */
+new.c_cc[VTIME] = 0;
+
+/* change the terminal attributes immediately */
+if (tcsetattr(0, TCSANOW, &new) == -1)
+  return(-1);
+
+/* read in a character */
+c = getchar();
+
+/* restore terminal attributes */
+tcsetattr(0, TCSANOW, &old);
+
+return(c);
+}
+
+/******************************************************************/
+/* Get a line of text, preventing it being echoed to the terminal */
+/******************************************************************/
+gchar *parse_getline_hidden(void)
+{
+gchar c;
+GString *line;
+
+line = g_string_new(NULL);
+
+for (;;)
+  {
+  c = parse_getchar_hidden(); 
+
+/* to display or not to display (more secure not to - but gives no feedback) */
+//  printf("*");
+
+  if (c == '\n')
+    {
+    printf("\n");
+    break;
+    }
+  else
+    g_string_sprintfa(line, "%c", c);
+  }
+
+return(g_string_free(line, FALSE));
+}
+
+#endif
+
