@@ -26,7 +26,6 @@ void capitals(gchar *, gchar *);
 gchar **get_tokens(gchar *, gint);
 gchar *get_token_pos(gchar *, gint);
 gchar **tokenize(const gchar *, gint *);
-gchar *strdup_basename(const gchar *);
 gchar *find_char(const gchar *, gint, gint);
 gint char_count(const gchar *, gchar);
 gdouble str_to_float(const gchar *);
@@ -39,4 +38,9 @@ GSList *get_keywords_anywhere(gchar *);
 void strip_extra(gchar *);
 gchar *parse_extension_set(const gchar *, const gchar *);
 void parse_space_replace(gchar *, gchar);
+gchar *parse_strip_extension(const gchar *);
+gchar *parse_strip_newline(const gchar *);
+gchar *parse_strip(const gchar *);
+
+gchar *parse_getline_hidden(void);
 
