@@ -380,7 +380,7 @@ g_assert_not_reached();
 printf("set dimension: %d x %d, scaling: %f x %f\n", max[0], max[1], scale[0], scale[1]);
 */
 
-    basename = strdup_basename(*buff);
+    basename = parse_strip(*buff);
     fullname =  g_strdup_printf("%s%s%s.got", sysenv.cwd, DIR_SEP, basename);
     fail = 0;
 
