@@ -130,6 +130,10 @@ printf("-------------------------------------------------------------\n");
 for (i=PIPE_DEPTH ; i-- ; )
   pipes[i] = NULL;
 
+/* bond display turned off? */
+if (!model->show_bonds)
+  return;
+
 /* common pipe width */
 radius = sysenv.render.stick_rad;
 
