@@ -11,7 +11,7 @@
 /* unified render (povray & openGL) */
 enum 
 {
-STICK, BALL_STICK, CPK, LIQUORICE, POLYHEDRAL,
+STICK, BALL_STICK, CPK, LIQUORICE, POLYHEDRAL, ZONE,
 LIGHT_TYPE, DIRECTIONAL, POSITIONAL,
 SCALE, HALO_QUALITY, FAST_ROTATION,
 ANTIALIAS, SHADOWLESS, DRAW_AXES, PERSPECTIVE, WIRE_FRAME, FOG,
@@ -19,9 +19,24 @@ ANIMATE, ANIMATE_TYPE, ANIM_GIF, ANIM_MPEG, ANIM_NAME, ANIM_FRAME,
 MORPH_STYLE, MORPH_FINISH
 };
 
-/*********************/
-/* POVray structures */
-/*********************/
+/************************/
+/* rendering structures */
+/************************/
+
+/* TODO - move model to this & replace model->scale with zoom */
+struct camera_pak
+{
+gint mode;         /* free/locked */
+gint perspective;  /* true/false */
+gdouble fov;       /* perspective field of view */
+gdouble zoom;      /* zoom/scale factor */
+gdouble x[3];      /* position */
+gdouble o[3];      /* orientation vector */
+gdouble v[3];      /* viewing vector */
+gdouble e[3];      /* crossproduct of viewing and orientation */
+gdouble q[4];      /* quaternion modifier */
+};
+
 struct light_pak
 {
 gint type;
@@ -53,12 +68,13 @@ gchar morph_finish[LINELEN];
 };
 
 /* prototypes */
-GSList *render_get_pipes(gint, gint, gint, struct model_pak *);
-
 void render_make_pipes(GSList **, struct model_pak *);
-
+GSList *render_get_pipes(gint, gint, gint, struct model_pak *);
 GSList *render_sort_pipes(GSList *);
 
 void create_polyhedra(struct model_pak *);
 void destroy_polyhedra(struct model_pak *);
 
+void povray_task(void);
+void povray_exec(gchar *);
+
