@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -30,5 +30,5 @@ gchar *scan_get_line(gpointer);
 gchar *scan_cur_line(gpointer);
 gchar **scan_get_tokens(gpointer, gint *);
 gboolean scan_put_line(gpointer);
-
+gpointer scan_offset_get(gpointer); 
 void scan_frame_new(gpointer, struct model_pak *);
