@@ -26,6 +26,7 @@ The GNU GPL can also be found at http://www.gnu.org
 
 #include "gdis.h"
 #include "coords.h"
+#include "edit.h"
 #include "matrix.h"
 #include "quaternion.h"
 #include "measure.h"
@@ -35,7 +36,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "select.h"
 #include "zone.h"
 #include "interface.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 
 extern struct sysenv_pak sysenv;
 extern struct elem_pak elements[];
@@ -109,24 +110,6 @@ redraw_canvas(SINGLE);
 /***********************************/
 /* change selection rendering mode */
 /***********************************/
-/* NB: mode = -1 means atom will follow the global rendering mode */
-/*
-void set_selection_rendering(gint mode, struct model_pak *model)
-void render_mode_set(gint mode, struct model_pak *model)
-{
-GSList *list;
-struct core_pak *core;
-
-for (list=model->selection ; list ; list=g_slist_next(list))
-  {
-  core = (struct core_pak *) list->data;
-  core->render_mode = mode;
-  }
-}
-*/
-
-/* TODO - same for shells */
-
 void core_render_mode_set(gint mode, GSList *cores)
 {
 GSList *list;
@@ -273,9 +256,11 @@ redraw_canvas(SINGLE);
 /********************/
 void select_colour(void)
 {
+#ifdef WITH_GUI
 /* no action if nothing is loaded */
 if (sysenv.active_model)
   new_csd("Selection colour", (gpointer) cb_select_colour);
+#endif
 }
 
 /********************/
@@ -298,9 +283,18 @@ while (list)
   {
   core = list->data;
   list = g_slist_next(list);
+/*
   delete_core(core);
+*/
+
+/* NEW */
+  core_delete_single(core, data);
+
   }
+
+/*
 delete_commit(data);
+*/
 
 data->selection = NULL;
 sysenv.select_source = NULL;
@@ -638,14 +632,22 @@ switch (sysenv.select_mode)
 #define DEBUG_ROTATE_SELECT 0
 void rotate_select(struct model_pak *data, gdouble *mat)
 {
+gint n;
 gdouble scent[3], vec[3];
 GSList *list;
 struct core_pak *core;
 struct shel_pak *shell;
 
-if (!g_slist_length(data->selection))
+g_assert(data != NULL);
+
+n = g_slist_length(data->selection);
+if (!n)
   return;
 
+#if DEBUG_ROTATE_SELECT
+P3MAT("rot: ", mat);
+#endif
+
 /* calculate selection's centroid */
 VEC3SET(scent, 0.0, 0.0, 0.0);
 for (list=data->selection ; list ; list=g_slist_next(list))
@@ -653,7 +655,7 @@ for (list=data->selection ; list ; list=g_slist_next(list))
   core = list->data;
   ARR3ADD(scent, core->x);
   }
-VEC3MUL(scent, 1.0/g_slist_length(data->selection));
+VEC3MUL(scent, 1.0 / (gdouble) n);
 
 /* FIXME - why is this different for INITIAL orientation */
 #if DEBUG_ROTATE_SELECT
