@@ -1,7 +1,7 @@
 
 void set_selection_mode(GtkWidget *, gpointer *);
 
-void select_clear(void);
+void select_clear(struct model_pak *);
 void select_copy(void);
 void select_paste(void);
 
@@ -11,8 +11,10 @@ void select_hide(void);
 void select_invert(void);
 void select_all(void);
 
-void set_selection_ghosts(void);
-void set_selection_normal(void);
+void select_flag_ghost(void);
+void select_flag_normal(void);
+
+void select_core(struct core_pak *, gint, struct model_pak *);
 
 gint select_add_core(struct core_pak *, struct model_pak *);
 void select_add_mol(struct mol_pak *, struct model_pak *);
@@ -20,6 +22,7 @@ void select_add_fragment(struct core_pak *, struct model_pak *);
 void select_add_region(struct core_pak *, struct model_pak *);
 void select_del_core(struct core_pak *, struct model_pak *);
 void select_all_labels(struct core_pak *, struct model_pak *);
+void select_all_types(struct core_pak *, struct model_pak *);
 void select_all_elem(gint, struct model_pak *);
 void select_all_elem_in_mol(struct core_pak *, struct model_pak *);
 void select_model_labels(void);
@@ -28,7 +31,7 @@ void core_render_mode_set(gint, GSList *);
 void core_render_wire_set(gint, GSList *);
 
 void rotate_select(struct model_pak *, gdouble *);
-void xlat_select(gdouble *, struct model_pak *);
+void select_translate(gint, gint, struct model_pak *);
 
 void construct_backbone(struct core_pak *, struct model_pak *);
 void construct_ribbon(GList *, struct model_pak *);
