@@ -1473,7 +1473,7 @@ static void SimplifyFraction(int nume, int deno, int *o_nume, int *o_deno)
 const char *FormatFraction(int nume, int deno, int Decimal,
                            char *Buffer, int SizeBuffer)
 {
-  int          n, d;
+  int          n=0, d=0;
   char         *cp, *cpp;
   static char  StaticBuffer[40];
 
