@@ -396,7 +396,11 @@ if (data->sginfo.spacenum < 1)
 
 /* crystal lattice type */
 data->sginfo.lattice = SgInfo.XtalSystem;
-switch(SgInfo.XtalSystem)
+
+if (data->sginfo.latticename)
+  g_free(data->sginfo.latticename);
+
+switch (SgInfo.XtalSystem)
   {
   case XS_CUBIC:
     data->sginfo.latticename = g_strdup("Cubic");
