@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,13 +20,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 The GNU GPL can also be found at http://www.gnu.org
 */
 
-gint space_lookup(struct model_pak *);
+void space_init(gpointer);
 void space_free(gpointer);
+gint space_lookup(struct model_pak *);
 
 void space_make_p1(struct model_pak *);
 void space_make_images(gint, struct model_pak *);
 void space_make_supercell(struct model_pak *);
 
+gint space_primitive_cell(struct model_pak *);
+
 void space_image_widget_setup(GtkWidget *);
 void space_image_widget_redraw(void);
 void space_image_widget_reset(void);
