@@ -34,7 +34,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "spatial.h"
 #include "select.h"
 #include "interface.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #ifdef __APPLE__
 #include <OpenGL/gl.h>
 #else
