@@ -33,6 +33,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "gdis.h"
 #include "coords.h"
 #include "model.h"
+#include "edit.h"
 #include "matrix.h"
 #include "numeric.h"
 #include "vector.h"
@@ -1389,6 +1390,126 @@ printf("\n");
 return(0);
 }
 
+/****************************************/
+/* construct and initialize the surface */
+/****************************************/
+#define MAKE_SURFACE_DEBUG 0
+gpointer make_surface(struct model_pak *data,
+                      struct plane_pak *pdata,
+                      struct shift_pak *sdata)
+{
+gint bmode, r1size;
+gdouble sbe;
+GSList *list;
+struct model_pak *surf;
+struct core_pak *core;
+
+/* checks */
+g_assert(data != NULL);
+if (data->periodic != 3)
+  {
+  gui_text_show(ERROR, "base model is not 3D periodic.\n");
+  return(NULL);
+  }
+if (!pdata->index[0] && !pdata->index[1] && !pdata->index[2])
+  {
+  gui_text_show(ERROR, "Don't be silly.\n");
+  return(NULL);
+  }
+
+#if MAKE_SURFACE_DEBUG
+printf("Generating surface for model: %d\n", source);
+#endif
+
+/* ignore mols when building surface? */
+bmode = data->build_molecules;
+if (data->surface.ignore_bonding)
+  {
+  data->build_molecules = FALSE;
+  connect_bonds(data);
+  connect_molecules(data);
+  }
+
+/* allocate & init for surface data */
+surf = model_new();
+
+/* NEW - label it as MARVIN, so it's build mode follows the */
+/* source model, rather than the GULP setup data - see model_prep() */
+surf->id = MARVIN;
+
+/* transfer appropriate GULP setup data to new model */
+gulp_data_copy(data, surf);
+
+surf->gulp.run = E_SINGLE;
+surf->gulp.method = CONV;
+
+/* transfer surface data to new model */
+/* NB: memcpy would produce undesired effects if pointers were */
+/* later added to the surface struct */
+ARR3SET(surf->surface.miller, pdata->index);
+surf->surface.shift = sdata->shift;
+surf->surface.region[0] = sdata->region[0];
+surf->surface.region[1] = sdata->region[1];
+/* setup for region display */
+if (surf->surface.region[0])
+  surf->region_empty[REGION1A] = FALSE;
+if (surf->surface.region[1])
+  surf->region_empty[REGION2A] = FALSE;
+
+surf->build_molecules = bmode;
+
+#if MAKE_SURFACE_DEBUG
+printf("Miller (%d,%d,%d)\n",surf->surface.miller[0]
+                            ,surf->surface.miller[1]
+                            ,surf->surface.miller[2]);
+printf("Shift %f\n",surf->surface.shift);
+printf("Region sizes (%d,%d)\n",surf->surface.region[0]
+                               ,surf->surface.region[1]);
+#endif
+
+/* TODO - valid shift/region size loop??? */
+generate_surface(data, surf);
+gulp_files_init(surf);
+
+/*
+coords_init(INIT_COORDS, surf);
+*/
+
+/* restore connectivity in source model */
+if (data->surface.ignore_bonding)
+  {
+  data->build_molecules = bmode;
+  connect_bonds(data);
+  connect_molecules(data);
+  }
+
+/* calculate the bulk energy needed for GULP surface calcs */
+sbe = data->gulp.energy;
+if (fabs(sbe) < FRACTION_TOLERANCE)
+  {
+  gui_text_show(WARNING, "Suspicious total energy. Has it been properly calculated?\n");
+  }
+
+/* calc the number of region 1 atoms */
+r1size=0;
+for (list=surf->cores ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+  if (core->region == REGION1A)
+    r1size++;
+  }
+
+sbe /= data->num_atoms;    /* E per atom in unit cell */
+sbe *= r1size;             /* E scaled to region 1 size */
+surf->gulp.sbulkenergy = sbe;
+
+/* employ src file? */
+tree_model_add(surf);
+tree_select_model(surf);
+
+return(surf);
+}
+
 /*************************************/
 /* molecule centroid z value sorting */
 /*************************************/
@@ -1402,6 +1523,95 @@ if (m1->centroid[2] > m2->centroid[2])
 return(1);
 }
 
+/********************************************************/
+/* scan a plane's shift list for the best energy values */
+/********************************************************/
+#define DEBUG_UPDATE_PLANE_ENERGY 0
+void update_plane_energy(struct plane_pak *plane, struct model_pak *model)
+{
+gint i;
+GSList *list;
+struct plane_pak *plane2;
+struct shift_pak *sdata;
+
+/* checks */
+g_assert(model != NULL);
+g_assert(plane != NULL);
+
+#if DEBUG_UPDATE_PLANE_ENERGY
+printf("(%d %d %d)\n", plane->index[0], plane->index[1], plane->index[2]);
+#endif
+
+/* set best values to those of the 1st shift */
+list = plane->shifts;
+if (list)
+  {
+  sdata = list->data;
+
+/* init the plane's best values */
+  for (i=0 ; i<2 ; i++)
+    {
+    plane->esurf[i] = sdata->esurf[i];
+    plane->esurf_shift = sdata->shift;
+    plane->eatt[i] = sdata->eatt[i];
+    plane->eatt_shift = sdata->shift;
+    plane->bbpa = sdata->bbpa;
+    plane->bbpa_shift = sdata->shift;
+    }
+  }
+else
+  return;
+
+/* search the shift list for better values */
+while (list != NULL)
+  {
+  sdata = list->data;
+  for (i=0 ; i<2 ; i++)
+    {
+/* surface energy */
+    if (sdata->esurf[i] < plane->esurf[i])
+      {
+      plane->esurf[i] = sdata->esurf[i];
+      plane->esurf_shift = sdata->shift;
+      }
+/* attachment energy */
+    if (sdata->eatt[i] > plane->eatt[i])
+      {
+      plane->eatt[i] = sdata->eatt[i];
+      plane->eatt_shift = sdata->shift;
+      }
+    }
+/* broken bonds */
+  if (sdata->bbpa < plane->bbpa)
+    {
+    plane->bbpa = sdata->bbpa;
+    plane->bbpa_shift = sdata->shift;
+    }
+  list = g_slist_next(list);
+  }
+
+#if DEBUG_UPDATE_PLANE_ENERGY
+printf(" %f\n", plane->bbpa);
+#endif
+
+/* update symmetry related planes */
+if (plane->primary)
+  {
+  for (list=model->planes ; list ; list=g_slist_next(list))
+    {
+    plane2 = list->data;
+
+    if (plane2->parent == plane)
+      {
+      plane2->esurf[0] = plane->esurf[0];
+      plane2->esurf[1] = plane->esurf[1];
+      plane2->eatt[0] = plane->eatt[0];
+      plane2->eatt[1] = plane->eatt[1];
+      }
+    }
+  }
+}
+
 /*******************************************************************/
 /* peturb a planar surface in order to test alternate terminations */
 /*******************************************************************/
