@@ -20,9 +20,10 @@ ALL_SHIFTS, ALL_PLANES, INVALID_SHIFTS
 /* 2D vdw surface parameter specifiers */
 enum 
 {
-MS_ACCESSIBLE, MS_MOLECULAR,
+MS_ACCESSIBLE, MS_MOLECULAR, MS_EDEN,
 MS_DSIZE, MS_DEPTH, MS_OFFSET, MS_PRAD, MS_ACCURACY,
-MS_TOUCH, MS_AFM, MS_EPOT, MS_HIRSHFELD
+MS_TOUCH, MS_AFM, MS_EPOT, MS_HIRSHFELD, MS_SOLVENT,
+MS_DE, MS_CURVEDNESS, MS_SHAPE_INDEX, MS_SSATOMS
 };
 
 /* surface specifiers */
@@ -42,6 +43,8 @@ GSList *cores;
 };
 
 /* prototypes */
+gint generate_surface(struct model_pak *, struct model_pak *);
+
 gpointer plane_new(gdouble *, struct model_pak *);
 gpointer shift_new(gdouble);
 void plane_free(gpointer);
@@ -58,6 +61,9 @@ gint calc_shifts(void);
 void calc_emp(struct model_pak *);
 gint region_max(struct model_pak *);
 gint surf_sysabs(struct model_pak *, gint, gint, gint);
+void surf_symmetry_generate(struct model_pak *);
+
+GSList *get_facet_equiv(struct model_pak *, gint *);
 
 void update_surface_dialog(struct model_pak *);
 
@@ -77,3 +83,15 @@ gint dhkl_compare(gpointer, gpointer);
 
 void free_vertices(struct model_pak *);
 
+void morph_sculpt(GtkWidget *, gpointer);
+
+gpointer plane_dup(struct plane_pak *);
+
+void surf_shift_explore(struct model_pak *, struct surface_pak *);
+
+gint region_move_atom(struct core_pak *, gint, struct model_pak *);
+
+gpointer make_surface(struct model_pak *,
+                      struct plane_pak *,
+                      struct shift_pak *);
+
