@@ -1,6 +1,7 @@
+#include <time.h>
 
 /* task modes */
-enum {RUNNING, QUEUED, COMPLETED, KILLED};
+enum {RUNNING, QUEUED, KILLED, COMPLETED, REMOVED};
 
 /************/
 /* task pak */
@@ -21,11 +22,18 @@ gint hour;
 gchar *label;
 gchar *message;
 
+gchar *status_file;
+FILE *status_fp;
+gint status_index;
+GString *status_text;
+/* JJM DEBUG
 #ifdef __WIN32
 gpointer start_time;
 #else
 time_t start_time;
 #endif
+*/
+time_t start_time;
 
 gchar *time;
 gdouble pcpu;
@@ -36,35 +44,25 @@ gpointer locked_model;
 
 /* main task and arguments (run by the child) */
 void (*primary)(gpointer, ...);
-gpointer *ptr1;
+gpointer ptr1;
 /* cleanup task and arguments (run by the parent) */
 void (*cleanup) (gpointer, ...);
-gpointer *ptr2;
+gpointer ptr2;
 };
 
 /* task control */
-gint init_task_thread(void);
-gint update_tasks(void);
 gint update_task_info(void);
-
-void submit_task(const gchar *, gpointer, gpointer, gpointer, gpointer, gpointer);
-
+void task_status_update(struct task_pak *);
 void task_dialog(void);
-
 gint task_sync(const gchar *);
 
-gint exec_gulp(const gchar *, const gchar *);
-
-/* work functions */
-void exec_gulp_task(gpointer *);
-void proc_gulp_task(gpointer *);
-void exec_pov_task(gpointer *);
-void exec_img_task(gpointer *);
-
 void task_queue_init(void);
 void task_queue_free(void);
+
+void task_free(gpointer);
 void task_new(const gchar *,
               gpointer, gpointer,
               gpointer, gpointer,
-              gpointer );
+              gpointer);
 
+gint exec_gulp(const gchar *, const gchar *);
