@@ -93,6 +93,25 @@ free_slist(type->rules);
 g_free(type);
 }
 
+/*************************************************/
+/* search for undefined types in a list of cores */
+/*************************************************/
+gint type_check_list(GSList *cores)
+{
+gint n=0;
+struct core_pak *core;
+GSList *list;
+
+for (list=cores ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+/* NULL or '?' */
+  if (!core->atom_type)
+    n++;
+  }
+return(n);
+}
+
 /***********************************/
 /* configure forcefield assignment */
 /***********************************/
@@ -576,6 +595,7 @@ void type_dreiding_gasteiger(struct model_pak *model, gint mode)
   }
   
   /* define temporary files */
+/*
 #ifdef __WIN32
   temp_in_file = "C:\\babel_in_temp.xyz";
   temp_out_file = "C:\\babel_out_temp.bgf";
@@ -583,16 +603,24 @@ void type_dreiding_gasteiger(struct model_pak *model, gint mode)
   temp_in_file = "/tmp/babel_in_temp.xyz";
   temp_out_file = "/tmp/babel_out_temp.bgf";
 #endif
-  
-  /* write an xyz file of the model being typed */
   write_xyz(temp_in_file, model);
-  
-  /* create and execute the command using babel */
   cmd = g_strdup_printf("%s -ixyz %s -obgf %s", sysenv.babel_path, temp_in_file, temp_out_file);
-  
-#if 1
-  g_message("executing %s", cmd);
+*/
+
+
+/* NEW - using PDB as temp file, since it has connect data */
+/* which babel may not generate correctly (at all?) for XYZ */
+#ifdef __WIN32
+  temp_in_file = "C:\\babel_in_temp.pdb";
+  temp_out_file = "C:\\babel_out_temp.bgf";
+#else
+  temp_in_file = "/tmp/babel_in_temp.pdb";
+  temp_out_file = "/tmp/babel_out_temp.bgf";
 #endif
+
+  write_pdb(temp_in_file, model);
+  cmd = g_strdup_printf("%s -ipdb %s -obgf %s", sysenv.babel_path, temp_in_file, temp_out_file);
+
   
   system(cmd);
   g_free(cmd);
@@ -621,9 +649,13 @@ void type_dreiding_gasteiger(struct model_pak *model, gint mode)
     if (mode == DREIDING)
     {
       g_free(core->atom_type); 
-      g_free(core->atom_label);
       core->atom_type = g_strdup(typed_core->atom_type);
+
+/*
+      g_free(core->atom_label);
       core->atom_label = g_strdup(typed_core->atom_label);
+*/
+
       if (model->gulp.species)
         g_free(model->gulp.species);
       model->gulp.species = g_strdup(temp_model.gulp.species);
@@ -638,10 +670,8 @@ void type_dreiding_gasteiger(struct model_pak *model, gint mode)
   
   unlink(temp_in_file);
   unlink(temp_out_file);
-  
 }
 
-
 /*********************************/
 /* type model according to label */
 /*********************************/
