@@ -34,7 +34,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "gdis.h"
 #include "coords.h"
 #include "interface.h"
-#include "shortcuts.h"
+#include "gui_shorts.h"
 #include "matrix.h"
 #include "spatial.h"
 #include "numeric.h"
